#include "Aggregate.h"
#include "CylinderElastin.h"
#include "utils.h"
#include "PointBag2D.hpp"
#include "Vector2D.hpp"
#include "CircleGridMap.h"

#include <cmath>
#include <iostream>
//#include <fstream>
#include <set>
#include <list>
#include <stack>
#include <exception>
#include <algorithm>
#include <functional>
#include <limits>
#include <string>
#include <sstream>


using namespace std;

//
//Functor for find_if searching for an element with a given id in the aggregate 
//
class ElementIdEqual : public binary_function<DLAElement*, int, bool>{
public:
	bool operator()(DLAElement* p, int id) const{
		return (p->getElementId()==id);
	}
};

//
//Function for sorting elements based on x coordinate
//
class XCompFunctor : public binary_function<DLAElement*, DLAElement*, bool>{
public:
	bool operator()(DLAElement* p1, DLAElement* p2){
		return (p1->getX() < p2->getX());
	}
};

//
//Function for sorting elements based on y coordinate
//
class YCompFunctor : public binary_function<DLAElement*, DLAElement*, bool>{
public:
	bool operator()(DLAElement* p1, DLAElement* p2){
		return (p1->getY() < p2->getY());
	}
};

//bool yFunc(DLAElement* p1, DLAElement* p2){
//	return p1->getY() < p2->getY();
//}

Aggregate::Aggregate()  
{
  context = SimuContext::getInstance();
  maxX = context->getElementWidth()/2.0f;
  minX = -context->getElementWidth()/2.0f;
  maxY = context->getElementHeight()/2.0f;
  minY = -(context->getElementHeight()/2.0f);
  maxZ = (context->getElementDepth()/2.0f);
  minZ = -(context->getElementDepth()/2.0f);	
}

Aggregate::~Aggregate(){
  list<DLAElement*>::iterator itr;
  itr = dlaElements.begin();
  while(itr!=dlaElements.end()){
    delete (*itr);
    itr++;
  } 
  dlaElements.clear(); 
}

SimuContext* Aggregate::getContext(){
  return context;
}

void Aggregate::addElement(DLAElement *p){
  dlaElements.push_front(p);
  
  mapById[p->getElementId()] = p;
  
  float mapIndex = ceil(p->getX()/context->getElementWidth());

  if(elementMap.find(mapIndex)==elementMap.end()){
    list<DLAElement *> pList;
    pList.push_front(p);
    elementMap[mapIndex] = pList;
  }
  else{
    elementMap[mapIndex].push_front(p);
  }

  if(p->getX()-context->getElementWidth()/2.0f < minX){
    minX = p->getX()-context->getElementWidth()/2.0f;
  }
  if(p->getX()+context->getElementWidth()/2.0f > maxX){
    maxX = p->getX()+context->getElementWidth()/2.0f;
  }
  if(p->getY()-context->getElementHeight()/2.0f < minY){
    minY = p->getY()-context->getElementHeight()/2.0f;
  }
  if(p->getY()+context->getElementHeight()/2.0f > maxY){
    maxY = p->getY()+context->getElementHeight()/2.0f;
  }

  if(p->getZ()-context->getElementDepth()/2.0f < minZ){
    minZ = p->getZ()-context->getElementDepth()/2.0f;
  }
  if(p->getZ()+context->getElementDepth()/2.0f > maxZ){
    maxZ = p->getZ()+context->getElementDepth()/2.0f;
  }
  
  width  = maxX - minX;
  height = maxY - minY;
  depth  = maxZ - maxY; 
  numberOfElements++;
}

DLAElement*& Aggregate::operator[](int eId){
	if(eId < 0 || eId >= numberOfElements){		
		ostringstream errInfo;
		errInfo << "Access error, the element id " << eId;
		errInfo << " is smaller than 0 or bigger than the number of particles in the aggregate !\n";
		throw(runtime_error(errInfo.str()));
	}
	return mapById[eId];
}

//const DLAElement*& Aggregate::operator[](int eId) const{
//	if(eId < 0 || eId >= numberOfElements){
//		ostringstream errInfo;
//		errInfo << "Access error, the element id " << eId;
//		errInfo << " is smaller than 0 or bigger than the number of particles in the aggregate !\n";
//		throw(runtime_error(errInfo.str()));		
//	}	
//	return mapById[eId];
//}

void  Aggregate::save(ostream& outFile){
  outFile << "<aggregate>" << endl;
  outFile << "<seconds-spent>" << this->getSecondsSpent() << "</seconds-spent>\n";
  outFile << "<minX>" << minX << "</minX>\n"; 
  outFile << "<maxX>" << maxX << "</maxX>\n";
  outFile << "<minY>" << minY << "</minY>\n";
  outFile << "<maxY>" << maxY << "</maxY>\n";
  outFile << "<minZ>" << minZ << "</minZ>\n";
  outFile << "<maxZ>" << maxZ << "</maxZ>\n";
  list<DLAElement*>::iterator itr;
  itr = dlaElements.begin();
  while(itr!=dlaElements.end()){
    (*itr)->save(outFile);
    itr++;
  }
  outFile << "</aggregate>" << endl;
}

int Aggregate::getNumberOfElements() const{
  return dlaElements.size();
}

const list<DLAElement*>* Aggregate::getElements() const{
  return &dlaElements;
}
list<DLAElement*>* Aggregate::getElements(){
  return &dlaElements;
}


map<float, list<DLAElement* > >* Aggregate::getElementMap(){
  return &elementMap;
}

float Aggregate::getMinX(){
  return minX;
}
void Aggregate::setMinX(float value){
	minX = value;
}

float Aggregate::getMaxX(){
  return maxX;
}
void Aggregate::setMaxX(float value){
	maxX = value;
}

float Aggregate::getMinY(){
  return minY;
}
void Aggregate::setMinY(float value){
	minY = value;
}

float Aggregate::getMaxY(){
  return maxY;
}
void Aggregate::setMaxY(float value){
	maxY = value;
}


float Aggregate::getMinZ(){
  return minZ;
}
void Aggregate::setMinZ(float value){
	minZ = value;
}

float Aggregate::getMaxZ(){
  return maxZ;
}
void Aggregate::setMaxZ(float value){
	maxZ = value;
}

float Aggregate::getWidth(){
  return width;
}

float Aggregate::getHeight(){
  return height;
}

float Aggregate::getDepth(){
  return depth;
}

/** 
 * Let the coming element circle around the existed particle to find a position,
 * where the coming element's center distance to the coordinate center is minimum.
 * @param existed Pointer to the existed element
 * @param coming  Pointer to the coming particle
 * @return None 
 */
 
void Aggregate::diffuse(DLAElement* existed, DLAElement* coming){
	float lAdjustAngle, lMinDistance; //=std::numeric_limits<float>::max();
	float rAdjustAngle, rMinDistance; //=std::numeric_limits<float>::max();
	bool  lAdjustable=false;
	bool  rAdjustable=false;
    list<DLAElement*> lCollideTarget;
    list<DLAElement*> rCollideTarget;
    bool lChoice = false;
    bool rChoice = false;
	
	float initAngle = existed->getYZAngle(coming);
	float stepSize = context->getDiffusionStep();
	//lAdjustable = existed->isAdjustable(coming, stepSize, lAdjustAngle, lMinDistance, lOccupy);
	//rAdjustable = existed->isAdjustable(coming, -stepSize, rAdjustAngle, rMinDistance, rOccupy);
	lAdjustable = adjustTest(existed, coming, stepSize, lAdjustAngle, lMinDistance, lCollideTarget);
	rAdjustable = adjustTest(existed, coming, -stepSize, rAdjustAngle, rMinDistance, rCollideTarget);	
		
	//cout << "left: " << lAdjustable << " - " << lAdjustAngle << " - " << lMinDistance << endl;
	//cout << "right: " << rAdjustable << " - " << rAdjustAngle << " - " << rMinDistance << endl;	
	
	float newAngle, newY, newZ;
	float elementHeight = context->getElementHeight();
	if(lAdjustable && lMinDistance<=rMinDistance){
		newAngle = normalAngle(initAngle+lAdjustAngle);
		newY = existed->getY() + elementHeight*sin(newAngle);
		newZ = existed->getZ() + elementHeight*cos(newAngle);
		coming->setPos(coming->getX(), newY, newZ);
		lChoice = true;
	}
	else if(rAdjustable && rMinDistance<lMinDistance){
		newAngle = normalAngle(initAngle-rAdjustAngle);
		newY = existed->getY() + elementHeight*sin(newAngle);
		newZ = existed->getZ() + elementHeight*cos(newAngle);		
		coming->setPos(coming->getX(), newY, newZ);
	  rChoice = true;
	}

  float cAngle = existed->getYZAngle(coming);
  existed->addNeighbour(cAngle, coming);
  float tAngle = coming->getYZAngle(existed);
  coming->addNeighbour(tAngle, existed);
  
  DLAElement* pCollide;
  
  if(lChoice && lCollideTarget.size()>0){
  	if(lCollideTarget.size()>1){
  	  pCollide = getUniqueTarget(coming, lCollideTarget);
  	}
  	else{
  		pCollide = lCollideTarget.front();
  	}
 	  cAngle = pCollide->getYZAngle(coming);
 	  pCollide->addNeighbour(cAngle, coming);
 	  tAngle = coming->getYZAngle(pCollide);
	  coming->addNeighbour(tAngle, pCollide);
  }
  
  if(rChoice && rCollideTarget.size()>0){
  	if(rCollideTarget.size()>1){
  	  pCollide = getUniqueTarget(coming, rCollideTarget);
  	}
  	else{
  		pCollide = rCollideTarget.front();
  	}  	
 	  cAngle = pCollide->getYZAngle(coming);
 	  pCollide->addNeighbour(cAngle, coming);
 	  tAngle = coming->getYZAngle(pCollide);
	  coming->addNeighbour(tAngle, pCollide);
  }
   
}

 
void Aggregate::recursiveDiffusion(DLAElement* existed, DLAElement* coming){
	float lAdjustAngle, lMinDistance; //=std::numeric_limits<float>::max();
	float rAdjustAngle, rMinDistance; //=std::numeric_limits<float>::max();
	bool  lAdjustable=false;
	bool  rAdjustable=false;
    list<DLAElement*> lCollideTarget;
    list<DLAElement*> rCollideTarget;
    bool lChoice = false;
    bool rChoice = false;
	
	//float initAngle = existed->getYZAngle(coming);
	float stepSize = context->getDiffusionStep();
	//lAdjustable = existed->isAdjustable(coming, stepSize, lAdjustAngle, lMinDistance, lOccupy);
	//rAdjustable = existed->isAdjustable(coming, -stepSize, rAdjustAngle, rMinDistance, rOccupy);
	lAdjustable = adjustTest(existed, coming, stepSize, lAdjustAngle, lMinDistance, lCollideTarget);
	rAdjustable = adjustTest(existed, coming, -stepSize, rAdjustAngle, rMinDistance, rCollideTarget);	
		
	//cout << "left: " << lAdjustable << " - " << lAdjustAngle << " - " << lMinDistance << endl;
	//cout << "right: " << rAdjustable << " - " << rAdjustAngle << " - " << rMinDistance << endl;	
	
	//float newAngle, newY, newZ;
	//float elementHeight = context->getElementHeight();
  DLAElement* pCollide;
  	
	if((!lAdjustable) && (!rAdjustable)){
    float cAngle = existed->getYZAngle(coming);
    existed->addNeighbour(cAngle, coming);
    float tAngle = coming->getYZAngle(existed);
    coming->addNeighbour(tAngle, existed);

    if(lChoice && lCollideTarget.size()>0){
   	  if(lCollideTarget.size()>1){
  	    pCollide = getUniqueTarget(coming, lCollideTarget);
  	  }
  	  else{
  		  pCollide = lCollideTarget.front();
  	  }
 	    cAngle = pCollide->getYZAngle(coming);
 	    pCollide->addNeighbour(cAngle, coming);
 	    tAngle = coming->getYZAngle(pCollide);
	    coming->addNeighbour(tAngle, pCollide);
    }
  
    if(rChoice && rCollideTarget.size()>0){
  	  if(rCollideTarget.size()>1){
  	    pCollide = getUniqueTarget(coming, rCollideTarget);
  	  }
  	  else{
  		  pCollide = rCollideTarget.front();
  	  }  	
 	    cAngle = pCollide->getYZAngle(coming);
 	    pCollide->addNeighbour(cAngle, coming);
 	    tAngle = coming->getYZAngle(pCollide);
	    coming->addNeighbour(tAngle, pCollide);
    }    		
	}
	else{
	  if(lAdjustable && lMinDistance<=rMinDistance){
	  	if(lCollideTarget.size()==0){
	  		cout << "-------------********* L 0 ***********-------" << endl;
	  	}
	  	else{
	  		recursiveDiffusion(lCollideTarget.front(), coming);
	  	}
		  //newAngle = normalAngle(initAngle+lAdjustAngle);
		  //newY = existed->getY() + elementHeight*sin(newAngle);
		  //newZ = existed->getZ() + elementHeight*cos(newAngle);
		  //coming->setPos(coming->getX(), newY, newZ);
		  //lChoice = true;
	  }
	  else{ // if(rAdjustable && rMinDistance<lMinDistance){
	  	if(rCollideTarget.size()==0){
	  		cout << "-------------********* L 0 ***********-------" << endl;
	  	}
	  	else{
	  		recursiveDiffusion(rCollideTarget.front(), coming);
	  	}
	  	
		  //newAngle = normalAngle(initAngle-rAdjustAngle);
		  //newY = existed->getY() + elementHeight*sin(newAngle);
		  //newZ = existed->getZ() + elementHeight*cos(newAngle);		
		  //coming->setPos(coming->getX(), newY, newZ);
	    //rChoice = true;
	  }
	}   
}

bool Aggregate::adjustTest(DLAElement* pFixed
                          , DLAElement* pComing
                          , float stepSize
                          , float& adjustAngle
                          , float& adjustDistance
                          , list<DLAElement*>& collideTarget){
  float tAngle = pFixed->getYZAngle(pComing);
  float initDistance = sqrt(pComing->getY()*pComing->getY() + pComing->getZ()*pComing->getZ());
  float newAngle = tAngle + stepSize;
  float testAngle = fabs(stepSize);
  float minDistance = initDistance;
  float twopi = GL_PI * 2.0f;
  float elementHeight = context->getElementHeight();
  newAngle = normalAngle(newAngle);
  float newX  = pComing->getX();
  float newY  = pFixed->getY() + elementHeight * sin(newAngle);
  float newZ  = pFixed->getZ() + elementHeight * cos(newAngle);
  float newDistance = sqrt(newY*newY + newZ*newZ);
  list<DLAElement* > adjList;
  getAdjacentList(newX, adjList);
  collideTest(pFixed->getElementId(), newX, newY, newZ, adjList, collideTarget);
  //DLAElement* pOccupy = NULL;
  while( (testAngle < twopi)
       && (newDistance < minDistance)
       && (collideTarget.size()==0))
       {

  	//if(newDistance < minDistance){
  		minDistance = newDistance;
  		adjustAngle = testAngle;
  	//}
  	testAngle += fabs(stepSize);
  	newAngle   =  normalAngle(newAngle+stepSize);
  	newY = pFixed->getY() + elementHeight * sin(newAngle);
  	newZ = pFixed->getZ() + elementHeight * cos(newAngle);
  	newDistance = sqrt(newY*newY + newZ*newZ); 
  	collideTest(pFixed->getElementId(), newX, newY, newZ, adjList, collideTarget);  	
  }
  
  //if(collideTarget.size()>0){
  //	cout << "------------" << collideTarget.size() << "------------------" << endl;
  //}
  
  if(fabs(minDistance - initDistance) > EPSILON ){
  	adjustDistance = minDistance;
  	return true;
  }
  //adjustDistance = 0.0f;
  adjustDistance = std::numeric_limits<float>::max();
  return false;  	
}


/**
 * Diffuse based on the change of the free energy.
 * In the previous version, the coming particle attach to the left or right
 * of the existed one is determined by the probability of covered cross surface
 * areas.
 * In this version, the coming one will attach to the left or right which has bigger
 * covered area. 
 */
void Aggregate::diffuseBasedOnEnergy2(DLAElement* existed, DLAElement* coming, int factor){
	float lAdjustAngle = 0.0f; //=std::numeric_limits<float>::max();
	float rAdjustAngle = 0.0f; //=std::numeric_limits<float>::max();
	bool  lAdjustable=false;
	bool  rAdjustable=false;
    list<DLAElement*> lCollideTarget;
    list<DLAElement*> rCollideTarget;
	float initAngle;
	
	//list<DLAElement*> elmsAtX;
    //int numOfElms = getElementsAtXFromMap(coming->getX(), elmsAtX);
    //if(numOfElms>5){
    	initAngle = existed->getYZAngle(coming);
    	lAdjustable = getLeftCollidingTargets(existed, coming, lAdjustAngle, lCollideTarget);
    	rAdjustable = getRightCollidingTargets(existed, coming, rAdjustAngle, rCollideTarget);
    //}
	float elementHeight = context->getElementHeight();
	float distThreshold  = context->getDiffusionDistanceThreshold();
	
	if(lAdjustable && rAdjustable){
		DLAElement* 		pCurLeft=NULL;
		DLAElement* 		pCurRight=NULL;
		DLAElement* 		pCurFixed=existed;
		list<DLAElement* > 	pCurList;
		float			   	pCurAffinity=0.0f;
		float              	pCurAngle=initAngle;	
		float 				newY, newZ;

		//cout << "Both sides are adjustable" << endl;
		int retValue = attachToLeftOrRight(existed, lCollideTarget, rCollideTarget);
		float newAngle;
		if(retValue==1){
		  	newAngle = normalAngle(initAngle+lAdjustAngle);
			newY = existed->getY() + elementHeight*sin(newAngle);
			newZ = existed->getZ() + elementHeight*cos(newAngle);
			coming->setPos(coming->getX(), newY, newZ);
			if(lCollideTarget.empty()){
				cout << "lCollideTarget is empty --- " << endl;
			}
			pCurLeft  = this->getUniqueTargetYZ(coming, lCollideTarget); //lCollideTarget.front();
			pCurRight = existed;
			pCurList  = lCollideTarget;
			pCurAngle = newAngle;
			pCurAffinity = computeAffinityWeight2( coming->getX()
		                                         , coming->getY()
		                                         , coming->getZ()
		                                         , pCurRight		                                         
		                                         , pCurLeft
		                                         , distThreshold
		                                         );			
		}
		else if(retValue==-1){
		  	newAngle = normalAngle(initAngle-rAdjustAngle);
			newY = existed->getY() + elementHeight*sin(newAngle);
			newZ = existed->getZ() + elementHeight*cos(newAngle);
			coming->setPos(coming->getX(), newY, newZ);

			if(rCollideTarget.empty()){
				cout << "rCollideTarget is empty --- " << endl;
			}			
			pCurRight = this->getUniqueTargetYZ(coming, rCollideTarget); //rCollideTarget.front();
			pCurLeft  = existed;			
			pCurList  = rCollideTarget;
			pCurAngle = newAngle;
			pCurAffinity = computeAffinityWeight2( coming->getX()
		                                         , coming->getY()
		                                         , coming->getZ()
		                                         , pCurLeft
		                                         , pCurRight
		                                         , distThreshold
		                                         );			
		}
		else{			
			cout << "Error situation, do nothing\n" ;
		}
		
		//further test		
		float              nextAngle=pCurAngle;
		float              nextAffinity=0.0f;
		list<DLAElement* > nextList;
		//DLAElement*        pFormerFixed = existed;		
		int tryTimes = context->getDiffusionTryTimes();
		if( rCollideTarget.size()==1
			&& lCollideTarget.size()==1
			&& rCollideTarget.front()==lCollideTarget.front()
			){
			tryTimes = 0;
		}
	    for(int i = 0; i < tryTimes; i++ ){
	    	nextList.clear();
	    	int ret=0;
	    	if(factor==1){ //Spin based on unfavorable factors
	    	    ret = movedToLeftOrRight( coming
	    	    						, pCurLeft
	    	    						, pCurRight
	    	    						, distThreshold    								 
	    	    						, nextAngle
	    	    						, nextAffinity
	    	    						, nextList
	    	    						);
	    		
	    	}
	    	
	    	if(ret==1){//move to left
				newY = pCurLeft->getY() + elementHeight*sin(nextAngle);
				newZ = pCurLeft->getZ() + elementHeight*cos(nextAngle);
				coming->setPos(coming->getX(), newY, newZ);	    		
	    		
				if(nextAffinity>pCurAffinity){
					pCurFixed = pCurLeft;
					pCurAngle = nextAngle;
					pCurAffinity = nextAffinity;
					pCurList = nextList;
				}

				pCurRight = pCurLeft;
	    		if(nextList.empty()){
	    			cout << "nextList is empty --------" << endl;
	    		}
	    		if(nextList.size()>1){
	    			pCurLeft = getElementWithMinimumAngle(pCurLeft, coming, nextList);
	    		}
	    		else{
	    			pCurLeft = nextList.front();
	    		//pCurLeft = this->getUniqueTargetYZ(coming, nextList); //nextList.front();
	    		}
	    	}
	    	else if(ret==-1){//move to right
				newY = pCurRight->getY() + elementHeight*sin(nextAngle);
				newZ = pCurRight->getZ() + elementHeight*cos(nextAngle);
				coming->setPos(coming->getX(), newY, newZ);	    		
	    		
				if(nextAffinity>pCurAffinity){
					pCurFixed = pCurRight;
					pCurAngle = nextAngle;
					pCurAffinity = nextAffinity;
					pCurList = nextList;
				}				
	    		pCurLeft = pCurRight;
	    		if(nextList.empty()){
	    			cout << "rTestList is empty --------" << endl;
	    		}
	    		
	    		if(nextList.size()>1){
	    			pCurRight = getElementWithMinimumAngle(pCurRight, coming, nextList);
	    		}
	    		else{
	    			pCurRight = nextList.front();
		    		//pCurRight = this->getUniqueTargetYZ(coming, nextList); //nextList.front();	    			
	    		}
	    	}
	    	else if(ret==-2){//both sides are unfavorable
	    		//cout << "stop from -2 " << "\n";
	    		break;
	    	}
	    	else{
	    	//	cout << "Not adjusted ! " << i << endl;
	    		continue;
	    	}
	    	//cout << "times " << i << ": " << pCurFixed->getElementId() << " - " << pCurAffinity << "\n";
	    	//cout << pCurList.size() << "\n";
		}
	    
		float cAngle, tAngle;

		newY = pCurFixed->getY() + elementHeight*sin(pCurAngle);
		newZ = pCurFixed->getZ() + elementHeight*cos(pCurAngle);
		coming->setPos(coming->getX(), newY, newZ);	    		
		
		cAngle = pCurFixed->getYZAngle(coming);
		pCurFixed->addNeighbour(cAngle, coming);
		tAngle = coming->getYZAngle(pCurFixed);
		coming->addNeighbour(tAngle, pCurFixed);
		
        list<DLAElement*>::iterator rItr;
        for(rItr=pCurList.begin(); rItr!=pCurList.end(); rItr++){
        	cAngle = (*rItr)->getYZAngle(coming);
        	(*rItr)->addNeighbour(cAngle, coming);
        	tAngle = coming->getYZAngle(*rItr);
        	coming->addNeighbour(tAngle, *rItr);
        }		
		
	}
	else if(!lAdjustable && !rAdjustable){ //If both sides are not adjustable
	                                       //, find a optimal position.
		//cout << "Both sides are not adjustable" << endl;
		/*
		float stepSize = context->getDiffusionStep();
		float adjustAngle;
		bool ret;
		ret = findOptimalAngle2(existed, coming, stepSize, adjustAngle);
		if(ret){    
			//cout << "Optimal Angle: " << adjustAngle << endl;
			float newAngle = normalAngle(initAngle+adjustAngle);
			float newY = existed->getY() + elementHeight*sin(newAngle);
			float newZ = existed->getZ() + elementHeight*cos(newAngle);
			coming->setPos(coming->getX(), newY, newZ);      
		}
    	*/                                       
		float cAngle, tAngle;
		cAngle = existed->getYZAngle(coming);
		existed->addNeighbour(cAngle, coming);
		tAngle = coming->getYZAngle(existed);
		coming->addNeighbour(tAngle, existed);		
	}
	else{
		cerr << "Weird thing happened in diffuseBasedOnEnergy2 !" << endl;
		cerr << "Element " << existed->getElementId() << " only one side adjustable." << endl;
	}
}

/**
 * Diffuse based on the change of the free energy.
 * 
 */
void Aggregate::diffuseBasedOnEnergy(DLAElement* existed, DLAElement* coming){
	float lAdjustAngle; //=std::numeric_limits<float>::max();
	float rAdjustAngle; //=std::numeric_limits<float>::max();
	bool  lAdjustable=false;
	bool  rAdjustable=false;
    list<DLAElement*> lCollideTarget;
    list<DLAElement*> rCollideTarget;
    bool lChoice = false;
    bool rChoice = false;
	
    
	float initAngle = existed->getYZAngle(coming);
	float stepSize = context->getDiffusionStep();
	
	//lAdjustable = existed->isAdjustable(coming, stepSize, lAdjustAngle, lMinDistance, lOccupy);
	//rAdjustable = existed->isAdjustable(coming, -stepSize, rAdjustAngle, rMinDistance, rOccupy);
	lAdjustable = adjustCollisionTest(existed, coming, stepSize, lAdjustAngle, lCollideTarget);
	rAdjustable = adjustCollisionTest(existed, coming, -stepSize, rAdjustAngle, rCollideTarget);	
		
	//cout << "left: " << lAdjustable << " - " << lAdjustAngle << endl;
	//cout << "right: " << rAdjustable << " - " << rAdjustAngle << endl;	

    DLAElement* lCollide=NULL;	
    DLAElement* rCollide=NULL;
    float newAngle, newY, newZ;
    float hiddenArea;
    float probability, tp;
	float elementHeight = context->getElementHeight();
	if(lAdjustable){ // Left side adjustable
		lCollide = getUniqueTarget(coming, lCollideTarget);
		hiddenArea = coveredCrossArea(existed, lCollide);
		//float diffusionProbability(float freeEnergy, float temptureConst)
		probability = diffusionProbability(hiddenArea, context->getDiffusionKTConstant()); //4.0f);
	    cout << "left probability: " << probability << endl;		
		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
		if(tp < probability){
			lChoice = true;
		}
	}
	if(rAdjustable){ // Right side adjustable
  	    rCollide = getUniqueTarget(coming, rCollideTarget);
		hiddenArea = coveredCrossArea(existed, rCollide);
		//float diffusionProbability(float freeEnergy, float temptureConst)
		probability = diffusionProbability(hiddenArea, context->getDiffusionKTConstant()); //4.0f);
	    cout << "right probability: " << probability << endl;		
		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
		if(tp < probability){
			rChoice = true;
		}
	}
	
  float cAngle, tAngle;  
  if(lChoice && rChoice){ //Both directions are possible
  	// Randomly choose to diffuse
  	if(rand()%2==1){ // Left side diffusion
	  	newAngle = normalAngle(initAngle+lAdjustAngle);
		newY = existed->getY() + elementHeight*sin(newAngle);
		newZ = existed->getZ() + elementHeight*cos(newAngle);
		coming->setPos(coming->getX(), newY, newZ);
  		
   	    cAngle = lCollide->getYZAngle(coming);
 	    lCollide->addNeighbour(cAngle, coming);
 	    tAngle = coming->getYZAngle(lCollide);
	    coming->addNeighbour(tAngle, lCollide);  		
  	}
  	else{ // Right side diffusion  			
		newAngle = normalAngle(initAngle-rAdjustAngle);
		newY = existed->getY() + elementHeight*sin(newAngle);
		newZ = existed->getZ() + elementHeight*cos(newAngle);		
		coming->setPos(coming->getX(), newY, newZ);
	    		
   	    cAngle = rCollide->getYZAngle(coming);
 	    rCollide->addNeighbour(cAngle, coming);
 	    tAngle = coming->getYZAngle(rCollide);
	    coming->addNeighbour(tAngle, rCollide); 		
  	}
  }
  else if(lChoice){ // Left side diffusion
	  	newAngle = normalAngle(initAngle+lAdjustAngle);
		newY = existed->getY() + elementHeight*sin(newAngle);
		newZ = existed->getZ() + elementHeight*cos(newAngle);
		coming->setPos(coming->getX(), newY, newZ);
  		
   	    cAngle = lCollide->getYZAngle(coming);
 	    lCollide->addNeighbour(cAngle, coming);
 	    tAngle = coming->getYZAngle(lCollide);
	    coming->addNeighbour(tAngle, lCollide);  	
  }
  else if(rChoice){ // Right side diffusion
		newAngle = normalAngle(initAngle-rAdjustAngle);
		newY = existed->getY() + elementHeight*sin(newAngle);
		newZ = existed->getZ() + elementHeight*cos(newAngle);		
		coming->setPos(coming->getX(), newY, newZ);
	    		
   	    cAngle = rCollide->getYZAngle(coming);
 	    rCollide->addNeighbour(cAngle, coming);
 	    tAngle = coming->getYZAngle(rCollide);
	    coming->addNeighbour(tAngle, rCollide);  	
  }
  //else{ // No diffusion at all
  cAngle = existed->getYZAngle(coming);
  existed->addNeighbour(cAngle, coming);
  tAngle = coming->getYZAngle(existed);
  coming->addNeighbour(tAngle, existed);  	
  //}
}

/**
 * CascadeDiffusen
 * 
 */
void Aggregate::cascadeDiffuse(DLAElement* existed, DLAElement* coming, int times){
	DLAElement* pDlaOld;
	DLAElement* pDlaNew;
	pDlaOld = existed;
	//float newAngle, newY, newZ;
	//float elementHeight = context->getElementHeight();
	pDlaNew = pDlaOld;
	//Using the first (times - 1) times to looking for the final element to attach to.
	for(int i = 0; i < times; i++){
		pDlaNew = getNextDiffusionTarget(pDlaOld, coming);
	    //if(pDlaNew == pDlaOld){
		//  break;
		//}
		//Reposition the coming element relative to the new circulate center
		//to reduce accumulated computation error.
		//cout << coming->getElementId( )<< " - " << pDlaNew->getElementId() << " : " << pDlaNew->getYZDistance(coming) << endl;
		//cout << pDlaNew->getYZDistance(coming) << endl;
        //cout << coming->getElementId() << ": " << coming->getY() << ", " << coming->getZ() << endl;
        //cout << pDlaNew->getYZAngle(coming) << endl;
		pDlaOld = pDlaNew;
	}
	
	float cAngle, tAngle;
    cAngle = pDlaNew->getYZAngle(coming);
	pDlaNew->addNeighbour(cAngle, coming);
	tAngle = coming->getYZAngle(pDlaNew);
	coming->addNeighbour(tAngle, pDlaNew);  	
	
//diffuseBasedOnEnergy(pDlaNew, coming);
}

/**
 * Get next diffusion target.
 * 
 */
DLAElement* Aggregate::getNextDiffusionTarget(DLAElement* existed, DLAElement* coming){
	float lAdjustAngle=0.0f; //=std::numeric_limits<float>::max();
	float rAdjustAngle=0.0f; //=std::numeric_limits<float>::max();
	bool  lAdjustable=false;
	bool  rAdjustable=false;
    list<DLAElement*> lCollideTarget;
    list<DLAElement*> rCollideTarget;
    bool lChoice = false;
    bool rChoice = false;
	
	float initAngle = existed->getYZAngle(coming);
	float stepSize = context->getDiffusionStep();
	lAdjustable = adjustCollisionTest(existed, coming, stepSize, lAdjustAngle, lCollideTarget);
	if(lAdjustable){
	  while(lCollideTarget.size()>1 && stepSize > 0.0001){
		lAdjustAngle = 0.0f;
		lCollideTarget.clear();
		stepSize = stepSize/2.0f;
		lAdjustable = adjustCollisionTest(existed, coming, stepSize, lAdjustAngle, lCollideTarget);
	  }
	}
	
	stepSize = context->getDiffusionStep();
	rAdjustable = adjustCollisionTest(existed, coming, -stepSize, rAdjustAngle, rCollideTarget);
	if(rAdjustable){
	  while(rCollideTarget.size()>1 && stepSize > 0.0001){
		rAdjustAngle = 0.0f;
		rCollideTarget.clear();
		stepSize = stepSize/2.0f;
		rAdjustable = adjustCollisionTest(existed, coming, -stepSize, rAdjustAngle, rCollideTarget);
	  }
	}
	
	list<DLAElement*>::iterator litr;
	if(lAdjustable && lCollideTarget.size()>1){
	  cout << "More than one left side collision targets from  " << coming->getElementId() << " -> " << existed->getElementId() << endl;
	  for(litr=lCollideTarget.begin(); litr!=lCollideTarget.end(); litr++){
		  cout << (*litr)->getElementId() << "\n";
	  }
	  lAdjustable = false;	  
	}
	
	if(rAdjustable && rCollideTarget.size()>1){
	  cout << "More than one right side collision targets from  " << coming->getElementId() << " -> " << existed->getElementId() << endl;
	  for(litr=rCollideTarget.begin(); litr!=rCollideTarget.end(); litr++){
		  cout << (*litr)->getElementId() << "\n";
	  }
	  rAdjustable = false;
	}
	
	//cout << "right: " << rAdjustable << " - " << rAdjustAngle << endl;	

    DLAElement* lCollide=NULL;	
    DLAElement* rCollide=NULL;
    float newAngle, newY, newZ;
    float hiddenArea;
    float probability, tp;
	float elementHeight = context->getElementHeight();
	if(lAdjustable){ // Left side adjustable
		lCollide = getUniqueTargetYZ(coming, lCollideTarget);
		hiddenArea = coveredCrossArea(existed, lCollide);
		probability = diffusionProbability(hiddenArea, context->getDiffusionKTConstant()); //4.0f);		
		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
		//cout << "tp=" <<tp << " -- " << "left: " << probability << endl;
		if(tp < probability){
			lChoice = true;
		}
	}
	if(rAdjustable){ // Right side adjustable
  	    rCollide = getUniqueTargetYZ(coming, rCollideTarget);
		hiddenArea = coveredCrossArea(existed, rCollide);
		//float diffusionProbability(float freeEnergy, float temptureConst)
		probability = diffusionProbability(hiddenArea, context->getDiffusionKTConstant()); //4.0f);
	    //cout << "right probability: " << probability << endl;		
		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
		//cout << "tp=" <<tp << " -- " << "right: " << probability << endl;		
		if(tp < probability){
			rChoice = true;
		}
	}
 
  if(lChoice && rChoice){ //Both directions are possible
  	// Randomly choose to diffuse
  	if(rand()%2==1){ // Left side diffusion
	  	  newAngle = normalAngle(initAngle+lAdjustAngle);
		  newY = existed->getY() + elementHeight*sin(newAngle);
		  newZ = existed->getZ() + elementHeight*cos(newAngle);
		  coming->setPos(coming->getX(), newY, newZ);
  		
		  return lCollide;
  	}
  	else{ // Right side diffusion  			
		  newAngle = normalAngle(initAngle-rAdjustAngle);
		  newY = existed->getY() + elementHeight*sin(newAngle);
		  newZ = existed->getZ() + elementHeight*cos(newAngle);		
		  coming->setPos(coming->getX(), newY, newZ);
	    		
		  return rCollide;
  	}
  }
  else if(lChoice){ // Left side diffusion
	  	  newAngle = normalAngle(initAngle+lAdjustAngle);
		  newY = existed->getY() + elementHeight*sin(newAngle);
		  newZ = existed->getZ() + elementHeight*cos(newAngle);
		  coming->setPos(coming->getX(), newY, newZ);
  		  
		  return lCollide;
  }
  else if(rChoice){ // Right side diffusion
		  newAngle = normalAngle(initAngle-rAdjustAngle);
		  newY = existed->getY() + elementHeight*sin(newAngle);
		  newZ = existed->getZ() + elementHeight*cos(newAngle);		
		  coming->setPos(coming->getX(), newY, newZ);
	      
		  return rCollide;
  }

  return existed;
}

/**
 * Get the maximum angle of the coming particle with two particles on the aggregate
 * to be attached.
 * 
 */
float Aggregate::coveredCrossAngle(DLAElement* pFixed, DLAElement* pCollided){
	float distance = pFixed->getYZDistance(pCollided);
	float halfAngle = asin(distance/2.0f/context->getElementHeight());
	return halfAngle*2.0f;
}

/**
 * Get the maximum area of the coming particle with two particles on the aggregate
 * to be attached.
 * 
 */
float Aggregate::coveredCrossArea(DLAElement* pFixed, DLAElement* pCollided){
  float angle = coveredCrossAngle(pFixed, pCollided);
  //float area  = (GL_PI*(context->getElementHeight()/2.0f*context->getElementHeight()/2.0) * angle)/(GL_PI*2.0f);
  float area  = (context->getElementHeight()*context->getElementHeight() * angle)/8.0f;
  return area;
}


float Aggregate::coveredCrossArea(float yPos, float zPos, DLAElement* pFixed, DLAElement* pCollided){
  Vector2D<float> vFirst(pFixed->getY()-yPos, pFixed->getZ()-zPos);
  Vector2D<float> vSecond(pCollided->getY()-yPos, pCollided->getZ()-zPos);  
  return fabs(vFirst.dotProduct(vSecond))/2.0f;
}

/**
 * Collision based adjustment test. The major difference between this function and
 * adjustTest is that this test will stop when the coming spins around 360 degree or
 * it collides with another existing particle in the aggregate. 
 */
bool Aggregate::adjustCollisionTest(DLAElement* pFixed
                          , DLAElement* pComing
                          , float stepSize
                          , float& adjustAngle
                          , list<DLAElement*>& collideTarget){
  if(!collideTarget.empty()){
	  collideTarget.clear();
  }
  adjustAngle = 0.0f;
  float tAngle = pFixed->getYZAngle(pComing);
  float newAngle = tAngle + stepSize;
  float testAngle = fabs(stepSize);
  float twopi = GL_PI * 2.0f;
  float elementHeight = context->getElementHeight();
  newAngle = normalAngle(newAngle);
  float newX  = pComing->getX();
  float newY  = pFixed->getY() + elementHeight * sin(newAngle);
  float newZ  = pFixed->getZ() + elementHeight * cos(newAngle);
  list<DLAElement* > adjList;
  getAdjacentList(newX, adjList);
  //DLAElement* pOccupy = NULL;
  collideTest(pFixed->getElementId(), newX, newY, newZ, adjList, collideTarget);
  while( (testAngle +fabs(stepSize) <= twopi)
       && (collideTarget.size()==0))
       {
 		adjustAngle = testAngle;
 		testAngle += fabs(stepSize);
 		newAngle   =  normalAngle(newAngle+stepSize);
 		newY = pFixed->getY() + elementHeight * sin(newAngle);
 		newZ = pFixed->getZ() + elementHeight * cos(newAngle); 	
 		collideTest(pFixed->getElementId(), newX, newY, newZ, adjList, collideTarget);  	
  }
  
  if(collideTarget.size()>0){
  	//cout << "------------" << collideTarget.size() << "------------------" << endl;
  	return true;
  }
  return false;  	
}


void Aggregate::collideTest(int elmId, float nx, float ny, float nz, list<DLAElement*>& elements, list<DLAElement*>& collideTarget){
  list<DLAElement*>::iterator itr;
  itr = elements.begin();  
  while(itr!=elements.end()){
    float xDist = fabs(nx - (*itr)->getX());
    float yzDist = sqrt((ny - (*itr)->getY())*(ny - (*itr)->getY()) + (nz - (*itr)->getZ())*(nz - (*itr)->getZ()));
    if((xDist < context->getElementWidth()) && (yzDist < context->getElementHeight())){
    	if(((*itr)->getElementId()!=elmId) && (!isInList((*itr), collideTarget))){
        collideTarget.push_front(*itr);
    	}
    }
    itr++; 
  }    
}

bool Aggregate::isInList(DLAElement* pE, list<DLAElement*>& tarList){
	list<DLAElement*>::iterator itr=tarList.begin();
	while(itr!=tarList.end()){
		if(pE==(*itr)){
			return true;
		}
		itr++;
	}
	return false;	
}

int Aggregate::getAdjacentList(float nx, list<DLAElement*>& adjacentList){
	if(!adjacentList.empty()){
		adjacentList.clear();
	}
	
    float mapIndex = ceil(nx/context->getElementWidth()); //aggregate->getDiagonalLength());
    if(mapIndex==-0){
	   mapIndex = 0;
	}
    list<DLAElement* >::const_iterator itr;
	if(elementMap.find(mapIndex)!=elementMap.end()){
		for(itr=elementMap[mapIndex].begin(); itr!=elementMap[mapIndex].end(); itr++ ){
			adjacentList.push_back(*itr);
		}
	}
	if(elementMap.find(mapIndex-1.0f)!=elementMap.end()){
		for(itr=(elementMap[mapIndex-1.0f]).begin(); itr!=(elementMap[mapIndex-1.0f]).end(); itr++ ){
			adjacentList.push_back(*itr);
		}	  
	}
	  
	if(elementMap.find(mapIndex+1.0f)!=elementMap.end()){
		for(itr=(elementMap[mapIndex+1.0f]).begin(); itr!=(elementMap[mapIndex+1.0f]).end(); itr++ ){
			adjacentList.push_back(*itr);
		}	  
	}
	return adjacentList.size();
}
/*
int Aggregate::neighbourTest(int elmId, float nx, float ny, float nz, list<DLAElement*>& collideTarget){
  if(!collideTarget.empty()){
    collideTarget.clear();
  }

  float mapIndex = ceil(nx/context->getElementWidth()); //aggregate->getDiagonalLength());

  if(mapIndex==-0){
    mapIndex = 0;
  }
  
  list<DLAElement* > adjacentList;
  list<DLAElement* >::const_iterator itr;
  if(elementMap.find(mapIndex)!=elementMap.end()){
	for(itr=elementMap[mapIndex].begin(); itr!=elementMap[mapIndex].end(); itr++ ){
		adjacentList.push_back(*itr);
	}
    //collideTest(elmId, nx, ny, nz, elementMap[mapIndex], collideTarget);
  }
  if(elementMap.find(mapIndex-1.0f)!=elementMap.end()){
	for(itr=(elementMap[mapIndex-1.0f]).begin(); itr!=(elementMap[mapIndex-1.0f]).end(); itr++ ){
		adjacentList.push_back(*itr);
	}	  
    //collideTest(elmId, nx, ny, nz, elementMap[mapIndex-1.0f], collideTarget);
  }
  
  if(elementMap.find(mapIndex+1.0f)!=elementMap.end()){
	for(itr=(elementMap[mapIndex+1.0f]).begin(); itr!=(elementMap[mapIndex+1.0f]).end(); itr++ ){
		adjacentList.push_back(*itr);
	}	  
    //collideTest(elmId, nx, ny, nz, elementMap[mapIndex+1.0f], collideTarget);
  }  
  collideTest(elmId, nx, ny, nz, adjacentList, collideTarget);
  return collideTarget.size();
}
*/
bool Aggregate::toCollideWith(DLAElement* p, list<DLAElement*>& elements, list<DLAElement*>& collideTarget){
  list<DLAElement*>::iterator itr;
  itr = elements.begin();  
  bool res = false;
  while(itr!=elements.end()){
    if( p->collideWith(*itr)){      	
      collideTarget.push_front(*itr);
      res = true;
    }
    itr++; 
  }    
  return res;
}

int Aggregate::toBeCollided(DLAElement* p, list<DLAElement*>& collideTarget){
  if(!collideTarget.empty()){
    collideTarget.clear();
  }
 
  //map<float, list<DLAElement*> >* pMap = aggregate->getElementMap();

  float mapIndex = ceil(p->getX()/context->getElementWidth()); //aggregate->getDiagonalLength());

  if(mapIndex==-0){
    mapIndex = 0;
  }
  
  if(elementMap.find(mapIndex)!=elementMap.end()){
    toCollideWith(p, elementMap[mapIndex], collideTarget);
  }
  if(elementMap.find(mapIndex-1.0f)!=elementMap.end()){
    toCollideWith(p, elementMap[mapIndex-1.0f], collideTarget);
  }
  if(elementMap.find(mapIndex+1.0f)!=elementMap.end()){
    toCollideWith(p, elementMap[mapIndex+1.0f], collideTarget);
  }

  return collideTarget.size();
}
//Get unique target based on center distance
DLAElement* Aggregate::getUniqueTarget(DLAElement* p, list<DLAElement*>& collideTarget){
  list<DLAElement*>::iterator itr;
  itr = collideTarget.begin();
  float distance;
  float minDistance = 10000.0f;
  DLAElement* minP = NULL;
  while(itr!=collideTarget.end()){
    //distance = p->distanceTo((*itr)->getX(), (*itr)->getY(), (*itr)->getZ() );
    distance = sqrt( (p->getX()-(*itr)->getX())*(p->getX()-(*itr)->getX())
                   + (p->getY()-(*itr)->getY())*(p->getY()-(*itr)->getY())
                   + (p->getZ()-(*itr)->getZ())*(p->getZ()-(*itr)->getZ()));
    if(distance < minDistance){
      minDistance = distance;
      minP = *itr;
    }
    itr ++;
  }
  return minP;
}

//Get unique target based on yz distance
DLAElement* Aggregate::getUniqueTargetYZ(DLAElement* p, list<DLAElement*>& collideTarget){
  list<DLAElement*>::iterator itr;
  itr = collideTarget.begin();
  float distance;
  float minDistance = 10000.0f;
  DLAElement* minP = NULL;
  while(itr!=collideTarget.end()){
    //distance = p->distanceTo((*itr)->getX(), (*itr)->getY(), (*itr)->getZ() );
    distance = sqrt( (p->getZ()-(*itr)->getZ())*(p->getZ()-(*itr)->getZ())
                   + (p->getY()-(*itr)->getY())*(p->getY()-(*itr)->getY())
                   );
    if(distance < minDistance){
      minDistance = distance;
      minP = *itr;
    }
    itr ++;
  }
  return minP;
}

float Aggregate::percentageOfOverlapped(DLAElement* coming, DLAElement* target){
	  float centerLength = fabs(coming->getX() - target->getX());  
	  float elementWidth = context->getElementWidth();
	  //cout<< "Center Length: " << centerLength << endl;
	  if(centerLength >= elementWidth){
	    return 0.0f;
	  }
	  float connectLength = elementWidth - centerLength;
	  return connectLength/elementWidth;
}
/**
 * 
 * Compute number of crosslinks of an aggregate 
 * 
 */
double Aggregate::computeCrossLinks(int method){
	if(method==1){
	  //Using 20% of average cross domain length as threshold to count number of cross links with neighbours
	  float threshold = context->getAverageCrossDomainLength()/5.0f;
	  return computeNumberOfOverlappedDomainPartsWithNeighbours('c', threshold, method);
	}
	else if(method==2){
	  return computeNumberOfOverlappedDomainPartsWithNeighbours('c', 0.5f, method);
	}
	else{
	  return 0.0;	
	}	
}
/**
 * domainType: c crosslink, h hydrophobic
 * 
 * return: the average number of overlapped parts for a given domain type in the aggregate
 */
double Aggregate::computeNumberOfOverlappedDomainPartsWithNeighbours(char domainType, float threshold, int method){
	  int number = dlaElements.size();
	  if(number==0){
	    return 0.0f;
	  }
	  
	  list<DLAElement*> adjList;
	  list<DLAElement*>::iterator itr = dlaElements.begin();
	  
	  
	  long totalNumber=0;	  
	  while(itr!=dlaElements.end()){
		if(!adjList.empty()){
			adjList.clear();
		}
		getInfluencingNeighbours( (*itr)->getX()
								, (*itr)->getY()
								, (*itr)->getZ()
								, context->getDiffusionDistanceThreshold()  //1.05f
								, adjList);
		
	    //list<DLANeighbour>& nList = (*itr)->getNeighbourList();		
		list<DLAElement*>::const_iterator cItr;
		long numberOfParts=0;
		for(cItr=adjList.begin(); cItr!=adjList.end(); cItr++){
			if(*cItr!=*itr){
				if(method==1){
				  numberOfParts=numberOfOverlappedDomainParts(*itr, *cItr, domainType);
				}
				else if(method==2){
				  numberOfParts=numberOfPartsCenterDist(*itr, *cItr, domainType, threshold);					
				}
				else{
					
				}
				totalNumber+=numberOfParts;
			}
		}
	    itr++;
	  }
	  //Eliminate double computation
	  //double realNumber=(double)totalNumber/2.0;
	  return (double)totalNumber/(double)number;
}
/**
 * 
 * Compute number of crosslinks based on center distance of domain parts 
 * 
 */
long Aggregate::numberOfPartsCenterDist(DLAElement* coming, DLAElement* target, char domainType, float threshold){
	  long numberOfCrosslinks=0;
	  vector<ElementTemplate>* elTemplate = context->getElementTemplate();
	  long numberOfDomains=elTemplate->size();
      
	  for(int i=0; i < numberOfDomains; i++){ //Iterate domains for coming element
		  ElementTemplate* pi = &((*elTemplate)[i]);
		  if(pi->exonType()==domainType){		  
//		  cout << pi->exonLength() << "---";
			  for(int j=0; j < numberOfDomains; j++){ //Iterate domains for target element
				  ElementTemplate* pj = &((*elTemplate)[j]);			  
				  if(pj->exonType()==domainType){				  
					  double iX = coming->getX() + pi->centerDistance();
					  double jX = target->getX() + pj->centerDistance();
					  double standardCenterDistance = (pi->simuLength()+pj->simuLength())/2.0;
					  double currentDistance = fabs(iX-jX);
					  //if( currentDistance < standardCenterDistance ){ //If there are overlapped parts
						  //double smallPartLength = pi->simuLength() < pj->simuLength() ? pi->simuLength() : pj->simuLength();
						  //double overlappedLength = standardCenterDistance - currentDistance;
						  //if(overlappedLength > smallPartLength * threshold){
						  //double bigPartLength = pi->simuLength() < pj->simuLength() ? pi->simuLength() : pj->simuLength();
					  if(currentDistance < standardCenterDistance * threshold){
						  numberOfCrosslinks ++ ;
					  }
					  //}
				  }
			  }
		  }
	  }
//	  cout << endl;
	  return numberOfCrosslinks;
}

/**
 * Compute the number of overlapped domain parts of two models.
 * If the overlapped length bigger than half length of both domains then the overlapped part is counted.
 * 
 * 1. ccccccchhhhhhhhcccccccchhhhhhhhhcchhhhcc 
 * 2.     ccccccchhhhhhhhcccccccchhhhhhhhhcchhhhcc
 *        ---    ----    ----    -----
 *         c       h       c        h 
 * So, number of overlapped parts for domain c is 2, and for domain h is also 2
 */
long Aggregate::numberOfOverlappedDomainParts(DLAElement* coming, DLAElement* target, char domainType ){
	  list<ElementTemplate> lConnectPart;
	  list<ElementTemplate> rConnectPart;
	  //vector<ElementTemplate>* elTemplate = context->getElementTemplate();
	  //cout << "::--------------------------------------::"<<endl;
	  //context->renderTemplateDomainPart(*elTemplate);
	  float centerLength = fabs(coming->getX() - target->getX());  
	  float elementWidth = context->getElementWidth();
	  //cout<< "Center Length: " << centerLength << endl;
	  if(centerLength >= elementWidth){
	    return 0;
	  }
	  float connectLength = elementWidth - centerLength;
	  //cout << "Element Width: " << elementWidth << " centerLength:" << centerLength << " connectLength: " << connectLength << endl;

	  if(coming->getX() <= target->getX()){ // coming is on the left side of the target
	    if(coming->getOrientation()==0){
	      context->getRightPart(connectLength, rConnectPart);
	    }
	    else{
	      context->getReverseRightPart(connectLength, rConnectPart);
	    }

	    if(target->getOrientation()==0){
	      context->getLeftPart(connectLength, lConnectPart);
	    }
	    else{
	      context->getReverseLeftPart(connectLength, lConnectPart);
	    }
	  }
	  else{
	    if(coming->getOrientation()==0){
	      context->getLeftPart(connectLength, lConnectPart);
	    }
	    else{
	      context->getReverseLeftPart(connectLength, lConnectPart);
	    }

	    if(target->getOrientation()==0){
	      context->getRightPart(connectLength, rConnectPart);
	    }
	    else{
	      context->getReverseRightPart(connectLength, rConnectPart);
	    }
	  }
	  //cout << "*********************************" << endl;
	  //context->renderDomainPart(lConnectPart);
	  //context->renderDomainPart(rConnectPart);
	  
	  //float hLength = computeDomainLength(connectLength, domainType, lConnectPart, rConnectPart);
	  long numberOfOverlappedParts = numberOfOverlappedDomainFromParts(domainType, lConnectPart, rConnectPart);
	  return numberOfOverlappedParts;	
}

long Aggregate::numberOfOverlappedDomainFromParts( char domainType, list<ElementTemplate>& lPart, list<ElementTemplate>& rPart){
  list<ElementTemplate>::iterator lItr = lPart.begin();
  list<ElementTemplate>::iterator rItr = rPart.begin();
	list<ElementTemplate> newLPart;
	list<ElementTemplate> newRPart;
	while(lItr!=lPart.end() && rItr!=rPart.end()){
		if(fabs((*lItr).simuLength()-(*rItr).simuLength())<EPSILON){
			newLPart.push_back(*lItr);
			newRPart.push_back(*rItr);
			lItr++;
			rItr++;
		}
		else{
			float preLength;
			if((*lItr).simuLength()<(*rItr).simuLength()){
				newLPart.push_back(*lItr);
				preLength = (*rItr).simuLength();
				//(*rItr).simuLength=(*lItr).simuLength;
				(*rItr).setSimuLength((*lItr).simuLength());
				newRPart.push_back(*rItr);
				//(*rItr).simuLength=preLength -(*lItr).simuLength;
				(*rItr).setSimuLength(preLength -(*lItr).simuLength());
				lItr++;
			}
			else{
				newRPart.push_back(*rItr);
				preLength = (*lItr).simuLength();
				//(*lItr).simuLength = (*rItr).simuLength;
				(*lItr).setSimuLength((*rItr).simuLength());
				newLPart.push_back((*lItr));
				//(*lItr).simuLength=preLength-(*rItr).simuLength;
				(*lItr).setSimuLength(preLength-(*rItr).simuLength());
				rItr++;				
			}
		}
	}
  //cout << "--------------------------------------" << endl;
  //context->renderDomainPart(newLPart);
  //context->renderDomainPart(newRPart);
  list<ElementTemplate>::iterator nLItr = newLPart.begin();
  list<ElementTemplate>::iterator nRItr = newRPart.begin();
  long numberOfOverlapped=0;
  while(nLItr!=newLPart.end() && nRItr!=newRPart.end()){
  	if((*nLItr).simuLength()>EPSILON && (*nLItr).exonType()==domainType && (*nRItr).exonType()==domainType ){
  		//overlappedLength+=(*nLItr).simuLength();
  		//if((*nLItr).simuLength()>threshold){
  		if((*nLItr).simuLength()>((*nLItr).exonLength()/2.0f) && (*nLItr).simuLength() > ((*nLItr).exonLength()/2.0f)) {  			
  		  numberOfOverlapped++;
  		}
  	}
  	nLItr++;
  	nRItr++;
  }
  //cout << numberOfOverlapped << "\n";
  return numberOfOverlapped;	
}


float Aggregate::percentageOfOverlappedDomain(DLAElement* coming, DLAElement* target, char domainType){
  float hLength = overlappedDomainLength(coming, target, domainType);
  if(domainType=='h'){
    if(context->getSimuHydrophobicLength()==0.0f){
      cout << "No hydrophobic exons in the tropoelastin ........... " << endl;
      return 0.0f;
    }  
    return hLength/context->getSimuHydrophobicLength();
  }
  else{
    if((context->getElementWidth() - context->getSimuHydrophobicLength())==0.0f){
      cout << "No cross link exons in the tropoelastin ........... " << endl;
      return 0.0f;
    }  
    return hLength/(context->getElementWidth() - context->getSimuHydrophobicLength());	  
  }
}

float Aggregate::overlappedDomainLength(DLAElement* coming, DLAElement* target, char domainType){
  list<ElementTemplate> lConnectPart;
  list<ElementTemplate> rConnectPart;

  //vector<ElementTemplate* >* elTemplate = context->getElementTemplate();
  float centerLength = fabs(coming->getX() - target->getX());  
  float elementWidth = context->getElementWidth();
  //cout<< "Center Length: " << centerLength << endl;
  if(centerLength >= elementWidth){
    return 0.0f;
  }
  float connectLength = elementWidth - centerLength;
  //cout << "Element Width: " << elementWidth << " centerLength:" << centerLength << " connectLength: " << connectLength << endl;

  if(coming->getX() <= target->getX()){ // coming is on the left side of the target
    if(coming->getOrientation()==0){
      context->getRightPart(connectLength, rConnectPart);
    }
    else{
      context->getReverseRightPart(connectLength, rConnectPart);
    }

    if(target->getOrientation()==0){
      context->getLeftPart(connectLength, lConnectPart);
    }
    else{
      context->getReverseLeftPart(connectLength, lConnectPart);
    }
  }
  else{
    if(coming->getOrientation()==0){
      context->getLeftPart(connectLength, lConnectPart);
    }
    else{
      context->getReverseLeftPart(connectLength, lConnectPart);
    }

    if(target->getOrientation()==0){
      context->getRightPart(connectLength, rConnectPart);
    }
    else{
      context->getReverseRightPart(connectLength, rConnectPart);
    }
  }
  //cout << "*********************************" << endl;
  //context->renderDomainPart(lConnectPart);
  //context->renderDomainPart(rConnectPart);
  
  //float hLength = computeDomainLength(connectLength, domainType, lConnectPart, rConnectPart);
  float hLength = overlappedDomainLengthFromParts(domainType, lConnectPart, rConnectPart);
  return hLength;    
}

/**
 * Compute the overlapped domain length of the domainType.
 * This method is replaced by the overlappedDomainLength method, because
 * it cause some precision problem on the cluster. 
 * 
 */
float Aggregate::computeDomainLength(float connectLength, char domainType, list<ElementTemplate>& lPart, list<ElementTemplate>& rPart){
  //cout << connectLength << endl;
  //context->renderDomainPart(lPart);
  //context->renderDomainPart(rPart);
  list<ElementTemplate>::iterator lItr = lPart.begin();
  list<ElementTemplate>::iterator rItr = rPart.begin();

  float lLength = 0.0f;
  float rLength = 0.0f;
  float hLength = 0.0f;

  char lType = (*lItr).exonType();
  char rType = (*rItr).exonType();
  bool previousH = false;
  cout.precision(10);
  //cout << "--------" << connectLength << endl;
  while(fabs(lLength - connectLength) > EPSILON || fabs(rLength - connectLength) > EPSILON){    
    //cout << "outer loop -- exontype: " << lType << ", " << rType ;
    //if(lType=='h' && rType=='h'){
    try{
    if(lType==domainType && rType==domainType){
      if(!previousH ){
        if(lLength==0.0f && rLength==0.0f){ // The first segments of both lPart and rPart are 'h'
          hLength += fmin((*lItr).simuLength(), (*rItr).simuLength());
        }
        else{
          if(lLength == rLength){
            hLength += fmin((*lItr).simuLength(), (*rItr).simuLength());
          }
          else{
            hLength += fmin(lLength+(*lItr).simuLength(), rLength+(*rItr).simuLength())-fmax(lLength, rLength);
          }
        }
      }
      else{
        if(lLength==rLength){
          hLength += fmin((*lItr).simuLength(), (*rItr).simuLength());
        }
        else{
          hLength += fmin(lLength+(*lItr).simuLength(), rLength+(*rItr).simuLength())-fmax(lLength, rLength);
          //cout << " -- here -- ";
          //hLength += fabs(lLength-rLength);          
        }
      }   
      previousH = true;
    }
    else{
      previousH = false;
    }
    //cout << " Left length: " << lLength << " Right length: " << rLength << " hLength: " << hLength << endl;

    if(lLength == rLength){

      if((*lItr).simuLength() == (*rItr).simuLength()){
        if(lItr!=lPart.end()){
          lLength += (*lItr).simuLength();
          lItr++;
        }
        if(rItr!=rPart.end()){
          rLength += (*rItr).simuLength();
          rItr++;
        }
      }
      else{
        if((*lItr).simuLength() < (*rItr).simuLength()){
          if(lItr!=lPart.end()){
            lLength += (*lItr).simuLength();
            lItr++;
          }
        }
        else{
          if(rItr!=rPart.end()){
            rLength += (*rItr).simuLength();
            rItr++;
          }
        }
      }
    }
    else if( lLength + (*lItr).simuLength() < rLength + (*rItr).simuLength()  ){
      if(lItr!=lPart.end()){
        lLength += (*lItr).simuLength();
        lItr++;
      }
    }
    else if(rLength + (*rItr).simuLength() < lLength + (*lItr).simuLength() ){
      if(rItr!=rPart.end()){
         rLength += (*rItr).simuLength();
         rItr++;
      }
    }
    else {
      if(lLength<rLength){
        if(lItr!=lPart.end()){
          lLength += (*lItr).simuLength();
          lItr++;
        }
      }
      else{
        if(rItr!=rPart.end()){
          rLength += (*rItr).simuLength();
          rItr++;
        }
      }
    }

    if(lItr!=lPart.end()){
      lType = (*lItr).exonType();
    }
    if(rItr!=rPart.end()){
      rType = (*rItr).exonType();
    }
    }
   catch(exception& e){
   	cout << e.what() << endl;
   }
  }
  //cout << "End of loop -- hLength: " << hLength << " Left length: " << lLength << " Right length: " << rLength << endl;
  return hLength;
}

float Aggregate::overlappedDomainLengthFromParts( char domainType,  list<ElementTemplate>& lPart, list<ElementTemplate>& rPart){
  list<ElementTemplate>::iterator lItr = lPart.begin();
  list<ElementTemplate>::iterator rItr = rPart.begin();
	list<ElementTemplate> newLPart;
	list<ElementTemplate> newRPart;
	while(lItr!=lPart.end() && rItr!=rPart.end()){
		if(fabs((*lItr).simuLength()-(*rItr).simuLength())<EPSILON){
			newLPart.push_back(*lItr);
			newRPart.push_back(*rItr);
			lItr++;
			rItr++;
		}
		else{
			float preLength;
			if((*lItr).simuLength()<(*rItr).simuLength()){
				newLPart.push_back(*lItr);
				preLength = (*rItr).simuLength();
				//(*rItr).simuLength=(*lItr).simuLength;
				(*rItr).setSimuLength((*lItr).simuLength());
				newRPart.push_back(*rItr);
				//(*rItr).simuLength=preLength -(*lItr).simuLength;
				(*rItr).setSimuLength(preLength -(*lItr).simuLength());
				lItr++;
			}
			else{
				newRPart.push_back(*rItr);
				preLength = (*lItr).simuLength();
				//(*lItr).simuLength = (*rItr).simuLength;
				(*lItr).setSimuLength((*rItr).simuLength());
				newLPart.push_back((*lItr));
				//(*lItr).simuLength=preLength-(*rItr).simuLength;
				(*lItr).setSimuLength(preLength-(*rItr).simuLength());
				rItr++;				
			}
		}
	}
	//cout << "--------------------------------------" << endl;
  //context->renderDomainPart(newLPart);
  //context->renderDomainPart(newRPart);
  list<ElementTemplate>::iterator nLItr = newLPart.begin();
  list<ElementTemplate>::iterator nRItr = newRPart.begin();
  float overlappedLength=0.0f;
  while(nLItr!=newLPart.end() && nRItr!=newRPart.end()){
  	if((*nLItr).exonType()==domainType && (*nRItr).exonType()==domainType){
  		overlappedLength+=(*nLItr).simuLength();
  	}
  	nLItr++;
  	nRItr++;
  }
  return overlappedLength;	
}
/**
 * Get the minimum y, z, and maximum y, z for a given value of x
 * 
 * 
 */
void  Aggregate::getYZ(float x, float* maxy, float* miny, float *maxz, float* minz){
  float mapIndex = ceil(x/context->getElementWidth()); //aggregate->getDiagonalLength());

  if(mapIndex==-0){
    mapIndex = 0;
  }  
  if(elementMap.find(mapIndex)!=elementMap.end()){
    getYZ(elementMap[mapIndex], maxy, miny, maxz, minz);
  }
  if(elementMap.find(mapIndex-1.0f)!=elementMap.end()){
    getYZ(elementMap[mapIndex-1.0f], maxy, miny, maxz, minz);
  }
  if(elementMap.find(mapIndex+1.0)!=elementMap.end()){
    getYZ(elementMap[mapIndex+1.0f], maxy, miny, maxz, minz);
  }  
}

void Aggregate::getYZ(list<DLAElement*>& elements, float* maxy, float* miny, float *maxz, float* minz){
  list<DLAElement*>::iterator itr = elements.begin();
  while(itr!=elements.end()){
    if((*itr)->getY() > (*maxy)){
      *maxy = (*itr)->getY();
    }
    if((*itr)->getY() < (*miny)){
      *miny = (*itr)->getY();
    }

    if((*itr)->getZ() > (*maxz)){
      *maxz = (*itr)->getZ();
    }
    if((*itr)->getZ() < (*minz)){
      *minz = (*itr)->getZ();
    }
    itr++;
  }
}

/* Methods for computing density */
//
// Compute the average density of the aggregate at the given step 
//
float Aggregate::computeAverageDensity(float step, float range){
  float lowerX = (minX/2.0f)*range + step;
  float topX   = (maxX/2.0f)*range - step;
  float totalDensity=0.0f;
  int   numOfComp=0;
  CircleGridMap cGrid(1000, 1000, 4, 4);
  
  while((lowerX) + step <= topX){
	//cout << "averae density ----------- \n";
	cGrid.reset();
    totalDensity += computeDensityAtX(lowerX, cGrid);
    numOfComp ++;
    lowerX += step;
  }
  float avgDensity = totalDensity/numOfComp;
  return avgDensity;
}

//
// Compute the density of the aggregate at the given position x
//
float Aggregate::computeDensityAtX(float xPos, CircleGridMap& /*cGrid*/){
  list<DLAElement*> resultList;
  //float totalDistance=0.0f;
  float density=0.0f;
  int num = getElementsAtXFromMap(xPos, resultList);
  if(num > 2){
	//Compute cross-section area using convex hull
    list<DLAElement* > hullList;	  
    getConvexHullPoints(xPos, hullList);
    float appArea = computeHullArea(hullList);
	  
    //Computer cross-section area using grid mapping.
    //cGrid.mapListOfPoints(resultList, 0);
    //float appArea = (float)cGrid.computeCircumference();

    density = (float)resultList.size()/appArea;
    //if(density > 1.0f){
    //  cout << "At position: x=" << xPos << endl;
    //  cout << "The total number is: " << num << endl;
    //  cout << "The cross area   is: " << appArea << endl;
    //  cout << "Density: " << density << endl;
    //}
  }
  //else if (num == 0){
  //	  density = 0.0f;
  //}
  else{
	  density = 0.0f;
  }
  return density;
}

//
// Get the elements that are at the position xPos from the list elementList
//
int Aggregate::getElementsAtXFromList(float xPos
                                     , list<DLAElement*>& elementList
                                     , list<DLAElement*>& resultList
                                     ){
  int num = 0;
  list<DLAElement*>::iterator itr;
  itr = elementList.begin();
  while(itr!=elementList.end()){
	//cerr << (*itr)->getX() << endl;
    if(  ((*itr)->getX()-context->getElementWidth()/2.0f <= xPos)
      && ((*itr)->getX()+context->getElementWidth()/2.0f >= xPos)
      ){
      //cerr << (*itr)->getX() << endl;    	
      resultList.push_back(*itr);
      num ++;
    }
    itr++;
  }
  return num;
}

//
// Get the elements that are at the position xPos from the aggregate
//
int Aggregate::getElementsAtXFromMap(float xPos, list<DLAElement*>& resultList){
  float mapIndex = ceil(xPos/context->getElementWidth());
  int num = 0;
  if(mapIndex==-0){
    mapIndex = 0;
  }  
  if(elementMap.find(mapIndex)!=elementMap.end()){
    num += getElementsAtXFromList(xPos, elementMap[mapIndex], resultList);
  }
  if(elementMap.find(mapIndex-1.0f)!=elementMap.end()){
    num += getElementsAtXFromList(xPos, elementMap[mapIndex-1.0f], resultList);
  }
  if(elementMap.find(mapIndex+1.0)!=elementMap.end()){
    num += getElementsAtXFromList(xPos, elementMap[mapIndex+1.0f], resultList);
  }  
  return num;
}

//
// Get the average radius of the aggregate at the position xPos
//
float Aggregate::getAverageRadius(list<DLAElement*>& resultList){
  float aMinY = 10000.0f;
  float aMaxY = -1.0f;
  float aMinZ = 10000.0f;
  float aMaxZ = -1.0f;
  list<DLAElement*>::iterator itr = resultList.begin();
  if(itr==resultList.end()){
    return 0.0f;
  }
  while(itr != resultList.end()){
    if((*itr)->getY() < aMinY){
      aMinY = (*itr)->getY();
    }
    if((*itr)->getY() > aMaxY){
      aMaxY = (*itr)->getY();
    }
    if((*itr)->getZ() < aMinZ){
      aMinZ = (*itr)->getZ();
    }
    if((*itr)->getZ() > aMaxZ){
      aMaxZ = (*itr)->getZ();
    }
    itr++;
  }
  float r = (fabs(aMaxY) + fabs(aMinY) + fabs(aMaxZ) + fabs(aMinZ))/4.0f;
  return (r+context->getElementHeight()/2.0f);
}

/* End of density computation methods */


/* Methods for computing surface area */
//
// Compute the average density of the aggregate at the given step 
//
//float Aggregate::computeSurfaceArea(float step){
//  float lowerX = minX;
//  float topX   = maxX;  
//  float r1   = 0.0f;
//  float r2   = 0.0f;
//  float totalArea=0.0f;
/*  list<DLAElement*> resultList;
  getElementsAtXFromMap( lowerX, resultList);
  r1 = getAverageRadius(resultList);  
  while((lowerX + step) <= topX){
    getElementsAtXFromMap( lowerX+step, resultList);
    r2 = getAverageRadius(resultList);
    totalArea += (GL_PI * (r1+r2)* step);
    lowerX += step;
    r1 = r2;
  }
*/
//  list<DLAElement*> hullList;
//  getConvexHullPoints(lowerX, hullList);
//  r1 = computeHullCircumference(hullList);  
//  while((lowerX + step) <= topX){
//	hullList.clear();
//    getConvexHullPoints(lowerX+step, hullList);
//	r2 = computeHullCircumference(hullList);  
//    totalArea += ((r1+r2)/2.0f * step);
//    lowerX += step;
//    r1 = r2;
//  }
//  
//  return totalArea;
//}

float Aggregate::computeSurfaceArea(float step){
  float lowerX = (minX/2.0f)+step;
  float topX   = (maxX/2.0f)-step;  
  float r1   = 0.0f;
  //float r2   = 0.0f;
  float totalArea=0.0f;
  list<DLAElement*> crossList;
  CircleGridMap cGrid(1000, 1000, 4, 4);
  
  while((lowerX + step) <= topX){
	//cout << "surface area --------- \n";
	crossList.clear();
	cGrid.reset();
    int num = getElementsAtXFromMap(lowerX, crossList);  
	//list<DLAElement*>::const_iterator citr;
	//for(citr=crossList.begin(); citr!=crossList.end(); citr++){
	//  cGrid.mapOnePoint((*citr)->getZ(), (*citr)->getY());
	//}
    if(num>2){
      cGrid.mapListOfPoints(crossList, 0);
      r1 = cGrid.computeCircumference();	
      totalArea += r1 * step;
    }
    lowerX += step;
  }  
  return totalArea;
}

/* End of surface area computation methods */

/* Methods for computing connectivity */

//
// Compute the average number of neighbours of all the elements in the aggregate
//
/*
float Aggregate::computeConnectivity(){
  int number = dlaElements.size();
  if(number==0){
    return 0.0f;
  }
  int totalNeighbours = 0;
  list<DLAElement*>::iterator itr = dlaElements.begin();
  while(itr!=dlaElements.end()){
    list<DLANeighbour>& nList = (*itr)->getNeighbourList();
    totalNeighbours += nList.size();
    itr++;
  }
  return (float)totalNeighbours/(float)number;
}
*/

float Aggregate::computeConnectivity(){
  int number = dlaElements.size();
  if(number==0){
    return 0.0f;
  }
  int totalNeighbours = 0;
  list<DLAElement*> adjList;
  list<DLAElement*>::iterator itr = dlaElements.begin();
  while(itr!=dlaElements.end()){
	//cout << *itr << endl;
	if(!adjList.empty()){
		adjList.clear();
	}
	/*
	getInfluencingNeighbours( (*itr)->getX()
							, (*itr)->getY()
							, (*itr)->getZ()
							, context->getDiffusionDistanceThreshold()  //1.05f
							, adjList);
	*/
	getInfluencingNeighbours( *itr
							, context->getDiffusionDistanceThreshold()  //1.05f
							, adjList);
	
    //list<DLANeighbour>& nList = (*itr)->getNeighbourList();
    totalNeighbours += adjList.size();
    itr++;
  }
  //cout << "Number of elements: "<< number << "\n";
  return (float)totalNeighbours/(float)number;
}


ostream& Aggregate::outputInfluencingMatrix(ostream& ostr){
  list<DLAElement*> adjList;
  list<DLAElement*>::iterator itr = dlaElements.begin();
  while(itr!=dlaElements.end()){
	if(!adjList.empty()){
		adjList.clear();
	}
	getInfluencingNeighbours( (*itr)->getX()
							, (*itr)->getY()
							, (*itr)->getZ()
							, context->getDiffusionDistanceThreshold()  //1.05f
							, adjList);

	list<DLAElement*>::const_iterator citr;
	for(citr = adjList.begin(); citr!=adjList.end(); citr++){
		if((*citr)!=(*itr)){
		  ostr << (*itr)->getElementId() << "\t" << (*citr)->getElementId() << "\n";
		}
	}
    itr++;
  }
  return ostr;
}

void Aggregate::collectConnectivityInfo(map<int, int>& conInfoMap)
{
 	list<DLAElement*>::iterator itr=dlaElements.begin();
 	int conNumber=0;
	list<DLAElement*> adjList;
 	while(itr!=dlaElements.end()){ 		
 		if(!adjList.empty()){
 			adjList.clear();
 		}
 		getInfluencingNeighbours( *itr
 								, context->getDiffusionDistanceThreshold()  //1.05f
 								, adjList);

 		conNumber = adjList.size(); //(*itr)->getNumberOfNeighbours();
 		if(conInfoMap.find(conNumber)!=conInfoMap.end()){
 			conInfoMap[conNumber]++;
 		}
 		else{
 			conInfoMap[conNumber]=1;
 		}
 		itr++;
 	}
}

// Compute neighbour with cross-linking contact for each element
// in the aggregate.
float Aggregate::computeCrossNeighbours(float threshold){
	  int number = dlaElements.size();
	  if(number==0){
	    return 0.0f;
	  }
	  
	  list<DLAElement*> adjList;
	  list<DLAElement*>::iterator itr = dlaElements.begin();
	  while(itr!=dlaElements.end()){
		if(!adjList.empty()){
			adjList.clear();
		}
		getInfluencingNeighbours( (*itr)->getX()
								, (*itr)->getY()
								, (*itr)->getZ()
								, context->getDiffusionDistanceThreshold()  //1.05f
								, adjList);
	    //list<DLANeighbour>& nList = (*itr)->getNeighbourList();
		list<DLAElement*>::const_iterator cItr;
		for(cItr=adjList.begin(); cItr!=adjList.end(); cItr++){
			if(*cItr!=*itr){
				float percent=percentageOfOverlappedDomain(*itr, *cItr, 'c');
				if(percent > threshold){
					(*itr)->addCrossNeighbour(*cItr);
				}
			}
		}
	    itr++;
	  }
	 
	  int totalCross=0;
	  for(itr=dlaElements.begin(); itr!=dlaElements.end(); itr++){
		  totalCross += (*itr)->getNumberOfCrossNeighbours();
	  }
	  return (float)totalCross/(float)number;
}

// Compute neighbours with certain number of connected cross-linking domains
// in the aggregate. The number of connectted cross-linking domains is computed
// using function numberOfOverlappedDomainParts.
// Parameter: threshold - the minimum number of links to be counted as a neighbour
float Aggregate::computeCrossNeighboursMinLinks(int minNumOfLinks){
	  int number = dlaElements.size();
	  if(number==0){
	    return 0.0f;
	  }
	  
	  list<DLAElement*> adjList;
	  list<DLAElement*>::iterator itr = dlaElements.begin();
	  while(itr!=dlaElements.end()){
		if(!adjList.empty()){
			adjList.clear();
		}
		getInfluencingNeighbours( (*itr)->getX()
								, (*itr)->getY()
								, (*itr)->getZ()
								, context->getDiffusionDistanceThreshold()  //1.05f
								, adjList);
	    //list<DLANeighbour>& nList = (*itr)->getNeighbourList();
		list<DLAElement*>::const_iterator cItr;
		for(cItr=adjList.begin(); cItr!=adjList.end(); cItr++){
			if(*cItr!=*itr){
				//float percent=percentageOfOverlappedDomain(*itr, *cItr, 'c');
				int numberOfParts=numberOfOverlappedDomainParts(*itr, *cItr, 'c');				
				if(numberOfParts >= minNumOfLinks){
					(*itr)->addCrossNeighbour(*cItr);
				}
			}
		}
	    itr++;
	  }
	 
	  int totalCross=0;
	  for(itr=dlaElements.begin(); itr!=dlaElements.end(); itr++){
		  totalCross += (*itr)->getNumberOfCrossNeighbours();
	  }
	  return (float)totalCross/(float)number;
}

/**
 * Collect information about the particles' distance to the aggregate's center, and
 * save the result into a map. 
 */     
 void Aggregate::collectDistToCenterInfo(map<float, int>& distInfoMap){
 	list<DLAElement*>::iterator itr=dlaElements.begin();
 	float width = context->getElementWidth();
 	float halfWidth = width/2.0f;
 	float key=0.0f; //, value=0.0f;
 	float ceilingIntegral=0.0f, floorIntegral=0.0f;
 	float x=0.0f;
 	while(itr!=dlaElements.end()){
 		x = (*itr)->getX();
 		ceilingIntegral = ceil((*itr)->getX()/context->getElementWidth()); 		
 		floorIntegral = floor((*itr)->getX()/context->getElementWidth());
 		//cout << (*itr)->getX() << ".."<< ceilingIntegral << ".." << floorIntegral << endl;
 		if(x>width*floorIntegral-halfWidth && x <= width*floorIntegral+halfWidth ){
 			//cout << width*floorIntegral << endl;
 			key = width*floorIntegral;
 		}
 		if(x>width*ceilingIntegral-halfWidth && x <= width*ceilingIntegral+halfWidth ){
 			//cout << width*ceilingIntegral << endl;
 			key = width*ceilingIntegral;
 		}
 		
 		if(key==-0.0f){
 			key = 0.0f;
 		}
 		//cout << key << endl;
 		//conNumber = (*itr)->getNumberOfNeighbours();
 		if(distInfoMap.find(key)!=distInfoMap.end()){
 			distInfoMap[key]++;
 		}
 		else{
 			distInfoMap[key]=1;
 		}
 		itr++;
 	}   	
 } 


void Aggregate::adjustAggregateCenter(){
	float cx=0.0f, cy=0.0f, cz=0.0f;
	float tx=0.0f, ty=0.0f, tz=0.0f;
  list<DLAElement*>::iterator itr = dlaElements.begin();
  while(itr!=dlaElements.end()){
  	tx += (*itr)->getX();
  	ty += (*itr)->getY();
  	tz += (*itr)->getZ();
    itr++;
  }
	//cx = (minX+maxX)/2.0f;
	//cy = (minY+maxY)/2.0f;
	//cz = (minZ+maxZ)/2.0f;
	//cout << tx << ", " << ty << ", " << tz << "--&&&&&&&&-" << numberOfElements << "-&&&&&&&&" << endl;
	cx = tx/(((float)numberOfElements)*2.0f);
	cy = ty/(((float)numberOfElements)*2.0f);
	cz = tz/(((float)numberOfElements)*2.0f);
	//cout << cx << ", " << cy << ", " << cz << "--&&&&&&&&-" << numberOfElements << "-&&&&&&&&" << endl;	
	//cout << log((float)numberOfElements) << endl;
	//float newx, newy, newz;
	if( fabs(cx)>context->getElementWidth()/(log((float)numberOfElements)*2.0f)  //4.0f
	  || fabs(cy)>context->getElementHeight()/(log((float)numberOfElements)*2.0f) //4.0f
	  || fabs(cz)>context->getElementHeight()/(log((float)numberOfElements)*2.0f)){
	  	
    itr = dlaElements.begin();
    while(itr!=dlaElements.end()){
  	  (*itr)->setPos((*itr)->getX() - cx, (*itr)->getY()-cy, (*itr)->getZ()-cz);
      itr++;
    }
  	
    minX -= cx;
    minY -= cy;
    minZ -= cz;
    maxX -= cx;
    maxY -= cy;
    maxZ -= cz;
	}
}
/* End of connectivity computation methods*/

// Member functions for aggregate-shaking

// Randomly choose a point on the aggregate, randomly select an element
// with maximum or minimun y or z coordinates, make further diffuse around 
// its left or right neighbour.
void Aggregate::shaking(int times){
	float halfLength = min(fabs(minX), fabs(maxX));
	halfLength = halfLength - context->getElementWidth();
	float samplingLength = halfLength * 2.0f;
	//cout << "The sampling length is :" << samplingLength << endl;
	int shakedTimes=0;
	for(int i = 0; i < times; i++){
		float samplingPoint = randomSamplingPoint(samplingLength);
		cout << "Sampling point is at: " << samplingPoint << endl;
		bool bRet;
		bRet=shakingAtSamplingPoint(samplingPoint);
		if(bRet){
			shakedTimes++;
		}		
	}
	cout << "Tried: " << times << " shaked: " << shakedTimes << endl;
}

bool Aggregate::shakingAtSamplingPoint(float samplingPoint){
	list<DLAElement*> elmList;
	int ret = getElementsAtXFromMap(samplingPoint, elmList);
	if(ret < 5){//If the sampling point has less than 5 elements,
				//choose another point.
		return false;
	}
	else{
		list<DLAElement*>::iterator litr;
		PointBag2D<Point2D<float> > pointBag;
		for(litr=elmList.begin(); litr!=elmList.end(); litr++ ){
			Point2D<float> point((*litr)->getZ(), (*litr)->getY(), (*litr)->getElementId());
			pointBag.addPoint(point);
		}
		cout << "Number of Points in the bag: " << pointBag.numberOfPoints() << "\n";
		list<Point2D<float> > pList = pointBag.getAllPoints();
		// Randomly select Ymax, Ymin, Zmax, or Zmin
		int randomChoice = rand()%4;
		Point2D<float> chosenPoint;
		switch(randomChoice){				
			case 0:{ //Xmax
				chosenPoint = pointBag.getXMaximum();
				break;
			}
			case 1: //Ymax
			{
				chosenPoint = pointBag.getYMaximum();
				break;
			}
			case 2: //Xmin
			{
				chosenPoint = pointBag.getXMinimum();
				break;
			}
			case 3: //Ymin
			{
				chosenPoint = pointBag.getYMinimum();
				break;
			}
			default:
			{
				cout << "Invalid choice: " << randomChoice << "\n";
				return false;
			}				
		}
		cout << chosenPoint << endl;
		DLAElement* pElm = findElementById(elmList, chosenPoint.id());
		if(pElm!=NULL){
			cout << "Found Element: " << pElm->getElementId() << "\n";
		}
		else{
			cout << "Element was not found: " << chosenPoint.id() << "\n";
			return false;
		}
		int numOfNeighbours = pElm->getNumberOfNeighbours();
		list<DLAElement*> leftList;
		list<DLAElement*> rightList;		
		if(numOfNeighbours<2){ //if the chosen element has less than two neighbours,
			                   //choose another one.
			return false;
		}
		else{
			//split the neighbours into left side and right side lists
			splitNeighbours(pElm, leftList, rightList);
			cout << "Left: ";
			for(litr=leftList.begin(); litr!=leftList.end(); litr++){
				cout << "  " << (*litr)->getElementId(); 
			}
			cout << endl;
			cout << "Right: ";
			for(litr=rightList.begin(); litr!=rightList.end(); litr++){
				cout << "  " << (*litr)->getElementId();
			}
			cout << endl;
		}
		
		//Based on different choices, select the right and left elements will be circled around
		DLAElement* leftElm=NULL;
		DLAElement* rightElm=NULL;
		
		switch(randomChoice){
			case 0:{//Xmax
				//sort the left neighbor list and right neighbour list based on x coordinate.
				leftList.sort(XCompFunctor()); 
				rightList.sort(XCompFunctor());
				if(leftList.empty() && rightList.empty()){
					//Both left and right lists are empty, error
					std::ostringstream exceptInfo;
					exceptInfo << "Exception in Shaking process\n";
					exceptInfo << "The left and right neighbour lists are both empty for element: " << pElm->getElementId() << "\n";
					throw runtime_error(exceptInfo.str());
				}
				else if(leftList.empty() && !rightList.empty()){
					leftElm = rightList.front();
					rightElm = rightList.back();
				}
				else if(!leftList.empty() && rightList.empty()){
					leftElm = leftList.back();
					rightElm = leftList.front();
				}
				else{
					leftElm = leftList.back();
					rightElm = rightList.back();
				}
				break;
			}
			case 1:{//Ymax
				//sort the left neighbor list and right neighbour list based on x coordinate.
				leftList.sort(YCompFunctor()); 
				rightList.sort(YCompFunctor());
				if(leftList.empty() && rightList.empty()){
					//Both left and right lists are empty, error
					ostringstream errInfo;
					errInfo << "Exception in Shaking process\n";
					errInfo << "The left and right neighbour lists are both empty for element: " << pElm->getElementId() << "\n";
					throw runtime_error(errInfo.str());
				}
				else if(leftList.empty() && !rightList.empty()){
					leftElm = rightList.front();
					rightElm = rightList.back();
				}
				else if(!leftList.empty() && rightList.empty()){
					leftElm = leftList.back();
					rightElm = leftList.front();
				}
				else{
					leftElm = leftList.back();
					rightElm = rightList.back();
				}
				break;				
			
			}
			case 2:{//Xmin
				//sort the left neighbor list and right neighbour list based on x coordinate.
				leftList.sort(XCompFunctor());
				rightList.sort(XCompFunctor());
				if(leftList.empty() && rightList.empty()){
					//Both left and right lists are empty, error
					ostringstream errInfo;
					errInfo << "Exception in Shaking process\n";
					errInfo << "The left and right neighbour lists are both empty for element: " << pElm->getElementId() << "\n";
					throw runtime_error(errInfo.str());
				}
				else if(leftList.empty() && !rightList.empty()){
					leftElm = rightList.back();
					rightElm = rightList.front();
				}
				else if(!leftList.empty() && rightList.empty()){
					leftElm = leftList.front();
					rightElm = leftList.back();
				}
				else{
					leftElm = leftList.front();
					rightElm = rightList.front();
				}
				break;
			
			}
			case 3:{//Ymin
				//sort the left neighbor list and right neighbour list based on x coordinate.
				leftList.sort(YCompFunctor());
				rightList.sort(YCompFunctor());
				if(leftList.empty() && rightList.empty()){
					//Both left and right lists are empty, error
					ostringstream errInfo;
					errInfo << "Exception in Shaking process\n";
					errInfo << "The left and right neighbour lists are both empty for element: " << pElm->getElementId() << "\n";
					throw runtime_error(errInfo.str());
				}
				else if(leftList.empty() && !rightList.empty()){
					leftElm = rightList.back();
					rightElm = rightList.front();
				}
				else if(!leftList.empty() && rightList.empty()){
					leftElm = leftList.front();
					rightElm = leftList.back();
				}
				else{
					leftElm = leftList.front();
					rightElm = rightList.front();
				}
				break;			
			}
		}
		cout << "Selected left element: " << leftElm->getElementId() << "\n";
		cout << "Selected right element: " << rightElm->getElementId() << "\n";
		float existCoveredArea = this->coveredCrossArea(leftElm, rightElm);
		cout << "Existing covered cross area: " << existCoveredArea << endl;


		bool  lAdjustable = false;
		bool  rAdjustable = false;
		list<DLAElement*> lCollideTarget;
		list<DLAElement*> rCollideTarget;
		float lAdjustAngle;
		float rAdjustAngle;
		DLAElement* lTarget = NULL;
		DLAElement* rTarget = NULL;

		float stepSize = context->getDiffusionStep();
		lAdjustable = shakingAdjustTest(leftElm, pElm, stepSize, lAdjustAngle, lCollideTarget);
		if(lAdjustable){
		  while(lCollideTarget.size()>1 && stepSize > 0.0001){
			lAdjustAngle = 0.0f;
			lCollideTarget.clear();
			stepSize = stepSize/2.0f;
			lAdjustable = shakingAdjustTest(leftElm, pElm, stepSize, lAdjustAngle, lCollideTarget);
		  }
		}
		
		stepSize = context->getDiffusionStep();
		rAdjustable = shakingAdjustTest(rightElm, pElm, -stepSize, rAdjustAngle, rCollideTarget);
		if(rAdjustable){
		  while(rCollideTarget.size()>1 && stepSize > 0.0001){
			rAdjustAngle = 0.0f;
			rCollideTarget.clear();
			stepSize = stepSize/2.0f;
			rAdjustable = shakingAdjustTest(rightElm, pElm, -stepSize, rAdjustAngle, rCollideTarget);
		  }
		}
        if(!rAdjustable || !lAdjustable){
			//Both left and right lists are empty, error
			ostringstream errInfo;
			errInfo << "Exception in Shaking process\n";
			errInfo << "Element: " << pElm->getElementId();
			
			if(!lAdjustable){
				return false;
				//errInfo << "cannot be displaced on the left side around " << leftElm << "\n";
			}			
			if(!rAdjustable){
				return false;
				//errInfo << "cannot be displaced on the right side around " << rightElm << "\n";
			}
			//throw runtime_error(errInfo.str());        	
        }
        
		lTarget = lCollideTarget.front();
        rTarget = rCollideTarget.front();
        float leftArea, rightArea;        
        leftArea = this->coveredCrossArea(leftElm, lTarget);
        rightArea = this->coveredCrossArea(rightElm, rTarget);
        cout << "left area: " << leftArea << " right area: " << rightArea << endl;
        
        float probability;
        float tp;
        bool lChoice = false;
        bool rChoice = false;
        
        if(leftArea > rightArea && leftArea > existCoveredArea){        	
    		probability = diffusionProbability(leftArea - existCoveredArea, context->getDiffusionKTConstant()); //4.0f);		
    		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
    		if(tp < probability){
        	    cout << "Left probability: " << probability << endl;    			
    			lChoice = true;
    		}
        	
        }
        else if (rightArea > leftArea && rightArea > existCoveredArea){
    		probability = diffusionProbability(rightArea - existCoveredArea, context->getDiffusionKTConstant()); //4.0f);
    		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
    		if(tp < probability){
    			rChoice = true;
        	    cout << "Right probability: " << probability << endl;    			
    		}
    	    
        }
        else if(leftArea==rightArea && rightArea > existCoveredArea){
    		probability = diffusionProbability(rightArea - existCoveredArea, context->getDiffusionKTConstant()); //4.0f);
    		
    		if(rand()%2 == 1){
    			lChoice = true;
    		}
    		else{
    			rChoice = true;    			
    		}
    	    cout << "Random probability: " << probability << endl;        	
        }
        else{
    	    cout << "Not shakable ! " << endl;
    	    return false;
        }
        
        float initAngle, newAngle, cAngle, tAngle;
        float newY, newZ;
        float elementHeight = context->getElementHeight();
        if(lChoice){
        	initAngle = leftElm->getYZAngle(pElm);
        	pElm->detachFromNeighbours();
      	  	newAngle = normalAngle(initAngle+lAdjustAngle);
      		newY = leftElm->getY() + elementHeight*sin(newAngle);
      		newZ = leftElm->getZ() + elementHeight*cos(newAngle);
      		pElm->setPos(pElm->getX(), newY, newZ);
      		
            cAngle = leftElm->getYZAngle(pElm);
            leftElm->addNeighbour(cAngle, pElm);
            tAngle = pElm->getYZAngle(leftElm);
            pElm->addNeighbour(tAngle, leftElm);  	
      		
            list<DLAElement*>::iterator lItr;
            for(lItr=lCollideTarget.begin(); lItr!=lCollideTarget.end(); lItr++){
            	cAngle = (*lItr)->getYZAngle(pElm);
            	(*lItr)->addNeighbour(cAngle, pElm);
            	tAngle = pElm->getYZAngle(*lItr);
            	pElm->addNeighbour(tAngle, *lItr);
            }
        }
        else if(rChoice){
        	initAngle = rightElm->getYZAngle(pElm);
        	pElm->detachFromNeighbours();
      	  	newAngle = normalAngle(initAngle-rAdjustAngle);
      		newY = rightElm->getY() + elementHeight*sin(newAngle);
      		newZ = rightElm->getZ() + elementHeight*cos(newAngle);
      		pElm->setPos(pElm->getX(), newY, newZ);
      		
            cAngle = rightElm->getYZAngle(pElm);
            rightElm->addNeighbour(cAngle, pElm);
            tAngle = pElm->getYZAngle(rightElm);
            pElm->addNeighbour(tAngle, rightElm);  	
      		
            list<DLAElement*>::iterator rItr;
            for(rItr=rCollideTarget.begin(); rItr!=rCollideTarget.end(); rItr++){
            	cAngle = (*rItr)->getYZAngle(pElm);
            	(*rItr)->addNeighbour(cAngle, pElm);
            	tAngle = pElm->getYZAngle(*rItr);
            	pElm->addNeighbour(tAngle, *rItr);
            }      		
        }      
	}
	return true;
}

void Aggregate::splitNeighbours(DLAElement* pChosen, list<DLAElement*>& lList, list<DLAElement*>& rList){
	Vector2D<float> vChosen(pChosen->getZ(), pChosen->getY());
	list<DLANeighbour> nList = pChosen->getNeighbourList();
	list<DLANeighbour>::iterator itr;
	for(itr=nList.begin(); itr!=nList.end(); itr++){
		Vector2D<float> vNeighbour((*itr).neighbour()->getZ(), (*itr).neighbour()->getY());
		float ret = vChosen.crossProduct(vNeighbour);
		if(ret > 0){
			lList.push_back((*itr).neighbour());
		}
		else if(ret < 0){
			rList.push_back((*itr).neighbour());
		}
		else{
			lList.push_back((*itr).neighbour());
			rList.push_back((*itr).neighbour());
		}
	}
	
}

float Aggregate::randomSamplingPoint(float sampleLength){
  return sampleLength * ((float)rand()/((float)(RAND_MAX)+1.0f)) - sampleLength/2.0f;
}

DLAElement* Aggregate::findElementById(int id){
	list<DLAElement*>::iterator itr = find_if(dlaElements.begin(), dlaElements.end(), bind2nd(ElementIdEqual(), id));
	if(itr!=dlaElements.end()){
		return (*itr);
	}
	return NULL;
}

DLAElement* Aggregate::findElementById(const list<DLAElement*>& elmList, int id){
	list<DLAElement*>::const_iterator itr = find_if(elmList.begin(), elmList.end(), bind2nd(ElementIdEqual(), id));
	if(itr!=elmList.end()){
		return (*itr);
	}
	return NULL;
}

/**
 * Collision based adjustment test. The major difference between this function and
 * adjustTest is that this test will stop when the coming spins around 360 degree or
 * it collides with another existing particle in the aggregate. 
 */
bool Aggregate::shakingAdjustTest(DLAElement* pFixed
                          , DLAElement* pComing
                          , float stepSize
                          , float& adjustAngle
                          , list<DLAElement*>& collideTarget){
  float tAngle = pFixed->getYZAngle(pComing);
  float newAngle = tAngle + stepSize;
  float testAngle = fabs(stepSize);
  float twopi = GL_PI * 2.0f;
  float elementHeight = context->getElementHeight();
 	newAngle = normalAngle(newAngle);
 	float newX  = pComing->getX();
 	float newY  = pFixed->getY() + elementHeight * sin(newAngle);
 	float newZ  = pFixed->getZ() + elementHeight * cos(newAngle);
  
  //DLAElement* pOccupy = NULL;
  while( (testAngle < twopi)
       && (shakingNeighbourTest(pFixed->getElementId(), pComing->getElementId(), newX, newY, newZ, collideTarget)==0))
       {
 		adjustAngle = testAngle;
  	testAngle += fabs(stepSize);
  	newAngle   =  normalAngle(newAngle+stepSize);
  	newY = pFixed->getY() + elementHeight * sin(newAngle);
  	newZ = pFixed->getZ() + elementHeight * cos(newAngle); 	
  }
  
  if(collideTarget.size()>0){
  	//cout << "------------" << collideTarget.size() << "------------------" << endl;
  	return true;
  }
  return false;  	
}


void Aggregate::shakingCollideTest(int elmId, int comingId, float nx, float ny, float nz, list<DLAElement*>& elements, list<DLAElement*>& collideTarget){
  list<DLAElement*>::iterator itr;
  itr = elements.begin();  
  while(itr!=elements.end()){
    float xDist = fabs(nx - (*itr)->getX());
    float yzDist = sqrt((ny - (*itr)->getY())*(ny - (*itr)->getY()) + (nz - (*itr)->getZ())*(nz - (*itr)->getZ()));
    if((xDist < context->getElementWidth()) && (yzDist < context->getElementHeight())){
    	if(((*itr)->getElementId()!=elmId) && ((*itr)->getElementId()!=comingId) && (!isInList((*itr), collideTarget))){
        collideTarget.push_front(*itr);
    	}
    }
    itr++; 
  }    
}

int Aggregate::shakingNeighbourTest(int elmId, int comingId, float nx, float ny, float nz, list<DLAElement*>& collideTarget){
  if(!collideTarget.empty()){
    collideTarget.clear();
  }

  float mapIndex = ceil(nx/context->getElementWidth()); //aggregate->getDiagonalLength());

  if(mapIndex==-0){
    mapIndex = 0;
  }
  
  if(elementMap.find(mapIndex)!=elementMap.end()){
    shakingCollideTest(elmId, comingId, nx, ny, nz, elementMap[mapIndex], collideTarget);
  }
  if(elementMap.find(mapIndex-1.0)!=elementMap.end()){
    shakingCollideTest(elmId, comingId, nx, ny, nz, elementMap[mapIndex-1.0f], collideTarget);
  }
  if(elementMap.find(mapIndex+1.0)!=elementMap.end()){
    shakingCollideTest(elmId, comingId, nx, ny, nz, elementMap[mapIndex+1.0f], collideTarget);
  }
  return collideTarget.size();
}

//
//This works in situations where the coming is not attached to the arrgregate,
//which means that the coming one is not in the existed one's neighbour list.
//
bool Aggregate::getLeftCollidingTargets(  DLAElement* existed
										, DLAElement* coming
										, float& lCollidingAngle
										, list<DLAElement*>& lCollidingList
										){
	bool  lAdjustable=false;
    //float elementHeight = context->getElementHeight();
	//float initAngle = existed->getYZAngle(coming);
	
	float stepSize = context->getDiffusionStep();
	lAdjustable = adjustCollisionTest(existed, coming, stepSize, lCollidingAngle, lCollidingList);
	if(lAdjustable && lCollidingList.size()>1){	  
		lCollidingAngle = 0.0f;
		lCollidingList.clear();
		stepSize = 0.0001;
		lAdjustable = adjustCollisionTest(existed, coming, stepSize, lCollidingAngle, lCollidingList);
	}
	return lAdjustable;
}

//
//This works in situations where the coming is not attached to the arrgregate,
//which means that the coming one is not in the existed one's neighbour list.
//
bool Aggregate::getRightCollidingTargets(  DLAElement* existed
										, DLAElement* coming
										, float& rCollidingAngle
										, list<DLAElement*>& rCollidingList
										){
	bool  rAdjustable=false;
    //float elementHeight = context->getElementHeight();
    
	float stepSize = context->getDiffusionStep();	
	rAdjustable = adjustCollisionTest(existed, coming, -stepSize, rCollidingAngle, rCollidingList);
	if(rAdjustable && rCollidingList.size()>1){
		rCollidingAngle = 0.0f;
		rCollidingList.clear();
		stepSize = 0.0001f;
		rAdjustable = adjustCollisionTest(existed, coming, -stepSize, rCollidingAngle, rCollidingList);		
	}
	return rAdjustable;
}

/**
*return: 1 - to the left
*        0 - error, neither left nor right
*        -1 - to the right
*/
int Aggregate::attachToLeftOrRight(   DLAElement* existed
									, list<DLAElement*>& lCollidingTarget
									, list<DLAElement*>& rCollidingTarget
									){
	DLAElement* lCollide=NULL;
	DLAElement* rCollide=NULL;
	bool lChoice = false;
	bool rChoice = false;
	float leftCoveredArea, rightCoveredArea;
	lCollide = lCollidingTarget.front();
	rCollide = rCollidingTarget.front();
	leftCoveredArea = coveredCrossArea(existed, lCollide);
	rightCoveredArea = coveredCrossArea(existed, rCollide);
	
	//float probability, tp;
	if(leftCoveredArea > rightCoveredArea){//Try left first
		lChoice = true;
		//cout << "Left Area: " << leftCoveredArea << " - " << rightCoveredArea << endl;			
	}
	else if(rightCoveredArea > leftCoveredArea){//Try right first
		rChoice = true;
		//cout << "Right Area: " << rightCoveredArea << " - " << leftCoveredArea << endl;	
	}
	else{
		if(rand()%2==1){
			lChoice = true;
		}
		else{
			rChoice = true;
		}
	}
	
	if(lChoice){
		return 1;
	}
	else if(rChoice){
		return -1;
	}
	else{
		cerr << "Error in Aggregate::attachToLeftOrRight, neither left nor right, no decision had been made!" << endl;
		return 0;
	}
}

/**
 * To continuely move the coming particle around the first contact's left or right
 * 
 * return: 1 - left
 *         0 - not move
 *         -1 - right
 */
int Aggregate::movedToLeftOrRight( DLAElement* coming
								 , DLAElement* pCurLeft
								 , DLAElement* pCurRight
								 , float distThreshold								 
								 , float& nextAngle
								 , float& nextAffinity
								 , list<DLAElement*>& nextList
								 ){
  //cout << "Coming element: " << coming->getElementId() << "\n";
  //cout << "Existing Affinity: ";
	

	float existAffinity = computeAffinityWeight2( coming->getX()
                                             , coming->getY()
                                             , coming->getZ()
                                             , pCurLeft
                                             , pCurRight
                                             , distThreshold
                                             );
                                             
	bool lPossible = false;
	bool rPossible = false;
	float lTestAngle, rTestAngle;
	list<DLAElement*> lTestList;
	list<DLAElement*> rTestList;
	
	lPossible = getLeftCollidingTargets(pCurLeft, coming, lTestAngle, lTestList);
	rPossible = getRightCollidingTargets(pCurRight, coming, rTestAngle, rTestList);
	
	//float lCoveredArea;
	//float rCoveredArea;
	float lAffinity=0.0f;
	float rAffinity=0.0f;
	float tp=0.0f;
	//bool ret = false;
	
	if(!(lPossible && rPossible)){
		cout << "Something wrong in moveToLeftOrRight test ----------- !" << "\n";
		return -2;
	}
    //cout << "Possible: " << lPossible << " -- " << rPossible << "\n";
    
	float elementHeight = context->getElementHeight();
	float lAngle    = pCurLeft->getYZAngle(coming);
 	float lNewAngle = normalAngle(lAngle+lTestAngle);
 	float lNewY     = pCurLeft->getY() + elementHeight * sin(lNewAngle);
 	float lNewZ     = pCurLeft->getZ() + elementHeight * cos(lNewAngle); 	

	float rAngle    = pCurRight->getYZAngle(coming);
 	float rNewAngle = normalAngle(rAngle-rTestAngle);
 	float rNewY     = pCurRight->getY() + elementHeight * sin(rNewAngle);
 	float rNewZ     = pCurRight->getZ() + elementHeight * cos(rNewAngle); 	

	lAffinity = computeAffinityWeight2( coming->getX()
                                             , lNewY
                                             , lNewZ
                                             , pCurLeft
                                             , lTestList.front()
                                             , distThreshold
                                             );

	rAffinity = computeAffinityWeight2( coming->getX()
                                             , rNewY
                                             , rNewZ
                                             , pCurRight
                                             , rTestList.front()
                                             , distThreshold
                                             );

	float lProbability=0.0f;
	float rProbability=0.0f;
	
	int retValue;
	
	if(lAffinity>=existAffinity && rAffinity>=existAffinity){
		/*if(lAffinity>rAffinity){
			//return 1;
			retValue = 1;
		}
		else if(lAffinity<rAffinity){
			//return -1;
			retValue = -1;
		}
		else{*/
			if(rand()%2==1){
				//return 1;
				retValue = 1;
			}
			else{
				//return -1;
				retValue = -1;
			}
		//}
	}
	else if(lAffinity>=existAffinity){
		//return 1;
		retValue = 1;
	}
	else if(rAffinity >= existAffinity ){
		//return -1;
		retValue = -1;
	}
	else{
		bool lChoice = false;
		bool rChoice = false;		
		//cout << existedCoveredArea-lCoveredArea << endl;		
		//probability = diffusionProbability(existedCoveredArea-lCoveredArea, context->getDiffusionKTConstant()); //4.0f);
		//cout << existAffinity-lAffinity << "\n";
		lProbability = diffusionProbability(existAffinity-lAffinity, context->getDiffusionKTConstant()); //4.0f);
	    //cout << "left probability: " << lProbability << "\n";		
		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
		//cout << "tp: " << tp << "\n";
		if(tp < lProbability){
			lChoice = true;
		}
		//cout << existAffinity-rAffinity << "\n";
		//probability = diffusionProbability(existedCoveredArea-rCoveredArea, context->getDiffusionKTConstant()); //4.0f);
		rProbability = diffusionProbability(existAffinity-rAffinity, context->getDiffusionKTConstant()); //4.0f);		
	    //cout << "right probability: " << rProbability << "\n";		
		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
		//cout << "tp: " << tp << "\n";
		if(tp < rProbability){
			rChoice = true;
		}
		
		if(lChoice && rChoice){
			/*if(lProbability > rProbability){		
				//return 1;
				retValue = 1;
			}
			else if(lProbability < rProbability){		
				//return -1;
				retValue = -1;
			}
			else{*/
				if(rand()%2==1){
					retValue = 1;
					//return 1;
				}
				else{
					//return -1;
					retValue = -1;
				}
			//}
		}
		else if(lChoice && !rChoice){
			//return 1;
			retValue = 1;
		}
		else if(!lChoice && rChoice){
			//return -1;
			retValue = -1;
		}
		else{
			//return 0;
			retValue = 0;
		}		
	}

	if(retValue==1){
		nextAngle = lNewAngle;
		nextAffinity = lAffinity;
		nextList = lTestList;
	}
	else if(retValue==-1){
		nextAngle = rNewAngle;
		nextAffinity = rAffinity;
		nextList = rTestList;
	}	
	return retValue;
}

DLAElement* Aggregate::getElementWithMinimumAngle(DLAElement* pFixed, DLAElement* pComing, const list<DLAElement* >& tList){
	Vector2D<float> baseVector(pComing->getZ()-pFixed->getZ(), pComing->getY()-pFixed->getY());			
	float minAngle = 1000.0f;
	float curAngle = 0.0f;
	DLAElement* minElm=NULL;			
	//cout << tList.size() << ": " << pComing->getElementId() << " - " << pFixed->getElementId() << "\n";
	list<DLAElement* >::const_iterator itr;
	for(itr=tList.begin(); itr!=tList.end(); itr++){
		Vector2D<float> curVector((*itr)->getZ()-pFixed->getZ(),(*itr)->getY()-pFixed->getY() );
		curAngle = baseVector.angleTo(curVector);
		//cout << " -- " << (*itr)->getElementId() << " : " << curAngle;
		//cout << " -- " << (*itr)->getElementId() << " : " << curVector.length();				
		if(curAngle<minAngle){
			minAngle = curAngle;
			minElm = *itr;
		}
	}			
	//cout << "\n";
	//cout << maxElm->getElementId() << " selected !\n";	
	return minElm;	
}

DLAElement* Aggregate::getElementWithMinimumAngle( DLAElement* pFixed 
												 , float yPos
												 , float zPos
												 , const list<DLAElement* >& tList
												 ){
	Vector2D<float> baseVector(zPos-pFixed->getZ(), yPos-pFixed->getY());			
	float minAngle = 1000.0f;
	float curAngle = 0.0f;
	DLAElement* minElm=pFixed;			
	//cout << tList.size() << ": " << pComing->getElementId() << " - " << pFixed->getElementId() << "\n";
	list<DLAElement* >::const_iterator itr;
	for(itr=tList.begin(); itr!=tList.end(); itr++){
		Vector2D<float> curVector((*itr)->getZ()-pFixed->getZ(),(*itr)->getY()-pFixed->getY() );
		curAngle = baseVector.angleTo(curVector);
		//cout << " -- " << (*itr)->getElementId() << " : " << curAngle;
		//cout << " -- " << (*itr)->getElementId() << " : " << curVector.length();				
		if(curAngle<minAngle){
			minAngle = curAngle;
			minElm = *itr;
		}
	}			
	//cout << "\n";
	//cout << maxElm->getElementId() << " selected !\n";	
	return minElm;	
}

/**
 * To decide the coming particle can spin around the first contact's left or right.
 * 
 * 
 * return: 1 - left
 *         0 - not move
 *         -1 - right
 */
int Aggregate::movedToLeftOrRight2( DLAElement* coming
								 , DLAElement* pCurLeft
								 , DLAElement* pCurRight
								 , float distThreshold								 
								 , float& lTestAngle
								 , list<DLAElement*>& lTestList
								 , float& rTestAngle
								 , list<DLAElement*>& rTestList
								 ){
	bool lChoice = false;
	bool rChoice = false;
	bool lPossible = false;
	bool rPossible = false;
	
	//float elementLength  = context->getElementWidth();
	//float influenceCoefficient = context->getDiffusionInfluenceCoefficient();
	//float distThreshold  = context->getDiffusionDistanceThreshold();
	
	float existAffinity = computeAffinityWeight2( coming->getX()
                                             , coming->getY()
                                             , coming->getZ()
                                             , pCurLeft
                                             , pCurRight
                                             , distThreshold
                                             );
                                             

	lPossible = getLeftCollidingTargets(pCurLeft, coming, lTestAngle, lTestList);
	rPossible = getRightCollidingTargets(pCurRight, coming, rTestAngle, rTestList);
	
	float lAffinity=0.0f;
	float rAffinity=0.0f;
	float tp=0.0f;
	//bool ret = false;
	
	if(!(lPossible && rPossible)){
		cout << "Something wrong in moveToLeftOrRight test ----------- !" << "\n";
		return -2;
	}

	//else
	float elementHeight = context->getElementHeight();
	float lAngle    = pCurLeft->getYZAngle(coming);
 	float lNewAngle = normalAngle(lAngle+lTestAngle);
 	float lNewY     = pCurLeft->getY() + elementHeight * sin(lNewAngle);
 	float lNewZ     = pCurLeft->getZ() + elementHeight * cos(lNewAngle); 	

	float rAngle    = pCurRight->getYZAngle(coming);
 	float rNewAngle = normalAngle(rAngle-rTestAngle);
 	float rNewY     = pCurRight->getY() + elementHeight * sin(rNewAngle);
 	float rNewZ     = pCurRight->getZ() + elementHeight * cos(rNewAngle); 	
	//cout << "Left: ";
	lAffinity = computeAffinityWeight2( coming->getX()
                                             , lNewY
                                             , lNewZ
                                             , pCurLeft
                                             , lTestList.front()
                                             , distThreshold
                                             );
	//cout << "Right: ";
	rAffinity = computeAffinityWeight2( coming->getX()
                                             , rNewY
                                             , rNewZ
                                             , pCurRight
                                             , rTestList.front()
                                             , distThreshold
                                             );

	//lCoveredArea = coveredCrossArea(pCurLeft, lTestList.front());
	//rCoveredArea = coveredCrossArea(pCurRight, rTestList.front());	

//	if(lCoveredArea >= existedCoveredArea){
	float lProbability=0.0f;
	float rProbability=0.0f;
	if(lAffinity < existAffinity && rAffinity < existAffinity){
		return -2;
	}
	else if(lAffinity>=existAffinity){
		//cout << lAffinity - existAffinity << endl;
		lProbability = diffusionProbability2(lAffinity-existAffinity, context->getDiffusionKTConstant()); //4.0f);
	    //cout << "left probability: " << lProbability << endl;		
		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
		//cout << "tp: " << tp << endl;
		if(tp < lProbability){
			lChoice = true;
		}				
	}
	else if(rAffinity >= existAffinity){
		//cout << rAffinity - existAffinity << endl;
		//probability = diffusionProbability(existedCoveredArea-rCoveredArea, context->getDiffusionKTConstant()); //4.0f);
		rProbability = diffusionProbability2(rAffinity-existAffinity, context->getDiffusionKTConstant()); //4.0f);		
	    //cout << "right probability: " << rProbability << endl;		
		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
		//cout << "tp: " << tp << endl;
		if(tp < rProbability){
			rChoice = true;
		}
	}
	if(lChoice && rChoice){
		if(lProbability > rProbability){		
			return 1;
		}
		else if(lProbability < rProbability){		
			return -1;
		}
		else{
			if(rand()%2==1){
				return 1;
			}
			else{
				return -1;
			}
		}
	}
	else if(lChoice && !rChoice){
		return 1;
	}
	else if(!lChoice && rChoice){
		return -1;
	}
	else{
		return 0;
	}
}

/**
* Find an optimal angle for the coming element to attach to the first contact
* on the aggregate.
*
*/
bool Aggregate::findOptimalAngle(DLAElement* fixed, DLAElement* coming, float stepSize, float& adjustAngle){
  list<DLAElement* > adjacentList;
  getCrossElements(coming->getX(), adjacentList);    
  //getElementsAtXFromMap(coming->getX(), adjacentList);
  //getElementsAtXFromMap(coming->getX() - context->getElementWidth()/2.0f + EPSILON, adjacentList);
  //getElementsAtXFromMap(coming->getX() + context->getElementWidth()/2.0f - EPSILON, adjacentList);
  if(adjacentList.empty()){
    return false;
  }

  adjustAngle = 0.0f;
  float tAngle = fixed->getYZAngle(coming);
  float newAngle = tAngle;
  float testAngle = 0.0f;
  float twopi = GL_PI * 2.0f;
  float elementHeight = context->getElementHeight();
  
  //DLAElement* pOccupy = NULL;
  float minDist = 100000.0f;
  float curDist = 0.0f;
  float newY, newZ;
  while(testAngle < twopi){
 		//adjustAngle = testAngle;
  	testAngle += fabs(stepSize);
  	newAngle   =  normalAngle(newAngle+stepSize);
  	newY = fixed->getY() + elementHeight * sin(newAngle);
  	newZ = fixed->getZ() + elementHeight * cos(newAngle); 	
  	curDist = computeDistanceSum(newY, newZ, adjacentList);
  	if(curDist < minDist){
  	  minDist = curDist;
  	  adjustAngle = testAngle;
  	}
  }
  if(adjustAngle!=0.0f){
    return true;
  }
  return false;  	 
}

/**
* Find an optimal angle for the coming element to attach to the first contact
* on the aggregate.
*
*/
bool Aggregate::findOptimalAngle2(DLAElement* fixed, DLAElement* coming, float stepSize, float& adjustAngle){
  //list<DLAElement* > adjacentList;
  //getCrossElements(coming->getX(), adjacentList);    
  //getElementsAtXFromMap(coming->getX(), adjacentList);
  //getElementsAtXFromMap(coming->getX() - context->getElementWidth()/2.0f + EPSILON, adjacentList);
  //getElementsAtXFromMap(coming->getX() + context->getElementWidth()/2.0f - EPSILON, adjacentList);
  //if(adjacentList.empty()){
  //  return false;
  //}

  adjustAngle = 0.0f;
  float tAngle = fixed->getYZAngle(coming);
  float newAngle = tAngle;
  float testAngle = 0.0f;
  float twopi = GL_PI * 2.0f;
  float elementHeight = context->getElementHeight();
  
  //DLAElement* pOccupy = NULL;
  float minDist = 100000.0f;
  float curDist = 0.0f;
  float newY, newZ;
  while(testAngle < twopi){
 		//adjustAngle = testAngle;
  	testAngle += fabs(stepSize);
  	newAngle   =  normalAngle(newAngle+stepSize);
  	newY = fixed->getY() + elementHeight * sin(newAngle);
  	newZ = fixed->getZ() + elementHeight * cos(newAngle); 	
  	//curDist = computeDistanceSum(newY, newZ, adjacentList);
  	curDist = sqrt(newY*newY + newZ*newZ);
  	if(curDist < minDist){
  	  minDist = curDist;
  	  adjustAngle = testAngle;
  	}
  }
  if(adjustAngle!=0.0f){
    return true;
  }
  return false;  	 
}

/**
*
*
*/
float Aggregate::computeDistanceSum(float posY, float posZ, list<DLAElement* >& tList){
  list<DLAElement*>::iterator itr;
  float distSum = 0.0f;
  for(itr=tList.begin(); itr!=tList.end(); itr++){
    distSum += sqrt(((*itr)->getY()-posY)*((*itr)->getY()-posY) + ((*itr)->getZ()-posZ)*((*itr)->getZ()-posZ));
  }
  return distSum;
}

float Aggregate::computeDistanceWeight(float posX, float posY, float posZ){
	  list<DLAElement* > adjacentList;
	  getCrossElements(posX, adjacentList);    
	  if(adjacentList.empty()){
	    return 0.0f;
	  }
	  return computeDistanceSum(posY, posZ, adjacentList);	
}

/**
*
*
*/
float Aggregate::computeAffinityWeight( float xPos
                                      , float yPos
                                      , float zPos
                                      , DLAElement* pFixed
                                      , DLAElement* pTarget
                                      , float distThreshold
                                      , float infCoeff
                                      , float elmLength
                                      ){
  float affWeight = 0.0f;
  affWeight += computeOverlappedVolume(pFixed, pTarget)*0.1f;
  //cout << "Volume Weight: " << affWeight << "\n"; 
 
  float neighbourWeight = computeNeighbourInfluencingFactor(xPos, yPos, zPos, distThreshold, infCoeff, elmLength);
  //cout << "NeighbourWeight: " << neighbourWeight << "\n";
  affWeight += neighbourWeight;
  //cout << affWeight << "\n";
  return affWeight;
}

/**
*
*
*/
/*
float Aggregate::computeAffinityWeight2( float xPos
                                      , float yPos
                                      , float zPos
                                      , DLAElement* pFixed
                                      , DLAElement* pTarget
                                      , float distThreshold
                                      ){
	float affWeight = 0.0f;
	list<DLAElement* > infList;
	map<float, DLAElement*> angleMap;

	getInfluencingNeighbours(xPos, yPos, zPos, distThreshold, infList);
  
	if(infList.size()<2){
	   affWeight=0.0f;	  
	}
	else if(infList.size()==2){
		Vector2D<float> vecFirst(pFixed->getZ()-zPos, pFixed->getY()-yPos);
		Vector2D<float> vecSecond(pTarget->getZ()-zPos, pTarget->getY()-yPos);
		//affWeight = vecFirst.angleTo(vecSecond);
		affWeight = fabs(vecFirst.dotProduct(vecSecond))/2.0f;
		//affWeight += computeOverlappedVolume(pFixed, pTarget)*0.1f;
	}
	else{
		//cout << infList.size() << "\n";
		//Compute the angles relative to the first element		
		DLAElement* pFirst = pFixed; //infList.front();
		Vector2D<float> baseVector(pFirst->getZ()-zPos, pFirst->getY()-yPos);
		list<DLAElement* >::iterator itr;
		for(itr=infList.begin(); itr!=infList.end(); itr++){
			if(pFirst==(*itr)){
				angleMap[0.0f]= *itr;
			}
			else{
				Vector2D<float> curVector((*itr)->getZ()-zPos, (*itr)->getY()-yPos);
				if(baseVector.length()==0.0f || curVector.length()==0.0f){
					cout << "Vector length is zero: " << baseVector << " - " << curVector << "\n";					
				}
				float angle;
				float direction = baseVector.dotProduct(curVector); 
				if( direction > 0.0f){
				  angle = curVector.angleTo(baseVector);
				}
				else if(direction < 0.0f){
				  angle = -curVector.angleTo(baseVector);					
				}
				else{
				  cout << "Attention ----------- collinear happened !" << "\n";
				  angle = -0.0f;					
				}
				//cout << "angle: " << angle << "\n";
				angleMap[angle]= *itr;
			}
		}
		map<float, DLAElement*>::iterator mitr;
		map<float, DLAElement*>::reverse_iterator ritr;		
		//float yzDist;
		mitr=angleMap.begin();		
		pFirst = (*mitr).second;
		DLAElement* pLast;
		ritr=angleMap.rbegin();
		pLast = (*ritr).second;
		if( (*mitr).first<0.0f
		   && (*ritr).first>0.0f
		   && (*mitr).second!=pFixed
		   && (*ritr).second!=pFixed
		   ){
			//cout << "-----------------------------------\n" ;
			Vector2D<float> vecFirst(pFirst->getZ()-zPos, pFirst->getY()-yPos);
			Vector2D<float> vecMiddle(pFixed->getZ()-zPos, pFixed->getY()-yPos);
			Vector2D<float> vecSecond(pLast->getZ()-zPos, pLast->getY()-yPos);
			//affWeight += vecFirst.angleTo(vecMiddle);
			//affWeight += vecMiddle.angleTo(vecSecond);
			affWeight += fabs(vecFirst.dotProduct(vecMiddle))/2.0f;
			affWeight += fabs(vecMiddle.dotProduct(vecSecond))/2.0f;			
			
		}
		else{
			Vector2D<float> vecFirst(pFirst->getZ()-zPos, pFirst->getY()-yPos);
			Vector2D<float> vecSecond(pLast->getZ()-zPos, pLast->getY()-yPos);
			//affWeight = vecFirst.angleTo(vecSecond);
			affWeight = fabs(vecFirst.dotProduct(vecSecond))/2.0f;			
		}
		//cout << "\n";
	}
	
  //cout << "Volume Weight: " << affWeight << "\n"; 
 
  //float neighbourWeight = computeNeighbourInfluencingFactor(xPos, yPos, zPos, distThreshold, infCoeff, elmLength);
  //cout << "NeighbourWeight: " << neighbourWeight << "\n";
  //affWeight += neighbourWeight;
  //cout << affWeight << "\n";
  return affWeight;
}
*/

/**
*
*
*/

float Aggregate::computeAffinityWeight2( float xPos
                                      , float yPos
                                      , float zPos
                                      , DLAElement* /*pFixed*/
                                      , DLAElement* pComing
                                      , float distThreshold
                                      ){
	float affWeight = 0.0f;
	list<DLAElement* > infList;
	map<float, DLAElement*> angleMap;
	
	float tmpX = pComing->getX();
	pComing->setPos(xPos, pComing->getY(), pComing->getZ());

	getInfluencingNeighbours(xPos, yPos, zPos, distThreshold, infList);
  
	list<DLAElement* >::iterator itr;
	for(itr=infList.begin(); itr!=infList.end(); itr++){
		affWeight += this->overlappedDomainLength(pComing, *itr, 'h');	
	}
	pComing->setPos(tmpX, pComing->getY(), pComing->getZ());	
	return affWeight;
}

//
//
//
float Aggregate::computeAffinityWeight2( DLAElement* pComing
                                       , float distThreshold
                                       ){
	float affWeight = 0.0f;
	list<DLAElement* > infList;
	
	getInfluencingNeighbours(pComing, distThreshold, infList);
  
	list<DLAElement* >::iterator itr;
	for(itr=infList.begin(); itr!=infList.end(); itr++){
		affWeight += this->overlappedDomainLength(pComing, *itr, 'h');	
	}
	
	return affWeight;
}

/**
*
*
*/
float Aggregate::computeOverlappedVolume(DLAElement* pElm1, DLAElement* pElm2){
  //cout << "(" << pElm1->getX() << ", "<< pElm1->getY() << ", " << pElm1->getZ() << ") (" << pElm2->getX() << ", "<< pElm2->getY() << ", " << pElm2->getZ() << ")\n";  
  float coveredArea = coveredCrossArea(pElm1, pElm2);
  //cout << "covered area: " << coveredArea << "\n";
  float overlappedLength = pElm1->overlappedLength(pElm2);
  //cout << "overlapped length: " << overlappedLength << "\n";
  return coveredArea*overlappedLength;
}


/**
*
*
*/
float Aggregate::computeOverlappedVolume(float /*xPos*/, float yPos, float zPos, DLAElement* pElm1, DLAElement* pElm2){
  ////cout << "(" << pElm1->getX() << ", "<< pElm1->getY() << ", " << pElm1->getZ() << ") (" << pElm2->getX() << ", "<< pElm2->getY() << ", " << pElm2->getZ() << ")\n";  
  //float coveredArea = coveredCrossArea(yPos, zPos, pElm1, pElm2);
  Vector2D<float> vFirst(pElm1->getY()-yPos, pElm1->getZ()-zPos);
  Vector2D<float> vSecond(pElm2->getY()-yPos, pElm2->getZ()-zPos);  
  //float coveredAngle = vFirst.angleTo(vSecond);
  float coveredArea = fabs(vFirst.dotProduct(vSecond))/2.0f;
  //cout << "covered area: " << coveredArea << "\n";
  
  //float overlappedLength = min( min(pElm1->overlappedLength(xPos), pElm2->overlappedLength(xPos))
  //		  					  , min(pElm1->overlappedLength(xPos), pElm1->overlappedLength(pElm2)));
  //cout << "overlapped length: " << overlappedLength << "\n";
  
  //in case voerlappedLength=0
  //overlappedLength+=1.0f;

  //return coveredAngle*overlappedLength;
  return coveredArea;
}

/**
*
*/
float Aggregate::computeNeighbourInfluencingFactor(float xPos, float yPos, float zPos, float distThreshold, float infCoefficient, float elmLength){

  list<DLAElement* > infList;
  getInfluencingNeighbours(xPos, yPos, zPos, distThreshold, infList);
  /*
  if(!infList.empty()){
    //cout << "Element : " << pElm->getElementId() << endl;
    cout << "Influenced by : " << endl;
    list<DLAElement* >::iterator it;
    for(it=infList.begin(); it!=infList.end(); it++){
      cout << (*it)->getElementId() << "-";
    }
    cout << endl;
  }
  */
  if(infList.empty()){
    return 0.0f;
  }
  cout << "Number of influencing neighbours: " << infList.size() << endl;
  
  float infFactor = 0.0f;
  list<DLAElement* >::iterator itr;
  float overLength;
  for(itr=infList.begin(); itr != infList.end(); itr++){
  	float centerLength = fabs(xPos - (*itr)->getX());  
  	if(centerLength >= elmLength){
	    overLength = 0.0f;
	  }
	  else{
	    overLength = elmLength - centerLength;	
	  }
    float yzDist = sqrt((yPos-(*itr)->getY())*(yPos-(*itr)->getY()) + (zPos-(*itr)->getZ())*(zPos-(*itr)->getZ()));
    //cout << "--" << yzDist; 
    infFactor += infCoefficient * overLength / yzDist;
  }
  //cout << endl;
  return infFactor;
}

void Aggregate::getInfluencingNeighbours(float xPos, float yPos, float zPos, float distThreshold, list<DLAElement* >& influencingList){
  if(!influencingList.empty()){
    influencingList.clear();
  }
  list<DLAElement* > adjList;
  getCrossElements(xPos, adjList);
  list<DLAElement* >::iterator itr;
  
  float tDist;
  cout.setf(ios::fixed,ios::floatfield);
  cout.width(12);  
  cout.precision(8);
  
  for(itr=adjList.begin(); itr!=adjList.end(); itr++){
    tDist = sqrt((yPos-(*itr)->getY())*(yPos-(*itr)->getY()) + (zPos-(*itr)->getZ())*(zPos-(*itr)->getZ()));
    if( (tDist<=distThreshold) && (fabs(xPos-(*itr)->getX())<context->getElementWidth())){
      //cout << tDist << endl;
      influencingList.push_back(*itr);
    }
  }
}

void Aggregate::getInfluencingNeighbours(DLAElement* pElm, float distThreshold, list<DLAElement* >& influencingList){
  if(!influencingList.empty()){
    influencingList.clear();
  }
  list<DLAElement* > adjList;
  getCrossElements(pElm->getX(), adjList);
  list<DLAElement* >::iterator itr;
  
  float tDist;
  cout.setf(ios::fixed,ios::floatfield);
  cout.width(12);  
  cout.precision(8);
  
  for(itr=adjList.begin(); itr!=adjList.end(); itr++){
	if(pElm->getElementId()!=(*itr)->getElementId()){
		tDist = sqrt((pElm->getY()-(*itr)->getY())*(pElm->getY()-(*itr)->getY()) + (pElm->getZ()-(*itr)->getZ())*(pElm->getZ()-(*itr)->getZ()));
		if( (tDist<=distThreshold) && (fabs(pElm->getX()-(*itr)->getX())<context->getElementWidth())){
			//cout << tDist << endl;
			influencingList.push_back(*itr);
		}
	}
  }
}

/**
*
*
*/
bool Aggregate::getCrossElements(float xPos, list<DLAElement*>& adjList){
  if(!adjList.empty()){
    adjList.clear();
  }
  
  list<DLAElement* > tmpList;
  list<DLAElement* >::iterator itr;
  
  getElementsAtXFromMap(xPos, adjList);
  
  getElementsAtXFromMap(xPos - context->getElementWidth()/2.0f + EPSILON, tmpList);
  for(itr=tmpList.begin(); itr!=tmpList.end(); itr++){
  	list<DLAElement*>::iterator fitr = find_if(adjList.begin(), adjList.end(), bind2nd(ElementIdEqual(), (*itr)->getElementId()));
	  if(fitr==adjList.end()){
	    adjList.push_back(*itr);
    }
  }
  
  if(!tmpList.empty()){
    tmpList.clear();
  }
  
  getElementsAtXFromMap(xPos + context->getElementWidth()/2.0f - EPSILON, tmpList);
  for(itr=tmpList.begin(); itr!=tmpList.end(); itr++){
  	list<DLAElement*>::iterator fitr = find_if(adjList.begin(), adjList.end(), bind2nd(ElementIdEqual(), (*itr)->getElementId()));
	  if(fitr==adjList.end()){
	    adjList.push_back(*itr);
    }
  }
    
  if(adjList.empty()){
    return false;
  }
  
  return true;
}


void Aggregate::diffuseBasedOnEnergy3(DLAElement* existed, DLAElement* coming){
	//float lAdjustAngle = 0.0f; //=std::numeric_limits<float>::max();
	//float rAdjustAngle = 0.0f; //=std::numeric_limits<float>::max();
	//bool  lAdjustable=false;
	//bool  rAdjustable=false;
    list<DLAElement*> lCollideTarget;
    list<DLAElement*> rCollideTarget;
    DLAElement* pOptimalFixed = existed;
    float       fOptimalX     = coming->getX();    
    float       fOptimalY     = coming->getY();
    float       fOptimalZ     = coming->getZ();
    
    int tryTimes = context->getDiffusionTryTimes();
    float distThreshold = context->getDiffusionDistanceThreshold();
    DLAElement* pCurFixed    = existed;
    float       fCurY        = coming->getY();
    float       fCurZ        = coming->getZ();

	float       fOptimalAffinity = computeAffinityWeight2( coming->getX()
													, fCurY
													, fCurZ
													, pCurFixed
													, coming         //NULL
													, distThreshold
													);
    float       fCurAffinity = fOptimalAffinity;
    
    for(int i = 0; i < tryTimes; i++){
    	fCurAffinity = tryOneTime(distThreshold
    							, fOptimalX
    							, fCurY
    							, fCurZ
    							, fCurAffinity
    							, pCurFixed
    							, coming
    							);
        //cout << "(" << fCurY << ", " << fCurZ << ")" << "\n";
    	if(fOptimalAffinity - fCurAffinity < 0.0f){
    		fOptimalAffinity = fCurAffinity;
    		pOptimalFixed = pCurFixed;
    		fOptimalY     = fCurY;
    		fOptimalZ     = fCurZ;
    		//don't forget to delete the line below
    	    //coming->setPos(fOptimalX, fOptimalY, fOptimalZ);    		
    	}    	
    }
    
    coming->setPos(fOptimalX, fOptimalY, fOptimalZ);
	float cAngle, tAngle;
	cAngle = pOptimalFixed->getYZAngle(coming);
	pOptimalFixed->addNeighbour(cAngle, coming);
	tAngle = coming->getYZAngle(pOptimalFixed);
	coming->addNeighbour(tAngle, pOptimalFixed);		
}

//Random try without considering KT
float Aggregate::tryOneTimeRandom( float //distThreshold
						   , float xPos
						   , float& yPos
						   , float& zPos
						   , float  existingAffinity
						   , DLAElement*& pFixed
						   , DLAElement*& //pComing
						   ){
	//float testAngle = randomAcuteAngle();
	float testAngle = randomSmallAngle();

	//float testAngle = randomSmallAngle(context->getDiffusionKTConstant());
	float stepSize  = context->getDiffusionStep();
	float leftYPos = yPos;
	float leftZPos = zPos;
	bool lChoice = false;
	bool rChoice = false;
	
	bool  leftTest = false;
	bool  rightTest = false;
	
	float  rightAffinity=existingAffinity,  leftAffinity=existingAffinity;
	//float  leftProbability=0.0f, rightProbability=0.0f, tp=0.0f;
	
	DLAElement*    pLeftFixed = pFixed;
	leftTest = angleTest( testAngle
			 , stepSize
			 , xPos
			 , leftYPos
			 , leftZPos
			 , pLeftFixed
			 );
	
    if(leftTest){
    	lChoice = true;  	
    }
    
	float rightYPos = yPos;
	float rightZPos = zPos;
	DLAElement*    pRightFixed = pFixed;
	rightTest =angleTest( testAngle
			 , -stepSize
			 , xPos
			 , rightYPos
			 , rightZPos
			 , pRightFixed
			 );

	if(rightTest){
		rChoice = true;
	}
	
	if(lChoice && rChoice){
			if(rand()%2==1){
				rChoice = false;
			}
			else{
				lChoice = false;
			}
	}
	
	if(lChoice){
		yPos = leftYPos;
		zPos = leftZPos;
		pFixed = pLeftFixed;
		//if(leftProbability!=0.0f){
		//cout << "Left: " << leftProbability << ", " << leftAffinity - existingAffinity <<  "\n";
		//}
		//cout <<
		//cout << testAngle << "--left\n";
		return leftAffinity;
	}
	else if(rChoice){
		yPos = rightYPos;
		zPos = rightZPos;
		pFixed = pRightFixed;
		//if(rightProbability!=0.0f){
		//cout << "Right: " << rightProbability << ", " << rightAffinity  - existingAffinity << "\n";
		//}
		//cout << testAngle << "--right\n";
		return rightAffinity;		
	}
	
	return existingAffinity;
}


float Aggregate::tryOneTime( float distThreshold
						   , float xPos
						   , float& yPos
						   , float& zPos
						   , float  existingAffinity
						   , DLAElement*& pFixed
						   , DLAElement*& pComing
						   ){
	//float testAngle = randomAcuteAngle();
	float testAngle = randomSmallAngle();	
	//float testAngle = randomSmallAngle(context->getDiffusionKTConstant());
	float stepSize  = context->getDiffusionStep();
	float leftYPos = yPos;
	float leftZPos = zPos;
	bool lChoice = false;
	bool rChoice = false;
	
	bool  leftTest = false;
	bool  rightTest = false;
	
	float  rightAffinity=existingAffinity,  leftAffinity=existingAffinity;
	float  leftProbability=0.0f, rightProbability=0.0f, tp=0.0f;
	
	DLAElement*    pLeftFixed = pFixed;
	leftTest = angleTest( testAngle
			 , stepSize
			 , xPos
			 , leftYPos
			 , leftZPos
			 , pLeftFixed
			 );
	
    if(leftTest){
    	leftAffinity = computeAffinityWeight2( xPos
												, leftYPos
												, leftZPos
												, pLeftFixed
												, pComing //NULL
												, distThreshold
												);

    	if(leftAffinity - existingAffinity >= 0.0f){
    		lChoice = true;
    	}
    	
    	else{
    		leftProbability = diffusionProbability(existingAffinity-leftAffinity, context->getDiffusionKTConstant()); //4.0f);
    		//cout << "left probability: " << leftProbability << "\n";		
    		tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
    		//cout << "tp: " << tp << "\n";
    		if(tp < leftProbability){
    			lChoice = true;
    		}		
    	}
    	
    }
    
	float rightYPos = yPos;
	float rightZPos = zPos;
	DLAElement*    pRightFixed = pFixed;
	rightTest =angleTest( testAngle
			 , -stepSize
			 , xPos
			 , rightYPos
			 , rightZPos
			 , pRightFixed
			 );

	if(rightTest){
		rightAffinity = computeAffinityWeight2( xPos
												, rightYPos
												, rightZPos
												, pRightFixed
												, pComing //NULL
												, distThreshold
												);
	
	

	
//	leftProbability = diffusionProbability(fabs(leftAffinity - existingAffinity), context->getDiffusionKTConstant()); //4.0f);
//    //cout << "left probability: " << lProbability << "\n";		
//	tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
//	//cout << "tp: " << tp << "\n";
//	if(tp < leftProbability){
//		lChoice = true;
//	}
	
//	rightProbability = diffusionProbability(fabs(rightAffinity - existingAffinity), context->getDiffusionKTConstant()); //4.0f);
//    //cout << "left probability: " << lProbability << "\n";		
//	tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
//	//cout << "tp: " << tp << "\n";
//	if(tp < rightProbability){
//		rChoice = true;
//	}		
	
	
	//float probability, tp;
	
		if(rightAffinity - existingAffinity >= 0.0f){
			rChoice = true;
		}
		
		else{
			rightProbability = diffusionProbability(existingAffinity-rightAffinity, context->getDiffusionKTConstant()); //4.0f);
			//cout << "Right probability: " << rightProbability << "\n";		
			tp = ((float)rand())/(RAND_MAX + 1.0f); //rand(0.0f, 1.0f);
			//cout << "tp: " << tp << "\n";
			if(tp < rightProbability){
				rChoice = true;
			}		
		}
		
	}
	
	if(lChoice && rChoice){
		
		if(leftProbability>rightProbability){
			rChoice = false;
		}
		else if(leftProbability<rightProbability){
			lChoice = false;
		}
		else{
			
			if(rand()%2==1){
				rChoice = false;
			}
			else{
				lChoice = false;
			}
		}
	}
	
	if(lChoice){
		yPos = leftYPos;
		zPos = leftZPos;
		pFixed = pLeftFixed;
		//if(leftProbability!=0.0f){
		//cout << "Left: " << leftProbability << ", " << leftAffinity - existingAffinity <<  "\n";
		//}
		return leftAffinity;
	}
	else if(rChoice){
		yPos = rightYPos;
		zPos = rightZPos;
		pFixed = pRightFixed;
		//if(rightProbability!=0.0f){
		//cout << "Right: " << rightProbability << ", " << rightAffinity  - existingAffinity << "\n";
		//}
		return rightAffinity;		
	}
	
	return existingAffinity;
}

bool Aggregate::angleTest( float  angleScope
					     , float  stepSize
						 , float  xPos
						 , float& yPos
						 , float& zPos
						 , DLAElement*& pTheFixed
                         ){
  //if(angleScope < stepSize){
	//  return false;
  //}
  /*
  if(angleScope <= fabs(stepSize)){
	  //pTheFixed = pCurFixed;
	  //yPos = newY;
	  //zPos = newZ;
	  return false;
  }
  */
	
  DLAElement* pCurFixed = pTheFixed;
  float       fCurAngleScope = angleScope;
  float tAngle = pCurFixed->getYZAngle(yPos, zPos);
  float newAngle;
  if(angleScope < fabs(stepSize)){
	  if(stepSize>0.0f){
		  newAngle = tAngle + angleScope;
	  }
	  else{
		  newAngle = tAngle - angleScope;
	  }
  }
  else{
	  newAngle = tAngle + stepSize;
  }
  newAngle = normalAngle(newAngle);
  float elementHeight = context->getElementHeight();  
  float newY  = pCurFixed->getY() + elementHeight * sin(newAngle);
  float newZ  = pCurFixed->getZ() + elementHeight * cos(newAngle);
  
  
  list<DLAElement* > adjList;
  list<DLAElement* > collideTarget;
  float testAngle = fabs(stepSize);  
  getAdjacentList(xPos, adjList);
  collideTest(pCurFixed->getElementId(), xPos, newY, newZ, adjList, collideTarget);
  while( testAngle +fabs(stepSize) <= fCurAngleScope){
	    if(collideTarget.size()==0){
	    	testAngle += fabs(stepSize);
	    	newAngle   =  normalAngle(newAngle+stepSize);
	    }
	    else{
	    	
	    	if(collideTarget.size()>1){
	    		pCurFixed = getElementWithMinimumAngle( pCurFixed, newY, newZ, collideTarget);
	    	}
	    	else{
	    		pCurFixed = collideTarget.front();
	    	}
	    	collideTarget.clear();	    	
	    	fCurAngleScope -= testAngle;
	    	testAngle = fabs(stepSize); //? or 0	    	
	    	newAngle = normalAngle(pCurFixed->getYZAngle(newY, newZ)+stepSize);
	    	
	    	//break;
	    }
    	newY = pCurFixed->getY() + elementHeight * sin(newAngle);
    	newZ = pCurFixed->getZ() + elementHeight * cos(newAngle); 	
    	collideTest(pCurFixed->getElementId(), xPos, newY, newZ, adjList, collideTarget);	    
  }
  
  pTheFixed = pCurFixed;
  yPos = newY;
  zPos = newZ;
  return true;
}

void Aggregate::getConvexHullPoints(float xPos, list<DLAElement*>& hullPointsList){
	list<DLAElement*> crossPointsList;
	getElementsAtXFromMap(xPos, crossPointsList);
	if(crossPointsList.size() <= 3){ //The number of elements in the cross section less than 3.
		hullPointsList = crossPointsList;
		return;
	}
	//Find an element with minimum y coordinate
	list<DLAElement*>::iterator itr;
	list<DLAElement*>::iterator minItr;
	minItr = crossPointsList.begin();
	for(itr=crossPointsList.begin(); itr!=crossPointsList.end(); itr++){
		if((*itr)->getY() < (*minItr)->getY()){
			minItr = itr;
		}
		else if((*itr)->getY() == (*minItr)->getY()){
			if((*itr)->getZ() < (*minItr)->getZ()){
				minItr = itr;
			}
		}
	}
	
	//Sort the points based on the vector angle from the point with minimum y coordinate
	map<float, Point2D<float> > polarAngleMap;
	polarAngleMap[0.0f]=Point2D<float>((*minItr)->getZ(), (*minItr)->getY(), (*minItr)->getElementId());
	for(itr=crossPointsList.begin(); itr!=crossPointsList.end(); itr++){
		if(*itr != *minItr){
			Vector2D<float> tmpVector((*itr)->getZ()-(*minItr)->getZ(), (*itr)->getY()-(*minItr)->getY());
			float angle = tmpVector.angle();
			if(polarAngleMap.find(angle)!=polarAngleMap.end()){				
				Vector2D<float> oldVector(polarAngleMap[angle].x()-(*minItr)->getZ(), polarAngleMap[angle].y()-(*minItr)->getY());				 
				if(tmpVector.length() > oldVector.length()){
					polarAngleMap[angle]=Point2D<float>((*itr)->getZ(), (*itr)->getY(), (*itr)->getElementId());
				}
			}
			else{
				//polarAngleMap[angle]=tmpVector;
				polarAngleMap[angle]=Point2D<float>((*itr)->getZ(), (*itr)->getY(), (*itr)->getElementId());				
			}
		}
	}
	
	map<float, Point2D<float> >::iterator mapItr;
	stack<Point2D<float> > hullPointsStack;
	Point2D<float> topPoint;
	Point2D<float> nextToTopPoint;
	mapItr = polarAngleMap.begin();
	//cout << "test1" << endl;
	hullPointsStack.push((*mapItr).second);
	mapItr++;
	hullPointsStack.push((*mapItr).second);
	mapItr++;
	hullPointsStack.push((*mapItr).second);
	//cout << "test2" << endl;
	mapItr++;
	while(mapItr!=polarAngleMap.end()){
		for(;;){
			if(hullPointsStack.size()>1){
				topPoint = hullPointsStack.top();
				hullPointsStack.pop(); // Pop the top to get next to top
				nextToTopPoint = hullPointsStack.top();
				hullPointsStack.push(topPoint); // Push the top back after getting the next to top				
			}
			else{
				break;
			}
			Vector2D<float> firstVector(nextToTopPoint, topPoint);
			Vector2D<float> secondVector(nextToTopPoint, (*mapItr).second);
			//cout << "test4" << endl;
			if(firstVector.crossProduct(secondVector)<=0.0f){ //nonleft turn
				hullPointsStack.pop();
			}
			else{
				break;
			}
		}
		hullPointsStack.push((*mapItr).second);
		mapItr++;
	}
	//cout << "test5" << endl;
	while(!hullPointsStack.empty()){
		hullPointsList.push_front(mapById[hullPointsStack.top().id()]);
		hullPointsStack.pop();
	}
}

float Aggregate::computeHullArea(const list<DLAElement*>& hullPointsList){
	if(hullPointsList.empty()){
		return 0.0f;
	}
	else if(hullPointsList.size()==1){
		return 1.0f;
	}
	else if(hullPointsList.size()==2){
		return 2.0f;
	}
	else{
		float hullArea = 0.0f;
		list<DLAElement*>::const_iterator itr;

		itr = hullPointsList.begin();
		Point2D<float> basePoint((*itr)->getZ(), (*itr)->getY());
		itr++;
		Point2D<float> firstPoint((*itr)->getZ(), (*itr)->getY());
		itr++;
		Point2D<float> secondPoint(0.0f, 0.0f);
		
		while(itr!=hullPointsList.end()){
			secondPoint.setPosition((*itr)->getZ(), (*itr)->getY());
			Vector2D<float> edgeOne(basePoint, firstPoint);
			Vector2D<float> edgeTwo(basePoint, secondPoint);
			hullArea += fabs(edgeOne.crossProduct(edgeTwo))/2.0f;
			firstPoint = secondPoint;
			itr++;
		}		
		return hullArea;
	}	
}

float Aggregate::computeHullCircumference(const list<DLAElement*>& hullPointsList){
	if(hullPointsList.empty()){
		return 0.0f;
	}
	else if(hullPointsList.size()==1){
		return 1.0f;
	}
	else if(hullPointsList.size()==2){
		return 2.0f;
	}
	else{
		float hullCircumference = 0.0f;
		list<DLAElement*>::const_iterator itr;
		itr = hullPointsList.begin();
		if(itr!=hullPointsList.end()){
			Point2D<float> pointBegin((*itr)->getZ(), (*itr)->getY());
			itr++;
			Point2D<float> pointEnd(0.0f, 0.0f);
			while(itr!=hullPointsList.end()){
				pointEnd.setPosition((*itr)->getZ(), (*itr)->getY());
				Vector2D<float> edgeVector(pointBegin, pointEnd);
				hullCircumference += edgeVector.length();
				pointBegin = pointEnd;
				itr++;
			}
		}
		return hullCircumference;
	}	
}
