#ifndef AGGREGATE_H
#define AGGREGATE_H

#include "DLAElement.h"
#include "SimuContext.h"
#include <list>
#include <map>

using namespace std;

class CircleGridMap;

struct TmpNeighbour{
	float angle;
	int   neighbourId;
};

class Aggregate 
{
  public:
    Aggregate();
    ~Aggregate();
    bool empty(){return dlaElements.empty();}
    void clear(){dlaElements.clear(); mapById.clear(); elementMap.clear();}
    void  addElement(DLAElement * p);
    DLAElement*& operator[](int eId);
    //const DLAElement*& operator[](int eId) const;
    int   getNumberOfElements() const;
    int   getSecondsSpent() const {return secondsSpent;};
    void  setSecondsSpent(int value){secondsSpent=value;}
    void  save(ostream& outFile);
    const list<DLAElement*>* getElements() const;
    list<DLAElement*>* getElements();    
    map<float, list<DLAElement*> >* getElementMap();
    float percentageOfOverlappedDomain(DLAElement* coming, DLAElement* target, char domainType);
    float overlappedDomainLength(DLAElement* coming, DLAElement* target, char domainType);    
    float percentageOfOverlapped(DLAElement* coming, DLAElement* target);
    bool  toCollideWith(DLAElement* p, list<DLAElement*>& elements, list<DLAElement*>& collideTarget);
    int   toBeCollided(DLAElement* p, list<DLAElement*>& collideTarget);
    void  collideTest(int elmId, float nx, float ny, float nz, list<DLAElement*>& elements, list<DLAElement*>& collideTarget);
    //int   neighbourTest(int elmId, float nx, float ny, float nz, list<DLAElement*>& collideTarget);
    DLAElement* getUniqueTarget(DLAElement* p, list<DLAElement*>& collideTarget);
    DLAElement* getUniqueTargetYZ(DLAElement* p, list<DLAElement*>& collideTarget);
    DLAElement* getElementWithMinimumAngle(DLAElement* pFixed, DLAElement* pComing, const list<DLAElement* >& tList);
    DLAElement* getElementWithMinimumAngle( DLAElement* pFixed 
    									 , float yPos
    									 , float zPos
    									 , const list<DLAElement* >& tList
    									 );
    
    bool adjustTest(DLAElement* pFixed
                   , DLAElement* pComing
                   , float stepSize
                   , float& adjustAngle
                   , float& adjustDistance
                   , list<DLAElement*>& collideTarget
                   );
   /**
    * Diffuse based on the change of the free energy.
    */                   
   void diffuseBasedOnEnergy(DLAElement* existed, DLAElement* coming);   
   /**
    * Diffuse based on the change of the free energy.
    * In the previous version, the coming particle attach to the left or right
    * of the existed one is determined by the probability of covered cross surface
    * areas.
    * In this version, the coming one will attach to the left or right which has bigger
    * covered area. 
    * factor: 1 - diffusion based on unfavorable factors
    * 		  2 - diffusion based on favorable factors
    */
   void diffuseBasedOnEnergy2(DLAElement* existed, DLAElement* coming, int factor);
   void diffuseBasedOnEnergy3(DLAElement* existed, DLAElement* coming);
   float tryOneTimeRandom( float distThreshold
   				   , float xPos
   				   , float& yPos
   				   , float& zPos
   				   , float  existingAffinity
   				   , DLAElement*& pFixed
   				   , DLAElement*& pComing   				   
   				   );
   
   float tryOneTime( float distThreshold
   				   , float xPos
   				   , float& yPos
   				   , float& zPos
   				   , float  existingAffinity
   				   , DLAElement*& pFixed
   				   , DLAElement*& pComing   				   
   				   );

   bool angleTest( float  angleScope
   					     , float  stepSize
   						 , float  xPos
   						 , float& yPos
   						 , float& zPos
   						 , DLAElement*& pTheFixed
                            );

   void cascadeDiffuse(DLAElement* existed, DLAElement* coming, int times);
   DLAElement* getNextDiffusionTarget(DLAElement* existed, DLAElement* coming);   
   /**
   * Get the maximum angle of the coming particle with two particles on the aggregate
   * to be attached.
   * 
   */
   float coveredCrossAngle(DLAElement* pFixed, DLAElement* pCollided);

   /**
   * Get the maximum area of the coming particle with two particles on the aggregate
   * to be attached.
   * 
   */
   float coveredCrossArea(DLAElement* pFixed, DLAElement* pCollided);   
    
   float coveredCrossArea(float yPos, float zPos, DLAElement* pFixed, DLAElement* pCollided);
   
   /**
   * Collision based adjustment test. The major difference between this function and
   * adjustTest is that this test will stop when the coming spins around 360 degree or
   * it collides with another existing particle in the aggregate. 
   */
   bool adjustCollisionTest(DLAElement* pFixed
                          , DLAElement* pComing
                          , float stepSize
                          , float& adjustAngle
                          , list<DLAElement*>& collideTarget
                          );
    int getAdjacentList(float nx, list<DLAElement*>& adjacentList);   
    bool  isInList(DLAElement* pE, list<DLAElement*>& tarList);
    void  recursiveDiffusion(DLAElement* existed, DLAElement* coming);
    
    float getMinX();
    void setMinX(float value);
    float getMaxX();
    void setMaxX(float value);
    float getMinY();
    void setMinY(float value);
    float getMaxY();
    void setMaxY(float value);
    float getMinZ();
    void setMinZ(float value);
    float getMaxZ();
    void setMaxZ(float value);
    float getWidth();
    float getHeight();
    float getDepth();
    
    /** 
     * Let the coming element circle around the existed particle to find a position,
     * where the coming element's center distance to the coordinate center is minimum.
     * @param existed Pointer to the existed element
     * @param coming  Pointer to the coming particle
     * @return None 
     */
    void  diffuse(DLAElement* existed, DLAElement* coming);
    
    void  getYZ(float x, float* maxy, float* miny, float *maxz, float* minz);
    SimuContext* getContext();
    map<int, DLAElement*>* getIdMap(){return &mapById;}
    DLAElement* getElementById(int id){return mapById[id];};

    // Methods for statistics purposes
    /* Compute the density */
    float computeAverageDensity(float step, float range=1.0f);
    float computeDensityAtX(float x, CircleGridMap& cGrid);
    int  getElementsAtXFromList(float xPos, list<DLAElement*>& elementList, list<DLAElement*>& resultList);
    int  getElementsAtXFromMap(float xPos, list<DLAElement*>& resultList);
    float getAverageRadius(list<DLAElement*>& resultList);
    void  adjustAggregateCenter();
    
    /* Compute the surface area */
    float computeSurfaceArea(float step);

    /* Compute connectivity */
    float computeConnectivity();
    //void computeAllNeighbours();
    //void computeNeighbourElement(DLAElement* element);
    //void computeNeighbourElementList(DLAElement* element, list<DLAElement*>& targetList);
    //int  saveConnectivity(char* fileName);
    /**
     * Collect connectivity information into a map.
     */
    void collectConnectivityInfo(map<int, int>& conInfoMap);
    
    // Compute neighbour with cross-linking contact for each element
    // in the aggregate.
    float computeCrossNeighbours(float threshold);

    // Compute neighbours with certain number of connected cross-linking domains
    // in the aggregate. The number of connectted cross-linking domains is computed
    // using function numberOfOverlappedDomainParts.
    // Parameter: minNumOfLinks - the minimum number of links to be counted as a neighbour
    float computeCrossNeighboursMinLinks(int minNumOfLinks);
    
    /**
     * 
     * Compute number of crosslinks of an aggregate 
     * 
     */
    double computeCrossLinks(int method);
    
    /**
     * domainType: c crosslink, h hydrophobic
     * 
     * return: the average number of overlapped parts for a given domain type in the aggregate
     */
    double computeNumberOfOverlappedDomainPartsWithNeighbours(char domainType, float threshold, int method);
    
    /**
     * 
     * Compute number of crosslinks based on center distance of domain parts 
     * 
     */
    long numberOfPartsCenterDist(DLAElement* coming, DLAElement* target, char domainType, float threshold);
    
    /**
     * Compute the number of overlapped domain parts of two models
     * 
     * 1. ccccccchhhhhhhhcccccccchhhhhhhhhcchhhhcc 
     * 2.     ccccccchhhhhhhhcccccccchhhhhhhhhcchhhhcc
     *        ---    ----    ----    -----
     *         c       h       c        h 
     * So, number of overlapped parts for domain c is 2, and for domain h is also 2
     */
    long numberOfOverlappedDomainParts(DLAElement* coming, DLAElement* target, char domainType);
    long numberOfOverlappedDomainFromParts( char domainType, list<ElementTemplate>& lPart, list<ElementTemplate>& rPart);    
    
    /**
     * Collect information about the particles' distance to the aggregate's center, and
     * save the result into a map. 
     */     
    void collectDistToCenterInfo(map<float, int>& conInfoMap); 
    
    void shaking(int times);
    bool shakingAtSamplingPoint(float samplingPoint);
    void splitNeighbours(DLAElement* pChosen, list<DLAElement*>& lList, list<DLAElement*>& rList);
    DLAElement* findElementById(int id);
    DLAElement* findElementById(const list<DLAElement*>& elmList, int id);

    //Same purpose as adjustCollisionTest, but calls different functions for shaking.
    bool shakingAdjustTest(DLAElement* pFixed
                          , DLAElement* pComing
                          , float stepSize
                          , float& adjustAngle
                          , list<DLAElement*>& collideTarget);
    
    void shakingCollideTest(int elmId, int comingId, float nx, float ny, float nz, list<DLAElement*>& elements, list<DLAElement*>& collideTarget);
    int shakingNeighbourTest(int elmId, int comingId, float nx, float ny, float nz, list<DLAElement*>& collideTarget);
    

    //
    //This works in situations where the coming is not attached to the arrgregate,
    //which means that the coming one is not in the existed one's neighbour list.
    //
    bool getLeftCollidingTargets(  DLAElement* existed
    							, DLAElement* coming
    							, float& lCollidingAngle
    							, list<DLAElement*>& lCollidingList
    							);
    
    //
    //This works in situations where the coming is not attached to the arrgregate,
    //which means that the coming one is not in the existed one's neighbour list.
    //
    bool getRightCollidingTargets(  DLAElement* existed
    							, DLAElement* coming
    							, float& rCollidingAngle
    							, list<DLAElement*>& rCollidingList
    							);
    
    /**
    *return: 1 - to the left
    *        0 - neither left nor right
    *        -1 - to the right
    */
    int attachToLeftOrRight(  DLAElement* existed
    						, list<DLAElement*>& lCollidingTarget
    						, list<DLAElement*>& rCollidingTarget
    						);
    
    /**
     * To continuely move the coming particle around the first contact's left or right
     * based on unfavorable factors
     * 
     * return: 1 - left
     *         0 - not move
     *         -1 - right
     */
    int movedToLeftOrRight( DLAElement* coming
    								 , DLAElement* pCurLeft
    								 , DLAElement* pCurRight
    								 , float distThreshold    								 
    								 , float& nextAngle
    								 , float& nextAffinity
    								 , list<DLAElement*>& nextList
    								 );

    /**
     * To continuely move the coming particle around the first contact's left or right
     * based on favorable factors
     * 
     * return: 1 - left
     *         0 - not move
     *         -1 - right
     */   
    int movedToLeftOrRight2( DLAElement* coming
    					  , DLAElement* pCurLeft
    					  , DLAElement* pCurRight
						  , float distThreshold    					  
    					  , float& lTestAngle
    					  , list<DLAElement*>& lTestList
    					  , float& rTestAngle
    					  , list<DLAElement*>& rTestList
    					  );


  bool findOptimalAngle(DLAElement* fixed, DLAElement* coming, float stepSize, float& adjustAngle);    					  
  bool findOptimalAngle2(DLAElement* fixed, DLAElement* coming, float stepSize, float& adjustAngle);  
  float computeDistanceSum(float posY, float posZ, list<DLAElement* >& tList);
  /**
  *
  *
  */
  float computeAffinityWeight( float xPos
                                      , float yPos
                                      , float zPos
                                      , DLAElement* pFixed
                                      , DLAElement* pTarget
                                      , float distThreshold
                                      , float infCoeff
                                      , float elmLength
                                      );
  /**
  *
  *
  */
  float computeAffinityWeight2( float xPos
                              , float yPos
                              , float zPos
                              , DLAElement* pFixed
                              , DLAElement* pComing
                              , float distThreshold
                              );
  float computeAffinityWeight2( DLAElement* pComing
                              , float distThreshold
                              );
  
  float computeDistanceWeight(float posX, float posY, float posZ);
  
  float computeOverlappedVolume(DLAElement* pElm1, DLAElement* pElm2);
  float computeOverlappedVolume(float xPos, float yPos, float zPos, DLAElement* pElm1, DLAElement* pElm2);  
  float computeNeighbourInfluencingFactor(float xPos, float yPos, float zPos, float distThreshold, float infCoefficient, float elmLength);
  void  getInfluencingNeighbours(float xPos, float yPos, float zPos, float distThreshold, list<DLAElement* >& influencingList);
  void  getInfluencingNeighbours(DLAElement* pElm, float distThreshold, list<DLAElement* >& influencingList);
  bool  getCrossElements(float xPos, list<DLAElement*>& adjList);  
  
  void getConvexHullPoints(float xPos, list<DLAElement*>& hullPointsList);
  float computeHullArea(const list<DLAElement*>& hullPointsList);
  float computeHullCircumference(const list<DLAElement*>& hullPointsList);
  ostream& outputInfluencingMatrix(ostream& ostr);
  
  protected:
    float computeDomainLength(float connectLength, char domainType, list<ElementTemplate>& lPart, list<ElementTemplate>& rPart);
    float overlappedDomainLengthFromParts( char domainType,  list<ElementTemplate>& lPart, list<ElementTemplate>& rPart);
    void  getYZ(list<DLAElement*>& elements, float* maxy, float* miny, float *maxz, float* minz);
    //float diagonalLength;
    int   numberOfElements;
    // Vector holding the particles in the aggregate
    list<DLAElement*> dlaElements;
    // Indexing map
    map<float, list<DLAElement*> > elementMap;
    // Map from id to element
    map<int, DLAElement*> mapById;

    float minX;
    float minY;
    float minZ;
    float maxX;
    float maxY;
    float maxZ;
    float width;
    float height;
    float depth;
    SimuContext* context;
    int   secondsSpent; //The simulation time (seconds) spent to generate the aggregate. 
    
  private:
	float randomSamplingPoint(float sampleLength);	  
    Aggregate(const Aggregate& src);
    Aggregate& operator=(const Aggregate& rhs);
};

#endif
