#include "DLAElement.h"
#include "SimuContext.h"
#include "utils.h"

#include <exception>
#include <algorithm>
#include <functional>
#include <string>
#include <sstream>
#include <cmath>
#include <iostream>


using namespace std;

ostream& operator<<(ostream& ostr, const DLAElement& dlaElement){
	ostr << "(" << dlaElement.getX() << ", " << dlaElement.getY() << ", " << dlaElement.getZ() << ")";
	return ostr;
}

ostream& operator<<(ostream& ostr, const DLAElement* dlaElement){
	ostr << "(" << dlaElement->getX() << ", " << dlaElement->getY() << ", " << dlaElement->getZ() << ")";
	return ostr;
}

//
//Functor for find_if searching for an element with a given id in the aggregate 
//
class NeighbourIdEqual : public binary_function<DLANeighbour, int, bool>{
public:
	bool operator()(const DLANeighbour& nb, int id) const{
		return ((nb.neighbour())->getElementId()==id);
	}
};

//
// Implementation of DLAElement
//

int DLAElement::idSerialNumber = 0;

DLAElement::DLAElement()
  : x(0.0f), y(0.0f), z(0.0f), diagonalLength(0.0f), active(true),valid(true),inCollidingArea(false), orientation(0)
{
  SimuContext* context = SimuContext::getInstance();
  speed = context->getElementSpeed();
  vx = randomSpeed(speed);
  vy = randomSpeed(speed);
  vz = randomSpeed(speed);
  elementWidth  = context->getElementWidth();
  elementHeight = context->getElementHeight();
  elementDepth  = context->getElementDepth();

  elementId = idSerialNumber++;
}

DLAElement::~DLAElement()
{
  //list<DLANeighbour*>::iterator itr = neighbourList.begin();
  //while(itr!=neighbourList.end()){
  //  delete *itr;
  //  itr++;
  //}
}

bool DLAElement::isActive() const{
  return active;
}

void DLAElement::setActive(bool active){
  this->active = active;
}

bool  DLAElement::isValid() const{
  return valid;
}

void DLAElement::setInCollidingArea(bool inArea){
  inCollidingArea = inArea;
}

void DLAElement::stop(){
  active = false;
}

float DLAElement::getSpeed() const{
  return speed;
}

void DLAElement::setSpeed(float speed){
  this->speed = speed;
  vx = randomSpeed(speed);
  vy = randomSpeed(speed);
  vz = randomSpeed(speed);
}

float DLAElement::getX() const{
  return x;
}
void DLAElement::setX(float value){
	x = value;
}

float DLAElement::getY() const{
  return y;
}
void DLAElement::setY(float value){
	y = value;
}

float DLAElement::getZ() const{
  return z;
}
void DLAElement::setZ(float value){
	z = value;
}

void DLAElement::setPos(float x, float y, float z){
  this->x = x;
  this->y = y;
  this->z = z;
}

float DLAElement::distanceTo(float xt, float yt, float zt){
  return sqrt((x-xt)*(x-xt) + (y-yt)*(y-yt) + (z-zt)*(z-zt));
}

float DLAElement::centerDistance(DLAElement* pE) const{
  return sqrt((x-pE->x)*(x-pE->x) + (y-pE->y)*(y-pE->y) + (z-pE->z)*(z-pE->z));
}

float DLAElement::getInitDistanceToCenter(){
  return initDistanceToCenter;
}

void DLAElement::setInitDistanceToCenter(float dist){
  initDistanceToCenter = dist;
}

void DLAElement::setElementId(int id){
  elementId = id;
}

int DLAElement::getElementId() const{
  return elementId;
}
int DLAElement::getElementId(){
  return elementId;
}

void DLAElement::setOrientation(int orient){
  orientation = orient;
}

int DLAElement::getOrientation() const{
  return orientation;
}
int DLAElement::getOrientation(){
  return orientation;
}

void DLAElement::addNeighbour(float angle, DLAElement* neighbour){
  DLANeighbour pN(angle, neighbour);
  neighbourList.push_back(pN);
}

//void DLAElement::addNeighbour(float angle, int neighbourId){
//	DLANeighbour neighbour;
//	neighbour.angle = angle;
//	neighbour.id    = neighbourId;
//	neighbourList.push_back(neighbour);
//}

list<DLANeighbour>& DLAElement::getNeighbourList(){
  return neighbourList;
}

int  DLAElement::getNumberOfNeighbours(){
	return neighbourList.size();
}

//Methods for cross neighbour operations
void DLAElement::addCrossNeighbour(DLAElement* cNeighbour){
	crossNeighbours.push_back(cNeighbour);
}

vector<DLAElement*>& DLAElement::getCrossNeighbours(){
	return crossNeighbours;
}

int  DLAElement::getNumberOfCrossNeighbours(){
	return crossNeighbours.size();
}


/**
 * Get the center angle with another particle
 * @param target The particle to which the angle will be computed
 * @return The angle in radian representation
 */
float DLAElement::getYZAngle(DLAElement* target){
  float y1 = target->getY();
  float z1 = target->getZ();

  float yy = y1-y;
  float zz = z1-z;

  float slopeLength = sqrt(yy*yy+zz*zz);
  
  float tAngle = acos(zz/slopeLength);

  if( (zz >= 0.0f && yy < 0.0f ) || ( zz < 0.0f && yy < 0.0f )){
    tAngle = GL_PI*2.0f - tAngle;
  }
  
  return tAngle;
}

float DLAElement::getYZAngle(float fromY, float fromZ){
  //float y1 = target->getY();
  //float z1 = target->getZ();

  //float yy = y1-y;
  //float zz = z1-z;
  float yy = fromY-y;
  float zz = fromZ-z;

  
  float slopeLength = sqrt(yy*yy+zz*zz);
  
  float tAngle = acos(zz/slopeLength);

  if( (zz >= 0.0f && yy < 0.0f ) || ( zz < 0.0f && yy < 0.0f )){
    tAngle = GL_PI*2.0f - tAngle;
  }
  
  return tAngle;
}

float DLAElement::getYZDistance(DLAElement* target){
  //float x1 = coming->getX();
  float y1 = y;
  float z1 = z;

  //float x2 = target->getX();
  float y2 = target->getY();
  float z2 = target->getZ();

  float yy = y1-y2;
  float zz = z1-z2;

  float distance = sqrt(yy*yy + zz*zz);
  return distance;
}

float DLAElement::overlappedLength(DLAElement* target){
	SimuContext* context = SimuContext::getInstance();
	float centerLength = fabs(x - target->getX());  
	float elementWidth = context->getElementWidth();
	//cout<< "Center Length: " << centerLength << endl;
	if(centerLength >= elementWidth){
	  return 0.0f;
	}
	return (elementWidth - centerLength);	
}

float DLAElement::overlappedLength(float xPos){
	SimuContext* context = SimuContext::getInstance();
	float centerLength = fabs(x - xPos);  
	float elementWidth = context->getElementWidth();
	//cout<< "Center Length: " << centerLength << endl;
	if(centerLength >= elementWidth){
	  return 0.0f;
	}
	return (elementWidth - centerLength);	
}

bool DLAElement::isNeighbour(DLAElement* target){
  if(  fabs(x-target->getX()) <= elementWidth
    && fabs(getYZDistance(target) - elementHeight) <= 0.5){
    return true;
  }
  return false;
}

bool DLAElement::inNeighbourList(DLAElement* target){
  int eId = target->getElementId();
  list<DLANeighbour>::iterator itr = neighbourList.begin();
  while(itr!=neighbourList.end()){
    if((*itr).neighbour()->getElementId()==eId){
      return true;
    }
    itr ++;
  }
  return false;
}

//bool DLAElement::equalId(int id){
//	if(elementId == id){
//		return true;
//	}
//	return false;
//}

DLAElement* DLAElement::findNeighbourById(int id){
	list<DLANeighbour>::iterator itr = find_if(neighbourList.begin(), neighbourList.end(), bind2nd(NeighbourIdEqual(), id));
	if(itr!=neighbourList.end()){
		return (*itr).neighbour();
	}
	return NULL;
	
}

bool  DLAElement::removeNeighbourById(int id){
	list<DLANeighbour>::iterator itr = find_if(neighbourList.begin(), neighbourList.end(), bind2nd(NeighbourIdEqual(), id));
	if(itr!=neighbourList.end()){
		neighbourList.erase(itr);
		return true;
	}
	return false;
}

//Detach an element from all its neighbours, and clear the element's neighbour list.
void  DLAElement::detachFromNeighbours(){
	list<DLANeighbour>::iterator itr;
	for(itr=neighbourList.begin(); itr!=neighbourList.end(); itr++){
		if(!((*itr).neighbour()->removeNeighbourById(elementId))){
			ostringstream errInfo;
			errInfo << "Error from detachFromNeighbours\n";
			errInfo << "Element " << elementId << " is not in the neighbour list of its neighbour " << (*itr).neighbour()->getElementId() << "\n";
			throw runtime_error(errInfo.str());
			//cout << errInfo.str() << endl;
		}
	}
	neighbourList.clear();
}

float DLAElement::getDiagonalLength(){ return diagonalLength;}


void  DLAElement::save(ostream& outFile){
  outFile << "<element id=\"" << elementId << "\" x=\"" << x << "\" y=\"" << y << "\" z=\"" << z << "\" orientation=\"" << orientation << "\">" << endl;
 
  list<DLANeighbour>::iterator itr = neighbourList.begin();
  
  while( itr!=neighbourList.end()){
    outFile << "  <neighbour angle=\"" << (*itr).angle() << "\" id=\"" << ((*itr).neighbour())->getElementId() << "\"/>" << endl;
    itr++;
  }
  outFile << "</element>" << endl;
}

DLA3DElement::DLA3DElement()
{
}

DLA3DElement::~DLA3DElement(){}

void DLA3DElement::walk(float deltaTime){
  if(active){
    x += vx*deltaTime;
    y += vy*deltaTime;
    z += vz*deltaTime;

    vx = randomSpeed(speed);
    vy = randomSpeed(speed);
    vz = randomSpeed(speed);

/*  //To be explored   
    vx += randomSpeed(speed)*0.2f;
    vy += randomSpeed(speed)*0.2f;
    vz += randomSpeed(speed)*0.2f;
    
    if(speed-fabs(vx)>0.2f){
    	vx = vx/2.0f;
    }
    if(speed-fabs(vy)>0.2f){
    	vy = vy/2.0f;
    }
    if(speed-fabs(vz)>0.2f){
    	vz = vz/2.0f;
    }
 */   
  }
}
