#include "SimuContext.h"
#include <math.h>
#include <iostream>
#include <iomanip>
#include <sys/stat.h> // for stat() function
#include <errno.h>    // for errno variable

SimuContext* SimuContext::simuInstance = NULL;

SimuContext* SimuContext::getInstance(){
	if(simuInstance == NULL){
	  simuInstance = new SimuContext();
	}
	return simuInstance;
}

void SimuContext::destroyCurrentContext(){
	if(simuInstance!=NULL){
		delete simuInstance;
		simuInstance=NULL;
	}
}

SimuContext::SimuContext()
  : numberOfElements(0), daysOfSimuTime(7.0f), templateDistance(0.0f), templateExpandStep(0.0f)
  , templateWidth(0.0f), templateHeight(0.0f), templateDepth(0.0f)
  , elementWidth(0.0f), elementHeight(0.0f), elementDepth(0.0f), elementSpeed(0.0f) 
  , centerAdjustment(0), simuHydrophobicLength(0.0f)
  , diffusionFlag(0), diffusionLevel(0), diffusionStep(0.0f), diffusionKTConstant(0.25f), diffusionTryTimes(0)
  , diffusionDistanceThreshold(1.0f), diffusionInfluenceCoefficient(0.1f)
  , hydrophobicFlag(0), hydroConstraintType(0), hydroConstraintCoefficient(0.0f), hydroConstraintConstant(0.0f)
{
  tmpHead.setSimuLength(0.0f);
  tmpHead.setExonType('c');
}

SimuContext::~SimuContext(){
}

void SimuContext::computeSimuLength(){
  vector<ElementTemplate>::iterator itr;
  itr = elementTemplate.begin();
  float totalLength = 0.0f;
  while(itr != elementTemplate.end()){
    totalLength += (*itr).exonLength();
    //cout << totalLength << endl;
    itr ++;
  }
  //float ratio = totalLength/elementWidth;
  itr = elementTemplate.begin();
  while(itr!=elementTemplate.end()){
    //(*itr).simuLength = elementWidth * (*itr).exonLength/totalLength;
	(*itr).setSimuLength(elementWidth * (*itr).exonLength()/totalLength);
    itr ++;
  }

  simuHydrophobicLength = 0.0f;
  int numberOfCrossDomain = 0;
  float crossDomainLength = 0.0f;
  itr = elementTemplate.begin();
  while(itr!=elementTemplate.end()){
    if((*itr).exonType()=='h'){
      simuHydrophobicLength += (*itr).simuLength();
    }
    else if ((*itr).exonType()=='c'){
    	crossDomainLength+=(*itr).simuLength();
    	numberOfCrossDomain++;
    }
    itr ++;
  }
  if(numberOfCrossDomain>0){
	  averageCrossDomainLength = crossDomainLength/numberOfCrossDomain;
  }
  else{
	  averageCrossDomainLength=0.0f;
  }
}

void SimuContext::save(ostream& outFile) const{
  outFile << "<simu-context>" << "\n";
  outFile << "  <number-of-elements>" << numberOfElements << "</number-of-elements>" << "\n";
  outFile << "  <days-of-simu-time>" << daysOfSimuTime << "</days-of-simu-time>" << "\n";  
  outFile << "  <template-distance>" << templateDistance << "</template-distance>" << "\n";
  outFile << "  <template-expand-step>" << templateExpandStep << "</template-expand-step>" << "\n";
  outFile << "  <template-adjust-ratio>" << templateAdjustRatio << "</template-adjust-ratio>"<< "\n";
  outFile << "  <template-width>" << templateWidth << "</template-width>" << "\n";
  outFile << "  <template-height>" << templateHeight << "</template-height>" << "\n";
  outFile << "  <template-depth>" << templateDepth << "</template-depth>" << "\n";

  outFile << "  <element-orientation>" << elementOrientation << "</element-orientation>" << "\n";
  outFile << "  <element-width>" << elementWidth << "</element-width>" << "\n";
  outFile << "  <element-height>" << elementHeight << "</element-height>" << "\n";
  outFile << "  <element-depth>" << elementDepth << "</element-depth>" << "\n";
  outFile << "  <element-speed>" << elementSpeed << "</element-speed>" << "\n";
  
  
  outFile << "\n  <center-adjustment enabled=\"" << centerAdjustment << "\">\n";
  outFile << "  </center-adjustment>" << "\n";
  
  //Output surface diffusion options 
  outFile << "\n  <surface-diffusion enabled=\"" << diffusionFlag << "\" level=\"" << diffusionLevel << "\" >\n";
  outFile << "    <diffusion-step>" << diffusionStep << "</diffusion-step>" << "\n";
  //if(diffusionLevel==2){  
    outFile << "    <kt-constant>" << diffusionKTConstant << "</kt-constant>" << "\n";
    outFile << "    <try-times>" << diffusionTryTimes << "</try-times>" << "\n";
    outFile << "    <distance-threshold>" << diffusionDistanceThreshold << "</distance-threshold>" << "\n";
    outFile << "    <influence-coefficient>" << diffusionInfluenceCoefficient << "</influence-coefficient>" << "\n";    
  //}  
  outFile << "  </surface-diffusion>\n";


  outFile << "\n  <hydrophobic-constraints enabled=\"" << hydrophobicFlag << "\" type=\"" << hydroConstraintType << "\">\n";
  outFile << "    <coefficient>" << hydroConstraintCoefficient << "</coefficient>\n";
  //if(hydroConstraintType==1){
  outFile << "    <constant>" << hydroConstraintConstant << "</constant>\n";
  //}  
/*  vector<HydroConstraint>::iterator hitr = hydroConstraints.begin();
  while(hitr!=hydroConstraints.end()){
  	outFile << "    <constraint lower=\"" << (*hitr).lower << "\" upper=\"" << (*hitr).upper << "\" probability=\""<< (*hitr).probability << "\" />\n"; 
  	hitr++;
  }
*/  
  outFile << "  </hydrophobic-constraints>" << "\n";

  
  outFile << "\n  <element-template>" << endl;
  vector<ElementTemplate>::const_iterator itr;
  itr = elementTemplate.begin();
  while(itr != elementTemplate.end()){
    outFile << "    <exon type=\"" << (*itr).exonType() << "\" length=\"" << (*itr).exonLength() << "\" />" << endl;
    itr ++;
  }  
  outFile << "  </element-template>" << endl;

  outFile << "</simu-context>" << endl;
}


int SimuContext::getNumberOfElements() const{
  return numberOfElements;
}

void SimuContext::setNumberOfElements(int number){
	numberOfElements = number;
}

float SimuContext::getTemplateDistance() const{
  return templateDistance;
}

void SimuContext::setTemplateDistance(float distance){
  templateDistance = distance;
}

float SimuContext::getTemplateExpandStep() const{
  return templateExpandStep;
}

void SimuContext::setTemplateExpandStep(float step){
	templateExpandStep = step;
}

float SimuContext::getTemplateAdjustRatio() const{
  return templateAdjustRatio;
}

void SimuContext::setTemplateAdjustRatio(float ratio){
	templateAdjustRatio = ratio;
}

float SimuContext::getTemplateWidth() const{
  return templateWidth;
}

void SimuContext::setTemplateWidth(float width){
	templateWidth = width;
}

float SimuContext::getTemplateHeight() const{
  return templateHeight;
}

void SimuContext::setTemplateHeight(float height){
	templateHeight = height;
}

float SimuContext::getTemplateDepth() const{
  return templateDepth;
}

void SimuContext::setTemplateDepth(float depth){
	templateDepth = depth;
}

int SimuContext::getElementOrientation() const{
  return elementOrientation;
}

void SimuContext::setElementOrientation(int orientation){
  elementOrientation = orientation;
}

float SimuContext::getElementWidth() const{
  return elementWidth;
}

void SimuContext::setElementWidth(float width){
	elementWidth = width;
}

float SimuContext::getElementHeight() const{
  return elementHeight;
}

void SimuContext::setElementHeight(float height){
	elementHeight = height;
}

float SimuContext::getElementDepth() const{
  return elementDepth;
}

void SimuContext::setElementDepth(float depth){
	elementDepth = depth;
}

float SimuContext::getElementSpeed() const{
  return elementSpeed;
}

void SimuContext::setElementSpeed(float speed){
	elementSpeed = speed;
}

int SimuContext::getCenterAdjustment() const{
	return centerAdjustment;
}
void  SimuContext::setCenterAdjustment(int adjustment){
	centerAdjustment = adjustment;
}


int SimuContext::getHydrophobicFlag() const{
  return hydrophobicFlag;
}

void SimuContext::setHydrophobicFlag(int hydroFlag){
  hydrophobicFlag = hydroFlag;
}

int SimuContext::getDiffusionFlag() const{
  return diffusionFlag;
}

void SimuContext::setDiffusionFlag(int diffFlag){
  diffusionFlag = diffFlag;
}

int SimuContext::getDiffusionLevel() const{
  return diffusionLevel;
}

void SimuContext::setDiffusionLevel(int level){
  diffusionLevel = level;
}


float SimuContext::getDiffusionStep() const{
  return diffusionStep;
}

void SimuContext::setDiffusionStep(float diffStep){
  diffusionStep = diffStep;
}

float SimuContext::getDiffusionKTConstant() const{
  return diffusionKTConstant;
}

void SimuContext::setDiffusionKTConstant(float ktConstant){
  diffusionKTConstant = ktConstant;
}

float SimuContext::getSimuHydrophobicLength() const{
  return simuHydrophobicLength;
}

float SimuContext::getAverageCrossDomainLength() const{
	return averageCrossDomainLength;
}

const vector<ElementTemplate>* SimuContext::getElementTemplate() const{
  return &elementTemplate;
}

vector<ElementTemplate>* SimuContext::getElementTemplate() {
  return &elementTemplate;
}

/***************************************************************************/
/* Get the left part of the domain structure of the tropoelastin           */
/* Parameters: @length the length of the left part to be taken             */
/*             @lConnectPart the result of the operation                   */
/* Return: the length of the lConnectPart                                  */
/*                                                                         */
/* For a tropoelastin length of 20 with equal length of 4 domains "hhcch", */
/* the connectLength is 12, then the result is "hhc"                       */
/***************************************************************************/
float SimuContext::getLeftPart(float connectLength, list<ElementTemplate>& lConnectPart) const{
  //vector<ElementTemplate* >* elTemplate = ntext->getElementTemplate();
  vector<ElementTemplate >::const_iterator itr;
  float tmpLength;
  ElementTemplate lEnd;
  
  tmpLength = 0.0f;
  itr = elementTemplate.begin();
  while(itr!=elementTemplate.end() && (tmpLength+(*itr).simuLength()) <= connectLength){
    tmpLength += (*itr).simuLength();
	lEnd = (*itr);
	lEnd.setExonLength(lEnd.simuLength());
    lConnectPart.push_back(lEnd);
    itr++;
  }
 
  if(itr!=elementTemplate.end() && connectLength>tmpLength){    
    lEnd.setExonType((*itr).exonType());
    // To preserve the original simulength for later use
    lEnd.setExonLength((*itr).simuLength());    
    lEnd.setSimuLength(connectLength - tmpLength);
    lEnd.setCenterDistance((*itr).centerDistance());
    lEnd.setReverseCenterDistance((*itr).reverseCenterDistance());
    tmpLength += (connectLength - tmpLength);
    lConnectPart.push_back(lEnd);
  }
  lConnectPart.push_front(tmpHead);  
  return tmpLength;
}

/***************************************************************************/
/* Get the left part of the domain structure of the tropoelastin           */
/* Parameters: @length the length of the left part to be taken             */
/*             @lConnectPart the result of the operation                   */
/* Return: the length of the lConnectPart                                  */
/*                                                                         */
/* For a tropoelastin length of 20 with equal length of 4 domains "hhcch", */
/* the connectLength is 12, then the result is "chh"                       */
/***************************************************************************/
float SimuContext::getReverseRightPart(float connectLength, list<ElementTemplate>& rConnectPart) const{
  //vector<ElementTemplate* >* elTemplate = context->getElementTemplate();
  vector<ElementTemplate >::const_iterator itr;
  float tmpLength;
  ElementTemplate lEnd;
  
  tmpLength = 0.0f;
  itr = elementTemplate.begin();
  while(itr!=elementTemplate.end() && (tmpLength+(*itr).simuLength()) <= connectLength){
    tmpLength += (*itr).simuLength();
	lEnd = (*itr);
	lEnd.setExonLength(lEnd.simuLength());
    rConnectPart.push_front(lEnd);
    itr++;
  }
 
  if(itr!=elementTemplate.end() && connectLength>tmpLength){
    lEnd.setExonType((*itr).exonType());
    // To preserve the original simulength for later use
    lEnd.setExonLength((*itr).simuLength());    
    lEnd.setSimuLength(connectLength - tmpLength);
    lEnd.setCenterDistance((*itr).centerDistance());
    lEnd.setReverseCenterDistance((*itr).reverseCenterDistance());    
    tmpLength += (connectLength - tmpLength);
    rConnectPart.push_front(lEnd);
  }  
  rConnectPart.push_front(tmpHead);  
  return tmpLength;
}

/********************************************************************************/
/* For a tropoelastin length of 20 with equal length of 4 domains "hhcch", the  */
/* connectLength is 12, then the result is "cch"                                */
/********************************************************************************/
float SimuContext::getRightPart(float connectLength, list<ElementTemplate>& rConnectPart) const{
  float tmpLength = 0.0f;
  vector<ElementTemplate >::const_reverse_iterator revItr;
  ElementTemplate rEnd;
  
  revItr = elementTemplate.rbegin();
  while(revItr!=elementTemplate.rend() && (tmpLength+(*revItr).simuLength()) < connectLength){
    tmpLength += (*revItr).simuLength();
	rEnd = (*revItr);
	rEnd.setExonLength(rEnd.simuLength());    
    rConnectPart.push_front(rEnd);
    revItr++;
  }

  if(revItr!=elementTemplate.rend() && tmpLength < connectLength){
    //ElementTemplate rEnd;
    rEnd.setExonType((*revItr).exonType());
    // To preserve the original simulength for later use
    rEnd.setExonLength((*revItr).simuLength());
    rEnd.setSimuLength(connectLength - tmpLength);
    rEnd.setCenterDistance((*revItr).centerDistance());
    rEnd.setReverseCenterDistance((*revItr).reverseCenterDistance());
    
    tmpLength += (connectLength - tmpLength);
    rConnectPart.push_front(rEnd);
  }
  rConnectPart.push_front(tmpHead);  
  return tmpLength;  
}

/********************************************************************************/
/* For a tropoelastin length of 20 with equal length of 4 domains "hhcch", the  */
/* connectLength is 12, then the result is "hcc"                                */
/********************************************************************************/
float SimuContext::getReverseLeftPart(float connectLength, list<ElementTemplate>& lConnectPart) const{
  float tmpLength = 0.0f;
  vector<ElementTemplate>::const_reverse_iterator revItr;
  ElementTemplate rEnd;
  
  revItr = elementTemplate.rbegin();
  while(revItr!=elementTemplate.rend() && (tmpLength+(*revItr).simuLength()) < connectLength){
    tmpLength += (*revItr).simuLength();
	rEnd = (*revItr);
	rEnd.setExonLength(rEnd.simuLength());    
    lConnectPart.push_back(rEnd);
    revItr++;
  }

  if(revItr!=elementTemplate.rend() && tmpLength < connectLength){
    //ElementTemplate rEnd;
    rEnd.setExonType((*revItr).exonType());
    // To preserve the original simulength for later use
    rEnd.setExonLength((*revItr).simuLength());    
    rEnd.setSimuLength(connectLength - tmpLength);
    rEnd.setCenterDistance((*revItr).centerDistance());
    rEnd.setReverseCenterDistance((*revItr).reverseCenterDistance());
    
    tmpLength += (connectLength - tmpLength);
    lConnectPart.push_back(rEnd);
  }
  lConnectPart.push_front(tmpHead);  
  return tmpLength;  
}

/*------------------------------------------------------------------------------*/
/*Display the structure of a part of a tropoelastin, mainly for debug purpose   */
/*------------------------------------------------------------------------------*/
void SimuContext::renderDomainPart(list<ElementTemplate>& domainPart) const{

  //Display domain types
  list<ElementTemplate>::iterator lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).exonType() << "--" ;
    lItr++;
  }
  cout << endl;
  
  //Display domain length
  lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).simuLength() << "--" ;
    lItr++;
  }
  cout << endl;
  
  //Display exon length or original simu length
  lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).exonLength() << "--" ;
    lItr++;
  }
  cout << endl;
  
  /*
  //Display domain center distance
  lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).centerDistance() << "--" ;
    lItr++;
  }
  cout << endl;
  
  //Display reverse domain center distance
  lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).reverseCenterDistance() << "--" ;
    lItr++;
  }
  cout << endl;
  */
}


/*------------------------------------------------------------------------------*/
/*Display the structure of a part of a tropoelastin, mainly for debug purpose   */
/*------------------------------------------------------------------------------*/
void SimuContext::renderTemplateDomainPart(vector<ElementTemplate>& domainPart) const{

  //Display domain types
  vector<ElementTemplate>::iterator lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).exonType() << "--" ;
    lItr++;
  }
  cout << endl;
  
  //Display domain length
  lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).simuLength() << "--" ;
    lItr++;
  }
  cout << endl;

  //Display exon length
  lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).exonLength() << "--" ;
    lItr++;
  }
  cout << endl;
  
/*  
  //Display domain center distance
  lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).centerDistance() << "--" ;
    lItr++;
  }
  cout << endl;
  
  //Display reverse domain center distance
  lItr = domainPart.begin();
  while(lItr!=domainPart.end()){
    cout << setiosflags( ios::left ) << setw(10) << (*lItr).reverseCenterDistance() << "--" ;
    lItr++;
  }
  cout << endl;  
*/  
}

//Compute the distance between domain part centers to the template center
// cccccchhhhhhcccccccchhhhhhhcccc
//    6     6      8      7     4
//Template center: (6+6+8+7+4)/2=15.5
//Orientation=0: then distances are
//    6/2-15.5=-12.5
//    6+6/2-15.5=-3.5
//    6+6+8/2-15.5=1.5
//    6+6+8+7/2-15.5=8
//    6+6+8+7+4/2-15.5=13.5
//If orientation=1, the computation begains in the reverse order from 4, 7, 8, 6, 6
void SimuContext::computeTemplateCenterDistance(){
	float templateCenter = elementWidth/2.0f;
	
	vector<ElementTemplate>::iterator lItr = elementTemplate.begin();
	
	float previousLength=0.0f;
	while(lItr!=elementTemplate.end()){
		float partDistance = previousLength + (*lItr).simuLength()/2.0f - templateCenter;
		(*lItr).setCenterDistance(partDistance);
		previousLength += (*lItr).simuLength();
		lItr++;
	}
	
	vector<ElementTemplate>::reverse_iterator rItr = elementTemplate.rbegin();
	
	previousLength=0.0f;
	while(rItr!=elementTemplate.rend()){
		float partDistance = previousLength + (*rItr).simuLength()/2.0f - templateCenter;
		(*rItr).setReverseCenterDistance(partDistance);
		previousLength += (*rItr).simuLength();
		rItr++;
	}	
}
