#include "XMLFileManager.h"
#include "SimuContext.h"
#include "Aggregate.h"
#include "CylinderElastin.h"

#include <sys/stat.h> // for stat() function
#include <errno.h>    // for errno variable
#include <map>
#include <list>


using namespace std;

XMLFileManager::XMLFileManager()
{
	try{
		xercesc::XMLPlatformUtils::Initialize(); //Initialize Xerces infrastructure
	}
	catch(xercesc::XMLException& e){
	  char* message = xercesc::XMLString::transcode(e.getMessage());
	  cerr << "XML toolkit initialization error:" << message << endl;
	  xercesc::XMLString::release(&message);
	  exit(1);
	}
	XDomParser = new xercesc::XercesDOMParser();		
	XDomParser->setValidationScheme(xercesc::XercesDOMParser::Val_Never);
	XDomParser->setDoNamespaces(false);
	XDomParser->setDoSchema(false);
	XDomParser->setLoadExternalDTD(false);	
}

XMLFileManager::~XMLFileManager()
{
	if(XDomParser!=NULL){
		delete XDomParser;
	}
	try{
		xercesc::XMLPlatformUtils::Terminate();
	}
	catch(xercesc::XMLException& e){
		char* message = xercesc::XMLString::transcode(e.getMessage());
		cerr << "XML toolkit teardown error: " << message << endl;
		xercesc::XMLString::release(&message);		
	}	
}

void XMLFileManager::loadTrace(const std::string& fileName, SimuContext* context, Aggregate* aggregate){
	cout << "loading trace file " << fileName << " ......... " << endl;
 	struct stat fileStatus;
 	int iretStat = stat(fileName.c_str(), &fileStatus);
 	if(iretStat!=0){
 		throw(std::runtime_error(string("Error when load simulation trace file: ") + fileName + " - "  + strerror(errno)));
 	}
 	
	try{
	  	XDomParser->parse(fileName.c_str());
	  	// no need to free this object, it is owned by the parent parser object
	  	xercesc::DOMDocument* xmlDoc = XDomParser->getDocument();
	  		
	  	xercesc::DOMElement* elementRoot = xmlDoc->getDocumentElement();
	    //if(root.tagName() != "dla-trace"){
	    //  cerr << "root tag isn't \"dla-tree\" " << endl;
	    //}
	        		
	  	if(!elementRoot){
	  		throw(std::runtime_error("The simulation trace file " + fileName + "is empty !"));
	  	}
	    //cout << "loading context ... " << endl;  
	    //context->fromDomElement(getFirstChildElement(elementRoot, "simu-context"));
	  	loadContext(getFirstChildElement(elementRoot, "simu-context"), context);
	  
	    //cout << "loading trace ... " << endl;
	    //aggregate->fromDomElement(getFirstChildElement(elementRoot, "aggregate"));
	  	loadAggregate(getFirstChildElement(elementRoot, "aggregate"), aggregate);
	  }
	  catch(exception &e){
	  	cout << "Exception Caught: " << e.what() << "\n";
	  	throw e;
	  }	
}

bool XMLFileManager::saveTrace(const std::string& fileName, SimuContext* context, Aggregate* aggregate){
	  ofstream outFile(fileName.c_str());
	  if(!outFile.good()) {
	    cerr << "Error while opening output file!" << endl;
	    return false;
	  }    
	  outFile << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;	
	  outFile << "<dla-trace>" << endl;
	  saveContext(outFile, context);
	  saveAggregate(outFile, aggregate);
	  outFile << "</dla-trace>" << endl;
	  outFile.close();
	  return true;	  
}

void XMLFileManager::saveTrace(ofstream& outFile, SimuContext* context, Aggregate* aggregate){
	  outFile << "<dla-trace>" << endl;
	  saveContext(outFile, context);
	  saveAggregate(outFile, aggregate);
	  outFile << "</dla-trace>" << endl;
}

void XMLFileManager::loadContext(const std::string& fileName, SimuContext* context){
	struct stat fileStatus;
	int iretStat = stat(fileName.c_str(), &fileStatus);
	if(iretStat!=0){
		throw(std::runtime_error(string("Error in context file: ") + fileName + " - "  + strerror(errno)));
	}
	
	try{
		XDomParser->parse(fileName.c_str());
		// no need to free this object, it is owned by the parent parser object
		xercesc::DOMDocument* xmlDoc = XDomParser->getDocument();
		
		xercesc::DOMElement* contextRoot = xmlDoc->getDocumentElement();
		if(!contextRoot){
			throw(std::runtime_error("The context file " + fileName + "is empty !"));
		}
		
		loadContext(contextRoot, context);
		
	}
	catch(exception &e){
		cout << "Exception Caught: " << e.what() << "\n";
		throw e;
	}	
}

bool XMLFileManager::saveContext(const std::string& fileName, SimuContext* context){
	  ofstream outFile(fileName.c_str());
	  if(!outFile.good()) {
	    cerr << "Error while opening output file!" << endl;
	    return false;
	  }    
	  outFile << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;	
	  saveContext(outFile, context);
	  outFile.close();
	  return true;	  
}

void XMLFileManager::saveContext(std::ofstream& outFile, SimuContext* context){
	context->save(outFile);
}

void XMLFileManager::loadContext(const xercesc::DOMElement* contextRoot, SimuContext* context){
	loadBasicContextParameters(contextRoot, context);
	loadCenterAdjustParameters(contextRoot, context);
	loadDiffusionParameters(contextRoot, context);
	loadHydroConstraintParameters(contextRoot, context);
	loadParticleDomainParamenters(contextRoot, context);
}

void XMLFileManager::loadBasicContextParameters(const xercesc::DOMElement* root, SimuContext* context){
  context->setNumberOfElements(getIntParameter(root, "number-of-elements"));
  try{
  context->setDaysOfSimuTime(getFloatParameter(root, "days-of-simu-time"));
  }
  catch(...){	  
  }
  context->setTemplateDistance(getFloatParameter(root, "template-distance"));
  context->setTemplateExpandStep(getFloatParameter(root, "template-expand-step"));
  context->setTemplateAdjustRatio(getFloatParameter(root, "template-adjust-ratio"));
  context->setTemplateWidth(getFloatParameter(root, "template-width"));
  context->setTemplateHeight(getFloatParameter(root, "template-height"));
  context->setTemplateDepth(getFloatParameter(root, "template-depth"));
  context->setElementOrientation(getIntParameter(root, "element-orientation"));
  context->setElementWidth(getFloatParameter(root, "element-width"));
  context->setElementHeight(getFloatParameter(root, "element-height"));
  context->setElementDepth(getFloatParameter(root, "element-depth"));
  context->setElementSpeed(getFloatParameter(root, "element-speed"));
}

void XMLFileManager::loadCenterAdjustParameters(const xercesc::DOMElement* contextRoot, SimuContext* context){    
	xercesc::DOMElement* domCenterAdjustment = getFirstChildElement(contextRoot, "center-adjustment");
	if(domCenterAdjustment!=NULL){ 
		char* centerAdjustmentEnabled = xercesc::XMLString::transcode(domCenterAdjustment->getAttribute(xercesc::XMLString::transcode("enabled")));
		if(string(centerAdjustmentEnabled)=="1"){
			context->setCenterAdjustment(1);
		}
		else{
			context->setCenterAdjustment(0);
		}
		xercesc::XMLString::release(&centerAdjustmentEnabled);      
	}	
}

void XMLFileManager::loadDiffusionParameters(const xercesc::DOMElement* contextRoot, SimuContext* context){
	xercesc::DOMElement* domSurface = getFirstChildElement(contextRoot, "surface-diffusion");
	if(domSurface!=NULL){ 
		char* cDiffusionEnabled = xercesc::XMLString::transcode(domSurface->getAttribute(xercesc::XMLString::transcode("enabled")));
		char* cDiffusionLevel = xercesc::XMLString::transcode(domSurface->getAttribute(xercesc::XMLString::transcode("level")));    
		if(string(cDiffusionEnabled)=="1"){
			context->setDiffusionFlag(1);
			//if(string(cDiffusionLevel)=="1"){
				//diffusionFlag       = getIntParameter(root, "diffusion-flag");
			//	context->setDiffusionLevel(1);
				//context->setDiffusionStep(getFloatParameter(domSurface, "diffusion-step"));    		
			//}
			//else if(string(cDiffusionLevel)=="2"){
			//	context->setDiffusionLevel(2);
			//}
			context->setDiffusionLevel(atoi(cDiffusionLevel));
			context->setDiffusionKTConstant(getFloatParameter(domSurface, "kt-constant"));
			context->setDiffusionStep(getFloatParameter(domSurface, "diffusion-step"));
			context->setDiffusionTryTimes(getIntParameter(domSurface, "try-times"));
			context->setDiffusionDistanceThreshold(getFloatParameter(domSurface, "distance-threshold"));
			context->setDiffusionInfluenceCoefficient(getFloatParameter(domSurface, "influence-coefficient"));			    	
		}
		else{
			context->setDiffusionFlag(0);
		}
		xercesc::XMLString::release(&cDiffusionEnabled);
		xercesc::XMLString::release(&cDiffusionLevel);      
	}	
}

void XMLFileManager::loadHydroConstraintParameters(const xercesc::DOMElement* contextRoot, SimuContext* context){
	xercesc::DOMElement* domHydro = getFirstChildElement(contextRoot, "hydrophobic-constraints");
	if(domHydro!=NULL){ 
		char* cHydroEnabled = xercesc::XMLString::transcode(domHydro->getAttribute(xercesc::XMLString::transcode("enabled")));
		char* cHydroType = xercesc::XMLString::transcode(domHydro->getAttribute(xercesc::XMLString::transcode("type")));    
		//cout << cHydroType << endl;
		if(string(cHydroEnabled)=="1"){
			context->setHydrophobicFlag(1);
			if(string(cHydroType)=="1"){
				//diffusionFlag       = getIntParameter(root, "diffusion-flag");
				context->setHydroConstraintType(1);
				context->setHydroConstraintCoefficient(getFloatParameter(domHydro, "coefficient"));
				context->setHydroConstraintConstant(getFloatParameter(domHydro, "constant"));    		
			}
			else if(string(cHydroType)=="2"){
				context->setHydroConstraintType(2);
				context->setHydroConstraintCoefficient(getFloatParameter(domHydro, "coefficient"));   		
			}
			else if(string(cHydroType)=="3" ){
				context->setHydroConstraintType(3);
				context->setHydroConstraintCoefficient(getFloatParameter(domHydro, "coefficient"));   		
			}
			else if(string(cHydroType)=="4" ){
				context->setHydroConstraintType(4);
				context->setHydroConstraintCoefficient(getFloatParameter(domHydro, "coefficient"));   		
			}    	      	
			else if(string(cHydroType)=="5"){
				context->setHydroConstraintType(5);
				context->setHydroConstraintCoefficient(getFloatParameter(domHydro, "coefficient"));
				context->setHydroConstraintConstant(getFloatParameter(domHydro, "constant"));    		
			}
			else{
				cout << "Error in reading hydrophobic-constraints type = " << cHydroType;
				cout << ", a valid value is in [1:5]." << endl;
				return;
			}
		}
		else{
			context->setHydrophobicFlag(0);
		}
		xercesc::XMLString::release(&cHydroEnabled);
		xercesc::XMLString::release(&cHydroType);      
	}	
}

void XMLFileManager::loadParticleDomainParamenters(const xercesc::DOMElement* contextRoot, SimuContext* context){
	xercesc::DOMElement* dlaTemplate = getFirstChildElement(contextRoot, "element-template");
	if(dlaTemplate!=NULL){  
		parseElementTemplate(dlaTemplate, *(context->getElementTemplate()));
		context->computeSimuLength();
		context->computeTemplateCenterDistance();
	}	
}

xercesc::DOMElement* XMLFileManager::getFirstChildElement(const xercesc::DOMElement* parent, const string& childName){
	xercesc::DOMNodeList* children = parent->getElementsByTagName(xercesc::XMLString::transcode(childName.c_str()));
 	const XMLSize_t nodeCount = children->getLength();
 	//cout << "Number of Nodes: " << nodeCount << endl;
 	if(nodeCount==1){
 		return dynamic_cast<xercesc::DOMElement*>(children->item(0));
 	}
 	return NULL;
}
bool XMLFileManager::tagExists(const xercesc::DOMElement* root, const string& tagName){
	xercesc::DOMNodeList* children = root->getElementsByTagName(xercesc::XMLString::transcode(tagName.c_str()));
	if(children->getLength() > 0 ){
		return true;
	}
	return false;
}

void XMLFileManager::getStringParameter(const xercesc::DOMElement* root, const string& paraName, string& paraValue) throw(std::runtime_error){
	xercesc::DOMNodeList* children = root->getElementsByTagName(xercesc::XMLString::transcode(paraName.c_str()));
 	const XMLSize_t nodeCount = children->getLength();
 	//cout << "Number of Nodes: " << nodeCount << endl;
 	if(nodeCount==1){
 		//cout << children->item(0)->hasChildNodes() << endl;  		
 		if(children->item(0)->hasChildNodes()){
 			xercesc::DOMNode* firstChild = children->item(0)->getFirstChild();
 		  char* strValue = xercesc::XMLString::transcode(firstChild->getNodeValue());
 		  if(strValue){
 		    paraValue = string(strValue);
 		    xercesc::XMLString::release(&strValue);
 		  }
 		}
 		else{
 			//cout << "No values" << endl;
 			paraValue="";
 		    throw(std::runtime_error("No value is set for parameter " + paraName + " in the context file !"));  			
 		}
 	}
 	else if(nodeCount<1){
 		throw(std::runtime_error("Parameter " + paraName + " does not exist in the context file !"));
 	}
 	else{
 		throw(std::runtime_error("More than one parameters named " + paraName + " exist in the context file !"));
 	}
}
  
float XMLFileManager::getFloatParameter(const xercesc::DOMElement* root, const string& paraName){
 	string tmpStr;
 	getStringParameter(root, paraName, tmpStr);
 	if(tmpStr.length()>0){
 	  return atof(tmpStr.c_str());  	  
 	}
 	return 0.0f;
}
  
int XMLFileManager::getIntParameter(const xercesc::DOMElement* root, const string& paraName){
  string tmpStr;
 	getStringParameter(root, paraName, tmpStr);
 	if(tmpStr.length()>0){
 		return atoi(tmpStr.c_str());
 	}
 	else{
 		return 0;
 	}
}

void XMLFileManager::parseElementTemplate(xercesc::DOMElement* dlaTemplate, vector<ElementTemplate>& tempVector){
  if(tempVector.size()>0){
    tempVector.clear();
  }  
  XMLCh* TAGNAME = xercesc::XMLString::transcode("exon");
  
  xercesc::DOMNode* child = dlaTemplate->getFirstChild();
  while(child!=NULL){
	  xercesc::DOMElement* curElement = static_cast<xercesc::DOMElement*>(child);
  	if(xercesc::XMLString::equals(curElement->getTagName(), TAGNAME )){
      char* cType = xercesc::XMLString::transcode(curElement->getAttribute(xercesc::XMLString::transcode("type")));
      char* cLength = xercesc::XMLString::transcode(curElement->getAttribute(xercesc::XMLString::transcode("length")));
      float eLength = atof(cLength);
      
      ElementTemplate pE;
      //pE = new ElementTemplate;
      pE.setExonType(cType[0]);
      pE.setExonLength(eLength);
      //elementTemplate.push_back(pE);
      tempVector.push_back(pE);
      xercesc::XMLString::release(&cType);
      xercesc::XMLString::release(&cLength);
  	}
    child = child->getNextSibling();
  } 
  xercesc::XMLString::release(&TAGNAME);
}

void XMLFileManager::loadAggregate(const xercesc::DOMElement* aggDom, Aggregate* aggregate){
	  //Before finishing reading all the traces in, the details about each neighbour can
	  //not be determined, so the neighbours' id will are saved in the map. 
	  map<int, list<TmpNeighbour> > neighbourMap;
		
	  if(!aggregate->empty()){
		  aggregate->clear();
	  }
	  //xercesc::DOMNodeList* children = aggDom->getElementsByTagName(xercesc::XMLString::transcode("no-name"));
	  //cout << children->getLength() << endl;
	  //list<DLAElement*>::iterator itr;
	  //itr = dlaElements.begin();
	  //while(itr!=dlaElements.end()){
	  //  delete (*itr);
	  //  itr ++;
	  //}
	  //Read the time spent for generating the aggregate.
	  //try{
		  aggregate->setSecondsSpent(getIntParameter(aggDom, "seconds-spent"));
		  if(tagExists(aggDom, "minX")){
			  aggregate->setMinX(getFloatParameter(aggDom, "minX"));
			  aggregate->setMaxX(getFloatParameter(aggDom, "maxX"));
			  aggregate->setMinY(getFloatParameter(aggDom, "minY"));
			  aggregate->setMaxY(getFloatParameter(aggDom, "maxY"));
			  aggregate->setMinZ(getFloatParameter(aggDom, "minZ"));
			  aggregate->setMaxZ(getFloatParameter(aggDom, "maxZ"));
		  }
	  //}
	  //catch(...){
		  
	  //}

	  XMLCh* TAGNAME = xercesc::XMLString::transcode("element");
	  XMLCh* NEIGHBOURTAG = xercesc::XMLString::transcode("neighbour");
	  
	  float ex, ey, ez, angle;
	  int   id, orientation, nId;
	  xercesc::DOMElement* child = static_cast<xercesc::DOMElement*>(aggDom->getFirstChild());

	  while(child!=NULL){
	  	//cout << XMLString::transcode(child->getTagName()) << endl;
	    if(xercesc::XMLString::equals(child->getTagName(), TAGNAME )){
	      //cout << child->attribute("type").toStdString() << ", " << child->attribute("length").toStdString() << endl;
	      char* cId = xercesc::XMLString::transcode(child->getAttribute(xercesc::XMLString::transcode("id")));
	      char* cx = xercesc::XMLString::transcode(child->getAttribute(xercesc::XMLString::transcode("x")));
	      char* cy = xercesc::XMLString::transcode(child->getAttribute(xercesc::XMLString::transcode("y")));
	      char* cz = xercesc::XMLString::transcode(child->getAttribute(xercesc::XMLString::transcode("z")));
	      char* co = xercesc::XMLString::transcode(child->getAttribute(xercesc::XMLString::transcode("orientation")));
	      id  = atoi(cId);
	      ex  = atof(cx);
	      ey  = atof(cy);
	      ez  = atof(cz);
	      orientation = atoi(co);
	      DLAElement* pE;
	      pE = new CylinderElastin; //DLA3DElement;
	      pE->setElementId(id);
	      pE->setPos(ex, ey, ez);
	      pE->setOrientation(orientation);
	      
	      //Read neighbour information
	      xercesc::DOMElement* grandChild = static_cast<xercesc::DOMElement*>(child->getFirstChild());
	      while(grandChild!=NULL){
	    	  if(xercesc::XMLString::equals(grandChild->getTagName(), NEIGHBOURTAG)){
	    		  char* cAngle = xercesc::XMLString::transcode(grandChild->getAttribute(xercesc::XMLString::transcode("angle")));
	    		  char* cNId   = xercesc::XMLString::transcode(grandChild->getAttribute(xercesc::XMLString::transcode("id")));
	    		  angle = atof(cAngle);
	    		  nId = atoi(cNId);
	    		  
	    		  TmpNeighbour neighbour;
	    		  neighbour.angle = angle;
	    		  neighbour.neighbourId = nId;
	    		  if(neighbourMap.find(id)!=neighbourMap.end()){
	    			  neighbourMap[id].push_back(neighbour);
	    		  }
	    		  else{
	    			  list<TmpNeighbour> tnList;
	    			  tnList.push_back(neighbour);
	    			  neighbourMap[id] = tnList;
	    		  }
	    		  //pE->addNeighbour(angle, nId);
	    		  xercesc::XMLString::release(&cAngle);
	    	      xercesc::XMLString::release(&cNId);    		  
	    	  }
	    	  grandChild = static_cast<xercesc::DOMElement*>(grandChild->getNextSibling());
	      }
	      
	      //dlaElements.push_back(pE); // commented it out on sep. 11, 2008
	      aggregate->addElement(pE);
	      //cout << "adding element " << pE->getElementId() << endl;
	      xercesc::XMLString::release(&cId);
	      xercesc::XMLString::release(&cx);
	      xercesc::XMLString::release(&cy);
	      xercesc::XMLString::release(&cz);
	      xercesc::XMLString::release(&co);      
	    }    
	    child = static_cast<xercesc::DOMElement*>(child->getNextSibling());
	  }
	  xercesc::XMLString::release(&TAGNAME); 
	  xercesc::XMLString::release(&NEIGHBOURTAG); 
	  
	  //Add neighbour information
	  map<int, DLAElement*>* mapById = aggregate->getIdMap();
	  map<int, DLAElement*>::iterator mItr;
	  for(mItr=mapById->begin(); mItr!=mapById->end(); mItr++){
		  list<TmpNeighbour> nList;
		  list<TmpNeighbour>::iterator nItr;
		  nList = neighbourMap[(*mItr).first];
		  for(nItr=nList.begin(); nItr!=nList.end(); nItr++){
			  (*mItr).second->addNeighbour((*nItr).angle, (*mapById)[(*nItr).neighbourId]);
		  }
		  //cout << (*mItr).first << "\n";
	  }	
}

void XMLFileManager::saveAggregate(ofstream& outFile, Aggregate* aggregate){
	aggregate->save(outFile);
}
