#include <QtGui>
#include <sys/stat.h> // for stat() function
#include <errno.h>    // for errno variable

#include "MainWindow.h"
#include "GLViewWidget.h"
#include "WidgetRotation.h"
#include "WidgetMovement.h"
#include "WidgetZoom.h"
#include "CrossViewExt.h"
#include "XMLFileManager.h"
#include "DetailViewExt.h"
#include "CylinderElastin.h"
#include "DLASerialization.h"
#include "RenderThread.h"
#include <QBuffer>
#include <QByteArray>
#include <QImage>
#include <QMutex>


QMutex         		  criticalSection;

MainWindow::MainWindow()
  :aggregate(NULL), context(NULL), m_renderThread(NULL),m_dataSize(0)
{
    readSettings();
    setCurrentFile("");
    setWindowTitle(tr("Elastin Studio"));

    connect(&commSocket, SIGNAL(aboutToClose()), this, SLOT(commSocketAboutToClose()));
    connect(&commSocket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(commTcpError(QAbstractSocket::SocketError)));
    connect(&controlSocket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(controlTcpError(QAbstractSocket::SocketError)));
    
    connect(&commSocket, SIGNAL(readyRead()), this, SLOT(tcpReady()));
}

MainWindow::~MainWindow(){
	if(aggregate!=NULL){
	  delete aggregate;
	}
	if(context!=NULL){
	  delete context;
	}
}

void MainWindow::initialize(string fileName){
  if(!fileName.empty()){
	  XMLFileManager xManager;
	  context = SimuContext::getInstance();
	  aggregate = new Aggregate();  
	  xManager.loadTrace(fileName, context, aggregate);
  }
	
  createActions();
  createMenus();
  createToolBars();
  createStatusBar();
    
  if(aggregate!=NULL){
	  glWidget = new GLViewWidget(aggregate);
  }
  else{
	  glWidget = new GLViewWidget();
	  connect(this, SIGNAL(newAggregate(Aggregate*)), glWidget, SLOT(setAggregate(Aggregate*)));
	  connect(this, SIGNAL(realtimeEvent()), glWidget, SLOT(realtimeEvent()));
	  connect(this, SIGNAL(newPosition(float, float, float)), glWidget, SLOT(newPosition(float, float, float)));	  
  }
  setCentralWidget(glWidget);
  createDockWindows();   	
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (maybeSave()) {
        writeSettings();
        event->accept();
    } else {
        event->ignore();
    }
}

void MainWindow::newFile()
{
    if (maybeSave()) {
//        textEdit->clear();
        setCurrentFile("");
    }
}

void MainWindow::open()
{  
  QString errorStr;
	
  QString fileName = QFileDialog::getOpenFileName(this);
  if (!fileName.isEmpty()){
     setCurrentFile(fileName);
     simuStartAct->setEnabled(true);
     setWindowTitle(tr("Elastin Studio - ") + fileName);
  }
  else{
	  return;
  }
   
  XMLFileManager xManager;
  SimuContext::destroyCurrentContext();
  context = SimuContext::getInstance();
  if(aggregate!=NULL){
	  delete aggregate;
	  aggregate=NULL;
  }
  aggregate = new Aggregate();  
  xManager.loadTrace(fileName.toStdString(), context, aggregate);
  
  ((GLViewWidget*)glWidget)->setAggregate(aggregate);
}

bool MainWindow::save()
{
    if (curFile.isEmpty()) {
        return saveAs();
    } else {
        return saveFile(curFile);
    }
}

bool MainWindow::saveAs()
{
    QString fileName = QFileDialog::getSaveFileName(this);
    if (fileName.isEmpty())
        return false;

    return saveFile(fileName);
}

bool MainWindow::exportImage(){
	QString fileName = QFileDialog::getSaveFileName(this);
	if(fileName.isEmpty()){
		return false;
	}
	QPixmap pixmap = glWidget->renderPixmap();
	
	return pixmap.save(fileName);
}
void MainWindow::about()
{
   QMessageBox::about(this, tr("About Application"),
            tr("The <b>Application</b> example demonstrates how to "
               "write modern GUI applications using Qt, with a menu bar, "
               "toolbars, and a status bar."));
}

void MainWindow::documentWasModified()
{
//    setWindowModified(textEdit->document()->isModified());
}

void MainWindow::createActions()
{
	// The file menu
    newAct = new QAction(QIcon(":/images/new.png"), tr("&New"), this);
    newAct->setShortcut(tr("Ctrl+N"));
    newAct->setStatusTip(tr("Create a new file"));
    connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

    openAct = new QAction(QIcon(":/images/open.png"), tr("&Open..."), this);
    openAct->setShortcut(tr("Ctrl+O"));
    openAct->setStatusTip(tr("Open an existing file"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

    saveAct = new QAction(QIcon(":/images/save.png"), tr("&Save"), this);
    saveAct->setShortcut(tr("Ctrl+S"));
    saveAct->setStatusTip(tr("Save the document to disk"));
    connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

    saveAsAct = new QAction(tr("Save &As..."), this);
    saveAsAct->setStatusTip(tr("Save the document under a new name"));
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

    exportAct = new QAction(tr("Export Image"), this);
    exportAct->setStatusTip(tr("Export the 3D view as a 2D image"));
    connect(exportAct, SIGNAL(triggered()), this, SLOT(exportImage()));
    
    exitAct = new QAction(tr("E&xit"), this);
    exitAct->setShortcut(tr("Ctrl+Q"));
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    aboutQtAct = new QAction(tr("About &Qt"), this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

    simuStartAct = new QAction(QIcon(":/images/start.png"), tr("Start"), this);
    simuStartAct->setStatusTip(tr("Start a new simulation"));
    connect(simuStartAct, SIGNAL(triggered()), this, SLOT(startVisualization()) );
    //simuStartAct->setEnabled(false);

    simuPauseAct = new QAction(QIcon(":/images/pause.png"), tr("Pause"), this);
    simuPauseAct->setStatusTip(tr("Pause the current simulation"));
    connect(simuPauseAct, SIGNAL(triggered()), this, SLOT(pauseVisualization()) );
    simuPauseAct->setEnabled(false);

    simuContinueAct = new QAction(QIcon(":/images/continue.png"), tr("Continue"), this);
    simuContinueAct->setStatusTip(tr("Continue the paused simulation"));
    connect(simuContinueAct, SIGNAL(triggered()), this, SLOT(resumeVisualization()) );
    simuContinueAct->setEnabled(false);

    simuStopAct = new QAction(QIcon(":/images/stop.png"), tr("Stop"), this);
    simuStopAct->setStatusTip(tr("Stop the current simulation"));
    connect(simuStopAct, SIGNAL(triggered()), this, SLOT(stopVisualization()) );
    simuStopAct->setEnabled(false);
    
    viewCrossAct = new QAction(QIcon(":/images/cross.png"), tr("Cross"), this);
    viewCrossAct->setStatusTip(tr("View cross section of the aggregate"));
    connect(viewCrossAct, SIGNAL(triggered()), this, SLOT(viewCrossSection()) );
    
    viewDetailAct = new QAction(QIcon(":/images/detail.png"), tr("Detail"), this);
    viewDetailAct->setStatusTip(tr("View details about an particle"));
    connect(viewDetailAct, SIGNAL(triggered()), this, SLOT(viewParticleDetails()) );
    
    realtimeVisualAct = new QAction(QIcon(":/images/realtime.png"), tr("Realtime Visualization"), this);
    realtimeVisualAct->setStatusTip(tr("Realtime visulaize the aggregate forming process"));
    connect(realtimeVisualAct, SIGNAL(triggered()), this, SLOT(realtimeVisualization()));    
}

void MainWindow::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(newAct);
    fileMenu->addAction(openAct);
    fileMenu->addAction(saveAct);
    fileMenu->addAction(saveAsAct);
    fileMenu->addAction(exportAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);

    viewMenu = menuBar()->addMenu(tr("&View"));
    viewMenu->addAction(viewCrossAct);
    viewMenu->addAction(viewDetailAct);
    viewMenu->addAction(realtimeVisualAct);
    viewMenu->addSeparator();
    
    simuMenu = menuBar()->addMenu(tr("&Simulate"));
    simuMenu->addAction(simuStartAct);
    simuMenu->addAction(simuPauseAct);
    simuMenu->addAction(simuContinueAct);
    simuMenu->addAction(simuStopAct);
    menuBar()->addSeparator();

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
}

void MainWindow::createToolBars()
{
    fileToolBar = addToolBar(tr("File"));
    fileToolBar->addAction(newAct);
    fileToolBar->addAction(openAct);
    fileToolBar->addAction(saveAct);
    
    viewToolBar = addToolBar(tr("View Options"));
    //QAction *viewRotationAct;
    //QAction *viewMovementAct;
    //QAction *viewZoomAct;
    viewToolBar->addAction(viewCrossAct);
    viewToolBar->addAction(viewDetailAct);
    viewToolBar->addAction(realtimeVisualAct);    

    simuToolBar = addToolBar(tr("Simulation Controls"));
    simuToolBar->addAction(simuStartAct);
    simuToolBar->addAction(simuPauseAct);
    simuToolBar->addAction(simuContinueAct);
    simuToolBar->addAction(simuStopAct);    
}

void MainWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

void MainWindow::readSettings()
{
    QSettings settings("Computational Biology", "DLA Example");
    QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
    QSize size = settings.value("size", QSize(400, 400)).toSize();
    resize(size);
    move(pos);
}

void MainWindow::writeSettings()
{
    QSettings settings("Computational Biology", "DLA Example");
    settings.setValue("pos", pos());
    settings.setValue("size", size());
}

bool MainWindow::maybeSave()
{
    return true;
}

void MainWindow::loadFile(const QString &fileName)
{
	cout << fileName.size() << endl;
}

bool MainWindow::saveFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox::warning(this, tr("Application"),
                             tr("Cannot write file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return false;
    }

    QTextStream out(&file);
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QApplication::restoreOverrideCursor();

    setCurrentFile(fileName);
    statusBar()->showMessage(tr("File saved"), 2000);
    return true;
}

void MainWindow::setCurrentFile(const QString &fileName)
{
    curFile = fileName;
    setWindowModified(false);

    QString shownName;
    if (curFile.isEmpty())
        shownName = "untitled.txt";
    else
        shownName = strippedName(curFile);

    setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(tr("Application")));
}

QString MainWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}

void MainWindow::startVisualization(){
     controlSocket.connectToHost("localhost", 9877);
     if(controlSocket.waitForConnected(1000)){
    	 controlSocket.write("Start");
    	 //Open communication port for data transfer
   		 commSocket.connectToHost("localhost", 9876);
    	 controlSocket.disconnectFromHost();
         simuStartAct->setEnabled(false);
         simuPauseAct->setEnabled(true);
         simuStopAct->setEnabled(true);    	 
     }
}

void MainWindow::pauseVisualization(){
     controlSocket.connectToHost("localhost", 9877);
     if(controlSocket.isOpen()){
    	 controlSocket.write("Pause");
    	 controlSocket.disconnectFromHost();
         simuPauseAct->setEnabled(false);
         simuContinueAct->setEnabled(true);
         simuStopAct->setEnabled(false);
     }
}

void MainWindow::resumeVisualization(){
     controlSocket.connectToHost("localhost", 9877);
     if(controlSocket.isOpen()){
    	 controlSocket.write("Resume");
    	 controlSocket.disconnectFromHost();
         simuPauseAct->setEnabled(true);
         simuContinueAct->setEnabled(false);
         simuStopAct->setEnabled(true);    	 
     }
}


void MainWindow::stopVisualization(){
     controlSocket.connectToHost("localhost", 9877);
     if(controlSocket.isOpen()){
    	 controlSocket.write("Stop");
    	 if(commSocket.isOpen()){
    	 	 commSocket.disconnectFromHost();
    	 }
    	 controlSocket.disconnectFromHost();
         simuStartAct->setEnabled(true);
         simuPauseAct->setEnabled(false);
         simuContinueAct->setEnabled(false);
         simuStopAct->setEnabled(false);    	 
     }
}

void MainWindow::viewCrossSection(){
  QDialog dlg;
  CrossViewExt* ext = new CrossViewExt(context, aggregate, &dlg);
  QVBoxLayout* lay = new QVBoxLayout(&dlg);
  lay->addWidget(ext);
  dlg.setWindowTitle(tr("Cross Viewer"));
  dlg.resize(1000,1000);
  dlg.exec();
}

void MainWindow::viewParticleDetails(){
  QDialog dlg;
  //CrossViewExt* ext = new CrossViewExt(context, aggregate, &dlg);
  DetailViewExt* ext = new DetailViewExt(context, aggregate, &dlg);
  QVBoxLayout* lay = new QVBoxLayout(&dlg);
  lay->addWidget(ext);
  dlg.setWindowTitle(tr("Details Viewer"));
  dlg.exec();
}

void MainWindow::realtimeVisualization(){
  commSocket.connectToHost("localhost", 9876);	
}

void MainWindow::tcpReady(){
	//qDebug() << "TCP Ready ... ";
	while(commSocket.bytesAvailable()>sizeof(qint32)){
	if(m_dataSize==0){
		//if(commSocket.bytesAvailable()<sizeof(qint32)){
		//	return;
		//}
		QDataStream stream(&commSocket);
		stream.setVersion(QDataStream::Qt_4_0);
		stream >> m_dataSize;
		//qDebug() << m_dataSize;		
	}

	if(m_dataSize>commSocket.bytesAvailable()){
		return;
	}
	criticalSection.lock();
	QByteArray array = commSocket.read(m_dataSize);
	QDataStream dataStream(&array, QIODevice::ReadOnly);
	qint32        dataType;
	dataStream >> dataType;
	if(dataType==0){ //Context
		if(context==NULL){
			context = SimuContext::getInstance();
		}
		dataStream >> (*context);
		context->save(std::cout);
	}
	else if(dataType==1){ //Aggregate
		if(aggregate!=NULL){
			delete aggregate;
			aggregate = NULL;
		}
		aggregate = new Aggregate();
		dataStream >> (*aggregate);
		emit newAggregate(aggregate);
		aggregate->save(std::cout);
	}
	else if(dataType==2){//Newly arriving element
		DLAElement* pElm = new CylinderElastin();
		dataStream >> (*pElm);
		qDebug() << pElm->getElementId();
		aggregate->addElement(pElm);
		emit realtimeEvent();
	}
	else if(dataType==3){ //Diffusion
		DLAElement* pElm = new CylinderElastin();
		dataStream >> (*pElm);
		emit newPosition(pElm->getX(), pElm->getY(), pElm->getZ());
		//qDebug() << pElm->getY();		
        delete pElm;		
	}
	m_dataSize = 0;
	criticalSection.unlock();
	}
}

void MainWindow::commSocketAboutToClose(){
	//criticalSection.lock();
	//g_commFinished = true;
	//criticalSection.unlock();
}

void MainWindow::commTcpError(QAbstractSocket::SocketError error){
	if(error==QAbstractSocket::RemoteHostClosedError){
		return;
	}
	QMessageBox::warning(this, tr("Error"), tr("TCP error: %1").arg(commSocket.errorString()));
	return;
	//qDebug()<< "No image !";
}

void MainWindow::controlTcpError(QAbstractSocket::SocketError error){
	if(error==QAbstractSocket::RemoteHostClosedError){
		return;
	}
	QMessageBox::warning(this, tr("Error"), tr("TCP error: %1").arg(controlSocket.errorString()));
	//qDebug()<< "No image !";
	return;
}

void MainWindow::createDockWindows(){
  QDockWidget *dock = new QDockWidget(tr("Rotation"), this);
  dock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
  
  WidgetRotation* rotWidget = new WidgetRotation(dock);  
  dock->setWidget(rotWidget);
  addDockWidget(Qt::RightDockWidgetArea, dock);
  viewMenu->addAction(dock->toggleViewAction());
  
  connect(rotWidget, SIGNAL(xSetRotationChanged(int)),
         glWidget, SLOT(setXRotation(int)));
  connect(rotWidget, SIGNAL(ySetRotationChanged(int)),
         glWidget, SLOT(setYRotation(int)));
  connect(rotWidget, SIGNAL(zSetRotationChanged(int)),
         glWidget, SLOT(setZRotation(int)));

  dock = new QDockWidget(tr("Movement"), this);
  dock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
  
  WidgetMovement* movWidget = new WidgetMovement(dock);  
  dock->setWidget(movWidget);
  addDockWidget(Qt::RightDockWidgetArea, dock);
  viewMenu->addAction(dock->toggleViewAction());

  connect(movWidget, SIGNAL(xSetMovementChanged(float)),
         glWidget, SLOT(setXMovement(float)));
  connect(movWidget, SIGNAL(ySetMovementChanged(float)),
         glWidget, SLOT(setYMovement(float)));
  connect(movWidget, SIGNAL(zSetMovementChanged(float)),
         glWidget, SLOT(setZMovement(float))); 
  

  dock = new QDockWidget(tr("Zoom"), this);
  dock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
  
  WidgetZoom* zoomWidget = new WidgetZoom(dock);  
  dock->setWidget(zoomWidget);
  addDockWidget(Qt::RightDockWidgetArea, dock);
  viewMenu->addAction(dock->toggleViewAction());
  connect(zoomWidget, SIGNAL(setZoomChanged(float)),
         glWidget, SLOT(setScale(float)));  
}
